@@ -2,22 +2,47 @@
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <assert.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#ifdef HAVE_EVIL
+# include <Evil.h>
+#endif
 
 #include "evas_common.h"
 #include "evas_private.h"
 
+#ifdef EVAS_CSERVE
+// FIXME: cache server and threaded preload clash badly atm - disable
+#undef BUILD_ASYNC_PRELOAD
+#endif
+
 #ifdef BUILD_ASYNC_PRELOAD
 #include <pthread.h>
 
-static Evas_List *preload = NULL;
+typedef struct _Evas_Cache_Preload Evas_Cache_Preload;
+
+struct _Evas_Cache_Preload
+{
+   EINA_INLIST;
+   Image_Entry *ie;
+};
+
+static Eina_Inlist *preload = NULL;
 static Image_Entry *current = NULL;
 
-static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;
+static pthread_cond_t cond_done = PTHREAD_COND_INITIALIZER;
+static pthread_cond_t cond_new = PTHREAD_COND_INITIALIZER;
+static pthread_mutex_t mutex_new = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mutex_surface_alloc = PTHREAD_MUTEX_INITIALIZER;
-static pthread_t tid;
+static pthread_t tid = 0;
 
 static Evas_Bool running = 0;
 
@@ -27,7 +52,7 @@ static void* _evas_cache_background_load(void *);
 #define FREESTRC(Var)              \
   if (Var)                         \
     {                              \
-       evas_stringshare_del(Var);  \
+       eina_stringshare_del(Var);  \
        Var = NULL;                 \
     }
 
@@ -41,11 +66,11 @@ _evas_cache_image_make_dirty(Evas_Cache_Image *cache,
    im->flags.dirty = 1;
    im->flags.activ = 0;
    im->flags.lru_nodata = 0;
-   cache->dirty = evas_object_list_prepend(cache->dirty, im);
+   cache->dirty = eina_inlist_prepend(cache->dirty, EINA_INLIST_GET(im));
 
    if (im->cache_key)
      {
-        evas_stringshare_del(im->cache_key);
+        eina_stringshare_del(im->cache_key);
         im->cache_key = NULL;
      }
 }
@@ -62,7 +87,7 @@ _evas_cache_image_make_activ(Evas_Cache_Image *cache,
         im->flags.activ = 1;
         im->flags.lru_nodata = 0;
         im->flags.dirty = 0;
-        cache->activ = evas_hash_direct_add(cache->activ, key, im);
+	eina_hash_direct_add(cache->activ, key, im);
      }
    else
      {
@@ -80,8 +105,8 @@ _evas_cache_image_make_inactiv(Evas_Cache_Image *cache,
 	im->flags.activ = 0;
 	im->flags.dirty = 0;
 	im->flags.cached = 1;
-	cache->inactiv = evas_hash_direct_add(cache->inactiv, key, im);
-	cache->lru = evas_object_list_prepend(cache->lru, im);
+	eina_hash_direct_add(cache->inactiv, key, im);
+	cache->lru = eina_inlist_prepend(cache->lru, EINA_INLIST_GET(im));
 	cache->usage += cache->func.mem_size_get(im);
      }
    else
@@ -97,7 +122,7 @@ _evas_cache_image_remove_lru_nodata(Evas_Cache_Image *cache,
    if (im->flags.lru_nodata)
      {
         im->flags.lru_nodata = 0;
-        cache->lru_nodata = evas_object_list_remove(cache->lru_nodata, im);
+        cache->lru_nodata = eina_inlist_remove(cache->lru_nodata, EINA_INLIST_GET(im));
         cache->usage -= cache->func.mem_size_get(im);
      }
 }
@@ -108,7 +133,7 @@ _evas_cache_image_activ_lru_nodata(Evas_Cache_Image *cache,
 {
    im->flags.need_data = 0;
    im->flags.lru_nodata = 1;
-   cache->lru_nodata = evas_object_list_prepend(cache->lru_nodata, im);
+   cache->lru_nodata = eina_inlist_prepend(cache->lru_nodata, EINA_INLIST_GET(im));
    cache->usage += cache->func.mem_size_get(im);
 }
 
@@ -120,19 +145,19 @@ _evas_cache_image_remove_activ(Evas_Cache_Image *cache,
      {
         if (ie->flags.activ)
           {
-             cache->activ = evas_hash_del(cache->activ, ie->cache_key, ie);
+	     eina_hash_del(cache->activ, ie->cache_key, ie);
              _evas_cache_image_remove_lru_nodata(cache, ie);
           }
         else
           {
              if (ie->flags.dirty)
                {
-                  cache->dirty = evas_object_list_remove(cache->dirty, ie);
+ 		  cache->dirty = eina_inlist_remove(cache->dirty, EINA_INLIST_GET(ie));
                }
              else
                {
-                  cache->inactiv = evas_hash_del(cache->inactiv, ie->cache_key, ie);
-                  cache->lru = evas_object_list_remove(cache->lru, ie);
+		  eina_hash_del(cache->inactiv, ie->cache_key, ie);
+                  cache->lru = eina_inlist_remove(cache->lru, EINA_INLIST_GET(ie));
                   cache->usage -= cache->func.mem_size_get(ie);
                }
           }
@@ -156,7 +181,7 @@ _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie)
 
    if (ie->cache_key)
      {
-        evas_stringshare_del(ie->cache_key);
+        eina_stringshare_del(ie->cache_key);
         ie->cache_key = NULL;
      }
 
@@ -164,6 +189,11 @@ _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie)
    FREESTRC(ie->key);
 
    cache->func.surface_delete(ie);
+
+#ifdef BUILD_ASYNC_PRELOAD
+   LKD(ie->lock);
+#endif
+
    cache->func.dealloc(ie);
 }
 
@@ -183,7 +213,7 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
    if (!ie)
      return NULL;
 
-   cache_key = hkey ? evas_stringshare_add(hkey) : NULL;
+   cache_key = hkey ? eina_stringshare_add(hkey) : NULL;
 
    ie->flags.loaded = 0;
    ie->flags.need_data = 1;
@@ -199,8 +229,8 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
    ie->references = 0;
    ie->cache = cache;
 
-   ie->file = file ? evas_stringshare_add(file) : NULL;
-   ie->key = key ? evas_stringshare_add(key) : NULL;
+   ie->file = file ? eina_stringshare_add(file) : NULL;
+   ie->key = key ? eina_stringshare_add(key) : NULL;
 
    ie->timestamp = timestamp;
    ie->laststat = time(NULL);
@@ -211,6 +241,11 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
    ie->load_opts.h = 0;
    ie->scale = 1;
 
+#ifdef BUILD_ASYNC_PRELOAD
+   LKI(ie->lock);
+   ie->targets = NULL;
+#endif
+
    if (lo)
      ie->load_opts = *lo;
 
@@ -229,6 +264,27 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
    return ie;
 }
 
+static void
+_evas_cache_image_entry_surface_alloc__locked(Evas_Cache_Image *cache,
+					      Image_Entry *ie,
+					      int wmin,
+					      int hmin)
+{
+   if (ie->allocated.w == wmin && ie->allocated.h == hmin)
+     return ;
+
+   if (cache->func.surface_alloc(ie, wmin, hmin))
+     {
+        wmin = 0;
+        hmin = 0;
+     }
+
+   ie->w = wmin;
+   ie->h = hmin;
+   ie->allocated.w = wmin;
+   ie->allocated.h = hmin;
+}
+
 static void
 _evas_cache_image_entry_surface_alloc(Evas_Cache_Image *cache,
                                       Image_Entry *ie,
@@ -240,62 +296,95 @@ _evas_cache_image_entry_surface_alloc(Evas_Cache_Image *cache,
 
    wmin = w > 0 ? w : 1;
    hmin = h > 0 ? h : 1;
-   if (ie->allocated.w == wmin && ie->allocated.h == hmin)
-     return ;
 
 #ifdef BUILD_ASYNC_PRELOAD
    pthread_mutex_lock(&mutex_surface_alloc);
 #endif
-
-   if (cache->func.surface_alloc(ie, wmin, hmin))
-     {
-        wmin = 0;
-        hmin = 0;
-     }
-
+   _evas_cache_image_entry_surface_alloc__locked(cache, ie, wmin, hmin);
 #ifdef BUILD_ASYNC_PRELOAD
    pthread_mutex_unlock(&mutex_surface_alloc);
 #endif
-
-   ie->w = wmin;
-   ie->h = hmin;
-   ie->allocated.w = wmin;
-   ie->allocated.h = hmin;
 }
 
 #ifdef BUILD_ASYNC_PRELOAD
+static void
+_evas_cache_image_async_call__locked(Image_Entry *im)
+{
+   while (im->targets)
+     {
+	Evas_Cache_Target *tmp = im->targets;
+
+	evas_async_events_put(tmp->target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL,
+			      (void (*)(void*, Evas_Callback_Type, void*))evas_object_event_callback_call);
+	im->targets = (Evas_Cache_Target*) eina_inlist_remove(EINA_INLIST_GET(im->targets), EINA_INLIST_GET(im->targets));
+	free(tmp);
+     }
+}
+
+static void
+_evas_cache_image_async_call(Image_Entry *im)
+{
+   pthread_mutex_lock(&mutex);
+   _evas_cache_image_async_call__locked(im);
+   pthread_mutex_unlock(&mutex);
+}
+
 static int
-_evas_cache_image_entry_preload_add(Evas_Cache_Image *cache,
-				    Image_Entry *ie,
+_evas_cache_image_entry_preload_add(Image_Entry *ie,
 				    const void *target)
 {
+   Evas_Cache_Target *tg;
    int ret = 0;
 
    pthread_mutex_lock(&mutex);
 
-   if (!ie->flags.preload)
+   if (!ie->flags.loaded)
      {
-	preload = evas_list_append(preload, ie);
-	ie->flags.preload = 1;
-	ie->target = target;
+	tg = malloc(sizeof (Evas_Cache_Target));
+	if (!tg) goto end;
 
-	if (!running)
+	tg->target = target;
+	ie->targets = (Evas_Cache_Target*) eina_inlist_append(EINA_INLIST_GET(ie->targets), EINA_INLIST_GET(tg));
+
+	if (!ie->flags.preload)
 	  {
-	     if (pthread_create(&tid, NULL, _evas_cache_background_load, NULL) == 0)
-	       running = 1;
-	  }
+	     Evas_Cache_Preload *tmp;
 
+	     tmp = malloc(sizeof (Evas_Cache_Preload));
+	     if (!tmp) goto end;
+
+	     tmp->ie = ie;
+	     preload = eina_inlist_append(preload, EINA_INLIST_GET(tmp));
+
+	     ie->flags.preload = 1;
+
+	     if (!running)
+	       {
+		  if (tid)
+		    {
+		       running = 1;
+		       pthread_cond_signal(&cond_new);
+		    }
+		  else
+		    {
+		       if (pthread_create(&tid, NULL, _evas_cache_background_load, NULL) == 0)
+			 running = 1;
+		    }
+	       }
+
+	     ret = 2;
+	  }
 	ret = 1;
      }
 
+ end:
    pthread_mutex_unlock(&mutex);
 
    return ret;
 }
 
 static int
-_evas_cache_image_entry_preload_remove(Evas_Cache_Image *cache,
-				       Image_Entry *ie)
+_evas_cache_image_entry_preload_remove(Image_Entry *ie, const void *target)
 {
    int ret = 0;
 
@@ -308,11 +397,47 @@ _evas_cache_image_entry_preload_remove(Evas_Cache_Image *cache,
 	     if (current == ie)
 	       {
 		  /* Wait until ie is processed. */
-		  pthread_cond_wait(&cond, &mutex);
+		  pthread_cond_wait(&cond_done, &mutex);
 	       }
 	     else
 	       {
-		  preload = evas_list_remove(preload, ie);
+		  Evas_Cache_Preload *l;
+		  EINA_INLIST_FOREACH(preload, l)
+		    {
+		       if (l->ie == ie)
+			 {
+			    Evas_Cache_Target *tg;
+
+			    if (target) {
+			       EINA_INLIST_FOREACH(ie->targets, tg)
+				 {
+				    if (tg->target == target) {
+				       ie->targets = (Evas_Cache_Target*) eina_inlist_remove(EINA_INLIST_GET(ie->targets), EINA_INLIST_GET(tg));
+				       free(tg);
+				       break;
+				    }
+				 }
+			    } else {
+			       _evas_cache_image_async_call__locked(ie);
+
+			       while (ie->targets)
+				 {
+				    tg = ie->targets;
+				    ie->targets = (Evas_Cache_Target*) eina_inlist_remove(EINA_INLIST_GET(ie->targets), EINA_INLIST_GET(tg));
+				    free(tg);
+				 }
+			    }
+
+			    if (!ie->targets)
+			      {
+				 preload = eina_inlist_remove(preload,
+							      EINA_INLIST_GET(l));
+				 free(l);
+			      }
+
+			    break;
+			 }
+		    }
 		  ie->flags.preload = 0;
 		  ret = 1;
 	       }
@@ -367,8 +492,8 @@ evas_cache_image_init(const Evas_Cache_Image_Func *cb)
    new->dirty = NULL;
    new->lru = NULL;
    new->lru_nodata = NULL;
-   new->inactiv = NULL;
-   new->activ = NULL;
+   new->inactiv = eina_hash_string_superfast_new(NULL);
+   new->activ = eina_hash_string_superfast_new(NULL);
 
    new->references = 1;
 
@@ -376,20 +501,35 @@ evas_cache_image_init(const Evas_Cache_Image_Func *cb)
 }
 
 static Evas_Bool
-_evas_cache_image_free_cb(const Evas_Hash *hash, const char *key, void *data, void *fdata)
+_evas_cache_image_free_cb(__UNUSED__ const Eina_Hash *hash, __UNUSED__ const void *key, void *data, void *fdata)
 {
-   Evas_Cache_Image    *cache = fdata;
-   Image_Entry         *im = data;
+   Eina_List **delete_list = fdata;
 
-   _evas_cache_image_entry_delete(cache, im);
+   *delete_list = eina_list_prepend(*delete_list, data);
 
    return 1;
 }
 
+#ifdef BUILD_ASYNC_PRELOAD
+static void
+_evas_cache_image_entry_clear_preloaders(Image_Entry *ie)
+{
+   while (ie->targets)
+     {
+	Evas_Cache_Target *t = ie->targets;
+	ie->targets = (Evas_Cache_Target *)
+	  eina_inlist_remove(EINA_INLIST_GET(ie->targets),
+			     EINA_INLIST_GET(ie->targets));
+	free(t);
+     }
+}
+#endif
+
 EAPI void
 evas_cache_image_shutdown(Evas_Cache_Image *cache)
 {
    Image_Entry  *im;
+   Eina_List *delete_list;
 
    assert(cache != NULL);
    cache->references--;
@@ -397,6 +537,29 @@ evas_cache_image_shutdown(Evas_Cache_Image *cache)
    if (cache->references > 0)
      return ;
 
+#ifdef BUILD_ASYNC_PRELOAD
+   pthread_mutex_lock(&mutex);
+   if (running)
+     {
+	Eina_Inlist *l, *l_next;
+	for (l = preload; l != NULL; l = l_next)
+	  {
+	     Evas_Cache_Preload *tmp = (Evas_Cache_Preload *)l;
+	     Image_Entry *ie = tmp->ie;
+
+	     l_next = l->next;
+
+	     if (ie->cache != cache)
+	       continue;
+
+	     preload = eina_inlist_remove(preload, l);
+	     _evas_cache_image_entry_clear_preloaders(ie);
+	     free(l);
+	  }
+     }
+   pthread_mutex_unlock(&mutex);
+#endif
+
    while (cache->lru)
      {
         im = (Image_Entry *) cache->lru;
@@ -416,9 +579,17 @@ evas_cache_image_shutdown(Evas_Cache_Image *cache)
         _evas_cache_image_entry_delete(cache, im);
      }
 
-   evas_hash_foreach(cache->activ, _evas_cache_image_free_cb, cache);
-   evas_hash_free(cache->activ);
-   evas_hash_free(cache->inactiv);
+   delete_list = NULL;
+   eina_hash_foreach(cache->activ, _evas_cache_image_free_cb, &delete_list);
+
+   while (delete_list)
+     {
+	_evas_cache_image_entry_delete(cache, eina_list_data_get(delete_list));
+	delete_list = eina_list_remove_list(delete_list, delete_list);
+     }
+
+   eina_hash_free(cache->activ);
+   eina_hash_free(cache->inactiv);
 
    free(cache);
 }
@@ -428,18 +599,38 @@ evas_cache_image_shutdown(Evas_Cache_Image *cache)
 EAPI Image_Entry *
 evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *key, RGBA_Image_Loadopts *lo, int *error)
 {
+   const char           *ckey = "(null)";
    const char           *format;
    char                 *hkey;
    Image_Entry          *im;
    Evas_Image_Load_Opts  prevent = { 0, 0, 0, 0 };
    int                   size;
    int                   stat_done = 0;
+   int                   file_length;
+   int                   key_length;
    struct stat           st;
 
    assert(cache != NULL);
 
    if (!file && !key) return NULL;
    if (!file) return NULL;
+
+   file_length = strlen(file);
+   key_length = key ? strlen(key) : 6;
+
+   size = file_length + key_length + 64;
+   hkey = alloca(sizeof (char) * size);
+
+   memcpy(hkey, file, file_length);
+   size = file_length;
+
+   memcpy(hkey + size, "//://", 5);
+   size += 5;
+
+   if (key) ckey = key;
+   memcpy(hkey + size, ckey, key_length);
+   size += key_length;
+
    if ((!lo) ||
        (lo &&
         (lo->scale_down_by == 0) &&
@@ -454,16 +645,30 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
      }
    else
      {
-        if (key)
-          format = "%s//://%s//@/%i/%1.5f/%ix%i";
-        else
-          format = "%s//://%p//@/%i/%1.5f/%ix%i";
-     }
-   size = strlen(file) + (key ? strlen(key) : 6) + 64;
-   hkey = alloca(sizeof (char) * size);
-   snprintf(hkey, size, format, file, key, lo->scale_down_by, lo->dpi, lo->w, lo->h);
+	memcpy(hkey + size, "//@/", 4);
+	size += 4;
 
-   im = evas_hash_find(cache->activ, hkey);
+	size += eina_convert_xtoa(lo->scale_down_by, hkey + size);
+
+	hkey[size] = '/';
+	size += 1;
+
+	size += eina_convert_dtoa(lo->dpi, hkey + size);
+
+	hkey[size] = '/';
+	size += 1;
+
+	size += eina_convert_xtoa(lo->w, hkey + size);
+
+	hkey[size] = 'x';
+	size += 1;
+
+	size += eina_convert_xtoa(lo->h, hkey + size);
+     }
+
+   hkey[size] = '\0';
+
+   im = eina_hash_find(cache->activ, hkey);
    if (im)
      {
         time_t  t;
@@ -471,7 +676,6 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 
         ok = 1;
         t = time(NULL);
-
         if ((t - im->laststat) > STAT_GAP)
           {
              stat_done = 1;
@@ -486,7 +690,7 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 	_evas_cache_image_make_dirty(cache, im);
      }
 
-   im = evas_hash_find(cache->inactiv, hkey);
+   im = eina_hash_find(cache->inactiv, hkey);
    if (im)
      {
         int     ok;
@@ -525,8 +729,7 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
      }
 
    im = _evas_cache_image_entry_new(cache, hkey, st.st_mtime, file, key, lo, error);
-   if (!im)
-     return NULL;
+   if (!im) return NULL;
 
    if (cache->func.debug)
      cache->func.debug("request", im);
@@ -558,7 +761,7 @@ evas_cache_image_drop(Image_Entry *im)
    if (im->references == 0)
      {
 #ifdef BUILD_ASYNC_PRELOAD
-	_evas_cache_image_entry_preload_remove(cache, im);
+	_evas_cache_image_entry_preload_remove(im, NULL);
 #endif
 
 	if (im->flags.dirty)
@@ -601,11 +804,26 @@ evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
    cache = im->cache;
    if (!(im->flags.dirty))
      {
+#ifndef EVAS_CSERVE
+// if ref 1 also copy if using shared cache as its read-only
         if (im->references == 1) im_dirty = im;
         else
+#endif
           {
              int        error;
 
+             im_dirty = evas_cache_image_copied_data
+               (cache, im->w, im->h, 
+                evas_cache_image_pixels(im), 
+                im->flags.alpha, 
+                im->space);
+             if (!im_dirty) goto on_error;
+             if (cache->func.debug)
+               cache->func.debug("dirty-src", im);
+             error = cache->func.dirty(im_dirty, im);
+             if (cache->func.debug)
+               cache->func.debug("dirty-out", im_dirty);
+/*             
              im_dirty = _evas_cache_image_entry_new(cache, NULL, im->timestamp, im->file, im->key, &im->load_opts, &error);
              if (!im_dirty) goto on_error;
 
@@ -616,7 +834,7 @@ evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
                cache->func.debug("dirty-out", im_dirty);
 
              if (error != 0) goto on_error;
-
+ */
              im_dirty->references = 1;
 
              evas_cache_image_drop(im);
@@ -661,6 +879,18 @@ evas_cache_image_alone(Image_Entry *im)
      {
         int     error;
 
+        im_dirty = evas_cache_image_copied_data
+          (cache, im->w, im->h, 
+           evas_cache_image_pixels(im), 
+           im->flags.alpha, 
+           im->space);
+        if (!im_dirty) goto on_error;
+        if (cache->func.debug)
+          cache->func.debug("dirty-src", im);
+        error = cache->func.dirty(im_dirty, im);
+        if (cache->func.debug)
+          cache->func.debug("dirty-out", im_dirty);
+/*        
         im_dirty = _evas_cache_image_entry_new(cache, NULL, im->timestamp, im->file, im->key, &im->load_opts, &error);
         if (!im_dirty) goto on_error;
 
@@ -671,7 +901,7 @@ evas_cache_image_alone(Image_Entry *im)
           cache->func.debug("dirty-out", im_dirty);
 
         if (error != 0) goto on_error;
-
+ */
         im_dirty->references = 1;
 
         evas_cache_image_drop(im);
@@ -819,22 +1049,32 @@ evas_cache_image_load_data(Image_Entry *im)
 
    assert(im);
    assert(im->cache);
+   cache = im->cache;
 
    if (im->flags.loaded) return ;
 
 #ifdef BUILD_ASYNC_PRELOAD
+   int preload = im->flags.preload;
    /* We check a first time, to prevent useless lock. */
-   _evas_cache_image_entry_preload_remove(cache, im);
+   _evas_cache_image_entry_preload_remove(im, NULL);
    if (im->flags.loaded) return ;
+   LKL(im->lock);
 #endif
 
-   cache = im->cache;
-
    error = cache->func.load(im);
 
+#ifdef BUILD_ASYNC_PRELOAD
+   LKU(im->lock);
+#endif
+
    if (cache->func.debug)
      cache->func.debug("load", im);
 
+#ifdef BUILD_ASYNC_PRELOAD
+   if (preload)
+     _evas_cache_image_async_call(im);
+#endif
+
    if (error)
      {
         _evas_cache_image_entry_surface_alloc(cache, im, im->w, im->h);
@@ -855,28 +1095,37 @@ evas_cache_image_preload_data(Image_Entry *im, const void *target)
    assert(im);
    assert(im->cache);
 
-   if (im->flags.loaded) return ;
+   if (im->flags.loaded)
+     {
+	evas_async_events_put(target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL,
+			      (void (*)(void*, Evas_Callback_Type, void*))evas_object_event_callback_call);
+	return ;
+     }
 
    cache = im->cache;
 
-   _evas_cache_image_entry_preload_add(cache, im, target);
+   if (!_evas_cache_image_entry_preload_add(im, target))
+     evas_async_events_put(target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL,
+			   (void (*)(void*, Evas_Callback_Type, void*))evas_object_event_callback_call);
 #else
    evas_cache_image_load_data(im);
 
-   evas_async_events_put(target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL, evas_object_event_callback_call);
+   evas_async_events_put(target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL,
+			 (void (*)(void*, Evas_Callback_Type, void*))evas_object_event_callback_call);
 #endif
 }
 
 EAPI void
-evas_cache_image_preload_cancel(Image_Entry *im)
+evas_cache_image_preload_cancel(Image_Entry *im, const void *target)
 {
 #ifdef BUILD_ASYNC_PRELOAD
    Evas_Cache_Image    *cache;
 
    assert(im);
    assert(im->cache);
+   cache = im->cache;
 
-   _evas_cache_image_entry_preload_remove(cache, im);
+   _evas_cache_image_entry_preload_remove(im, target);
 #else
    (void) im;
 #endif
@@ -968,7 +1217,7 @@ evas_cache_private_set(Evas_Cache_Image *cache, const void *data)
 {
    assert(cache);
 
-   cache->data = data;
+   cache->data = (void *)data;
 }
 
 EAPI DATA32 *
@@ -995,8 +1244,15 @@ _evas_cache_background_load(void *data)
      {
 	pthread_mutex_lock(&mutex);
 
-	current = evas_list_data(preload);
-	preload = evas_list_remove(preload, current);
+	if (preload)
+	  {
+	     Evas_Cache_Preload *tmp = (Evas_Cache_Preload*) preload;
+
+	     current = tmp->ie;
+	     preload = eina_inlist_remove(preload, preload);
+
+	     free(tmp);
+	  }
 
 	pthread_mutex_unlock(&mutex);
 
@@ -1005,6 +1261,7 @@ _evas_cache_background_load(void *data)
 	     Evas_Cache_Image *cache;
 	     int error;
 
+	     LKL(current->lock);
 	     cache = current->cache;
 
 	     error = cache->func.load(current);
@@ -1023,12 +1280,14 @@ _evas_cache_background_load(void *data)
 	       }
 
 	     current->flags.preload = 0;
+
+	     LKU(current->lock);
+
+	     _evas_cache_image_async_call(current);
+	     current = NULL;
 	  }
 
-	evas_async_events_put(current->target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL, evas_object_event_callback_call);
-
-	current = NULL;
-	pthread_cond_signal(&cond);
+	pthread_cond_signal(&cond_done);
      }
 
    pthread_mutex_lock(&mutex);
@@ -1041,6 +1300,11 @@ _evas_cache_background_load(void *data)
    running = 0;
    pthread_mutex_unlock(&mutex);
 
+   pthread_mutex_lock(&mutex_new);
+   pthread_cond_wait(&cond_new, &mutex_new);
+   pthread_mutex_unlock(&mutex_new);
+   goto restart;
+
    return NULL;
 }
 #endif
