@@ -1,17 +1,14 @@
 #include "evas_common.h"
 #include "evas_private.h"
 
-static Evas_List *
-_evas_event_object_list_in_get(Evas *e, Evas_List *in, const Evas_Object_List *list, Evas_Object *stop, int x, int y, int *no_rep)
+static Eina_List *
+_evas_event_object_list_in_get(Evas *e, Eina_List *in, const Eina_Inlist *list, Evas_Object *stop, int x, int y, int *no_rep)
 {
-   const Evas_Object_List *l;
+   Evas_Object *obj;
 
    if (!list) return in;
-   for (l = list->last; l; l = l->prev)
+   EINA_INLIST_REVERSE_FOREACH(list, obj)
      {
-	Evas_Object *obj;
-
-	obj = (Evas_Object *)l;
 	if (obj == stop)
 	  {
 	     *no_rep = 1;
@@ -43,7 +40,7 @@ _evas_event_object_list_in_get(Evas *e, Evas_List *in, const Evas_Object_List *l
 		           ((!obj->precise_is_inside) ||
 			    (evas_object_is_inside(obj, x, y))))
 			 {
-			    in = evas_list_append(in, obj);
+			    in = eina_list_append(in, obj);
 			    if (!obj->repeat_events)
 			      {
 				 *no_rep = 1;
@@ -58,35 +55,34 @@ _evas_event_object_list_in_get(Evas *e, Evas_List *in, const Evas_Object_List *l
    return in;
 }
 
-Evas_List *
+Eina_List *
 evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y)
 {
-   Evas_Object_List *l;
-   Evas_List *in = NULL;
+   Evas_Layer *lay;
+   Eina_List *in = NULL;
 
    if (!e->layers) return NULL;
-   for (l = ((Evas_Object_List *)(e->layers))->last; l; l = l->prev)
+   EINA_INLIST_REVERSE_FOREACH((EINA_INLIST_GET(e->layers)), lay)
      {
-	Evas_Layer *lay;
 	int norep;
 
-	lay = (Evas_Layer *)l;
 	norep = 0;
-	in = _evas_event_object_list_in_get(e, in, (Evas_Object_List *)lay->objects, stop,
+	in = _evas_event_object_list_in_get(e, in, EINA_INLIST_GET(lay->objects), stop,
 					    x, y, &norep);
 	if (norep) return in;
      }
    return in;
 }
 
-static Evas_List *evas_event_list_copy(Evas_List *list);
-static Evas_List *
-evas_event_list_copy(Evas_List *list)
+static Eina_List *evas_event_list_copy(Eina_List *list);
+static Eina_List *
+evas_event_list_copy(Eina_List *list)
 {
-   Evas_List *l, *new_l = NULL;
+   Eina_List *l, *new_l = NULL;
+   const void *data;
 
-   for (l = list; l; l = l->next)
-     new_l = evas_list_append(new_l, l->data);
+   EINA_LIST_FOREACH(list, l, data)
+     new_l = eina_list_append(new_l, data);
    return new_l;
 }
 /* public functions */
@@ -148,19 +144,14 @@ evas_event_thaw(Evas *e)
    e->events_frozen--;
    if (e->events_frozen == 0)
      {
-	Evas_Object_List *l;
+	Evas_Layer *lay;
 
-	for (l = (Evas_Object_List *)e->layers; l; l = l->next)
+	EINA_INLIST_FOREACH((EINA_INLIST_GET(e->layers)), lay)
 	  {
-	     Evas_Object_List *l2;
-	     Evas_Layer *lay;
+	     Evas_Object *obj;
 
-	     lay = (Evas_Layer *)l;
-	     for (l2 = (Evas_Object_List *)lay->objects; l2; l2 = l2->next)
+	     EINA_INLIST_FOREACH(lay->objects, obj)
 	       {
-		  Evas_Object *obj;
-
-		  obj = (Evas_Object *)l2;
 		  evas_object_clip_recalc(obj);
 		  evas_object_recalc_clippees(obj);
 	       }
@@ -206,8 +197,9 @@ evas_event_freeze_get(const Evas *e)
 EAPI void
 evas_event_feed_mouse_down(Evas *e, int b, Evas_Button_Flags flags, unsigned int timestamp, const void *data)
 {
-   Evas_List *l, *copy;
+   Eina_List *l, *copy;
    Evas_Event_Mouse_Down ev;
+   Evas_Object *obj;
 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
@@ -234,11 +226,8 @@ evas_event_feed_mouse_down(Evas *e, int b, Evas_Button_Flags flags, unsigned int
 
    _evas_walk(e);
    copy = evas_event_list_copy(e->pointer.object.in);
-   for (l = copy; l; l = l->next)
+   EINA_LIST_FOREACH(copy, l, obj)
      {
-	Evas_Object *obj;
-
-	obj = l->data;
 	if (obj->pointer_mode != EVAS_OBJECT_POINTER_MODE_NOGRAB)
 	  {
 	     obj->mouse_grabbed++;
@@ -249,7 +238,7 @@ evas_event_feed_mouse_down(Evas *e, int b, Evas_Button_Flags flags, unsigned int
 	  evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_DOWN, &ev);
 	if (e->delete_me) break;
      }
-   if (copy) copy = evas_list_free(copy);
+   if (copy) eina_list_free(copy);
    e->last_mouse_down_counter++;
    _evas_unwalk(e);
 }
@@ -263,7 +252,7 @@ evas_event_feed_mouse_down(Evas *e, int b, Evas_Button_Flags flags, unsigned int
 EAPI void
 evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int timestamp, const void *data)
 {
-   Evas_List *l, *copy;
+   Eina_List *l, *copy;
 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
@@ -278,6 +267,7 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 
      {
 	Evas_Event_Mouse_Up ev;
+	Evas_Object *obj;
 
 	ev.button = b;
 	ev.output.x = e->pointer.x;
@@ -293,11 +283,8 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 
 	_evas_walk(e);
 	copy = evas_event_list_copy(e->pointer.object.in);
-	for (l = copy; l; l = l->next)
+	EINA_LIST_FOREACH(copy, l, obj)
 	  {
-	     Evas_Object *obj;
-
-	     obj = l->data;
 	     if ((obj->pointer_mode != EVAS_OBJECT_POINTER_MODE_NOGRAB) &&
 		 (obj->mouse_in) && (obj->mouse_grabbed > 0))
 	       {
@@ -308,17 +295,17 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 	       evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_UP, &ev);
 	     if (e->delete_me) break;
 	  }
-	if (copy) copy = evas_list_free(copy);
+	if (copy) copy = eina_list_free(copy);
 	e->last_mouse_up_counter++;
      }
 
    if (!e->pointer.button)
      {
-	Evas_List *ins;
-	Evas_List *l;
-
+	Eina_List *ins;
+	Eina_List *l;
 	  {
 	     Evas_Event_Mouse_Out ev;
+	     Evas_Object *obj;
 
 	     ev.buttons = e->pointer.button;
 	     ev.output.x = e->pointer.x;
@@ -335,12 +322,9 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 	     ins = evas_event_objects_event_list(e, NULL, e->pointer.x, e->pointer.y);
 	     /* go thru old list of in objects */
 	     copy = evas_event_list_copy(e->pointer.object.in);
-	     for (l = copy; l; l = l->next)
+	     EINA_LIST_FOREACH(copy, l, obj)
 	       {
-		  Evas_Object *obj;
-
-		  obj = l->data;
-		  if ((!evas_list_find(ins, obj)) ||
+		  if ((!eina_list_data_find(ins, obj)) ||
 		      (!e->pointer.inside))
 		    {
 
@@ -351,10 +335,11 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 		  if (e->delete_me) break;
 	       }
 	  }
-	if (copy) copy = evas_list_free(copy);
+	if (copy) copy = eina_list_free(copy);
 	if (e->pointer.inside)
 	  {
 	     Evas_Event_Mouse_In ev;
+	     Evas_Object *obj;
 
 	     ev.buttons = e->pointer.button;
 	     ev.output.x = e->pointer.x;
@@ -367,13 +352,9 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 	     ev.timestamp = timestamp;
 	     ev.event_flags = EVAS_EVENT_FLAG_NONE;
 
-	     for (l = ins; l; l = l->next)
+	     EINA_LIST_FOREACH(ins, l, obj)
 	       {
-		  Evas_Object *obj;
-
-		  obj = l->data;
-
-		  if (!evas_list_find(e->pointer.object.in, obj))
+		  if (!eina_list_data_find(e->pointer.object.in, obj))
 		    {
 
 		       obj->mouse_in = 1;
@@ -385,10 +366,10 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
 	  }
 	else
 	  {
-	     ins = evas_list_free(ins);
+	     ins = eina_list_free(ins);
 	  }
 	/* free our old list of ins */
-	e->pointer.object.in = evas_list_free(e->pointer.object.in);
+	e->pointer.object.in = eina_list_free(e->pointer.object.in);
 	/* and set up the new one */
 	e->pointer.object.in = ins;
 	if (e->pointer.inside)
@@ -396,11 +377,14 @@ evas_event_feed_mouse_up(Evas *e, int b, Evas_Button_Flags flags, unsigned int t
      }
 
    if (e->pointer.mouse_grabbed < 0)
-     fprintf(stderr, "BUG? e->pointer.mouse_grabbed (=%d) < 0!\n",
-	     e->pointer.mouse_grabbed);
-
-   if ((e->pointer.button == 0) && (e->pointer.mouse_grabbed))
      {
+        fprintf(stderr, "BUG? e->pointer.mouse_grabbed (=%d) < 0!\n",
+                e->pointer.mouse_grabbed);
+     }
+
+   if ((e->pointer.button == 0) && (e->pointer.mouse_grabbed != 0))
+     {
+        printf("restore to 0 grabs (from %i)\n", e->pointer.mouse_grabbed);
 	e->pointer.mouse_grabbed = 0;
      }
    _evas_unwalk(e);
@@ -441,8 +425,9 @@ evas_event_feed_mouse_cancel(Evas *e, unsigned int timestamp, const void *data)
 EAPI void
 evas_event_feed_mouse_wheel(Evas *e, int direction, int z, unsigned int timestamp, const void *data)
 {
-   Evas_List *l, *copy;
+   Eina_List *l, *copy;
    Evas_Event_Mouse_Wheel ev;
+   Evas_Object *obj;
 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
@@ -466,15 +451,13 @@ evas_event_feed_mouse_wheel(Evas *e, int direction, int z, unsigned int timestam
    _evas_walk(e);
    copy = evas_event_list_copy(e->pointer.object.in);
 
-   for (l = copy; l; l = l->next)
+   EINA_LIST_FOREACH(copy, l, obj)
      {
-	Evas_Object *obj = l->data;
-
 	if (e->events_frozen <= 0)
 	  evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_WHEEL, &ev);
 	if (e->delete_me) break;
      }
-   if (copy) copy = evas_list_free(copy);
+   if (copy) copy = eina_list_free(copy);
 
    _evas_unwalk(e);
 }
@@ -515,11 +498,12 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
    if (e->pointer.mouse_grabbed > 0)
      {
 	/* go thru old list of in objects */
-	Evas_List *outs = NULL;
-	Evas_List *l, *copy;
+	Eina_List *outs = NULL;
+	Eina_List *l, *copy;
 
 	  {
 	     Evas_Event_Mouse_Move ev;
+	     Evas_Object *obj;
 
 	     ev.buttons = e->pointer.button;
 	     ev.cur.output.x = e->pointer.x;
@@ -536,11 +520,8 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	     ev.timestamp = timestamp;
 	     ev.event_flags = EVAS_EVENT_FLAG_NONE;
 	     copy = evas_event_list_copy(e->pointer.object.in);
-	     for (l = copy; l; l = l->next)
+	     EINA_LIST_FOREACH(copy, l, obj)
 	       {
-		  Evas_Object *obj;
-
-		  obj = l->data;
 		  if ((obj->cur.visible) &&
 		      (evas_object_clippers_is_visible(obj)) &&
 		      (!evas_event_passes_through(obj)) &&
@@ -553,7 +534,7 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 			 }
 		    }
 		  else
-		    outs = evas_list_append(outs, obj);
+		    outs = eina_list_append(outs, obj);
 		  if (e->delete_me) break;
 	       }
 	  }
@@ -571,16 +552,16 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	     ev.timestamp = timestamp;
 	     ev.event_flags = EVAS_EVENT_FLAG_NONE;
 
-	     if (copy) copy = evas_list_free(copy);
+	     if (copy) copy = eina_list_free(copy);
 	     while (outs)
 	       {
 		  Evas_Object *obj;
 
 		  obj = outs->data;
-		  outs = evas_list_remove(outs, obj);
-		  if ((!obj->mouse_grabbed) && (!e->delete_me))
+		  outs = eina_list_remove(outs, obj);
+		  if ((obj->mouse_grabbed == 0) && (!e->delete_me))
 		    {
-		       e->pointer.object.in = evas_list_remove(e->pointer.object.in, obj);
+		       e->pointer.object.in = eina_list_remove(e->pointer.object.in, obj);
 			 {
 			    obj->mouse_in = 0;
 			    if (e->events_frozen <= 0)
@@ -592,11 +573,12 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
      }
    else
      {
-	Evas_List *ins;
-	Evas_List *l, *copy;
+	Eina_List *ins;
+	Eina_List *l, *copy;
 	Evas_Event_Mouse_Move ev;
 	Evas_Event_Mouse_Out ev2;
 	Evas_Event_Mouse_In ev3;
+	Evas_Object *obj;
 
 	ev.buttons = e->pointer.button;
 	ev.cur.output.x = e->pointer.x;
@@ -639,11 +621,8 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	ins = evas_event_objects_event_list(e, NULL, x, y);
 	/* go thru old list of in objects */
 	copy = evas_event_list_copy(e->pointer.object.in);
-	for (l = copy; l; l = l->next)
+	EINA_LIST_FOREACH(copy, l, obj)
 	  {
-	     Evas_Object *obj;
-
-	     obj = l->data;
 	     /* if its under the pointer and its visible and its in the new */
 	     /* in list */
 // FIXME: i don't think we need this
@@ -651,7 +630,7 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	     if (evas_object_is_in_output_rect(obj, x, y, 1, 1) &&
 		 (obj->cur.visible) &&
 		 (evas_object_clippers_is_visible(obj)) &&
-		 (evas_list_find(ins, obj)) &&
+		 (eina_list_data_find(ins, obj)) &&
 		 (!evas_event_passes_through(obj)) &&
 		 (!obj->clip.clipees) &&
 		 ((!obj->precise_is_inside) ||
@@ -672,15 +651,12 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	       }
 	     if (e->delete_me) break;
 	  }
-	if (copy) copy = evas_list_free(copy);
+	if (copy) copy = eina_list_free(copy);
 	/* go thru our current list of ins */
-	for (l = ins; l; l = l->next)
+	EINA_LIST_FOREACH(ins, l, obj)
 	  {
-	     Evas_Object *obj;
-
-	     obj = l->data;
 	     /* if its not in the old list of ins send an enter event */
-	     if (!evas_list_find(e->pointer.object.in, obj))
+	     if (!eina_list_data_find(e->pointer.object.in, obj))
 	       {
 		  obj->mouse_in = 1;
 
@@ -690,7 +666,7 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 	     if (e->delete_me) break;
 	  }
 	/* free our old list of ins */
-	evas_list_free(e->pointer.object.in);
+	eina_list_free(e->pointer.object.in);
 	/* and set up the new one */
 	e->pointer.object.in = ins;
      }
@@ -706,9 +682,10 @@ evas_event_feed_mouse_move(Evas *e, int x, int y, unsigned int timestamp, const
 EAPI void
 evas_event_feed_mouse_in(Evas *e, unsigned int timestamp, const void *data)
 {
-   Evas_List *ins;
-   Evas_List *l;
+   Eina_List *ins;
+   Eina_List *l;
    Evas_Event_Mouse_In ev;
+   Evas_Object *obj;
 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
@@ -734,13 +711,9 @@ evas_event_feed_mouse_in(Evas *e, unsigned int timestamp, const void *data)
    _evas_walk(e);
    /* get new list of ins */
    ins = evas_event_objects_event_list(e, NULL, e->pointer.x, e->pointer.y);
-   for (l = ins; l; l = l->next)
+   EINA_LIST_FOREACH(ins, l, obj)
      {
-	Evas_Object *obj;
-
-	obj = l->data;
-
-	if (!evas_list_find(e->pointer.object.in, obj))
+	if (!eina_list_data_find(e->pointer.object.in, obj))
 	  {
 
 	     obj->mouse_in = 1;
@@ -750,7 +723,7 @@ evas_event_feed_mouse_in(Evas *e, unsigned int timestamp, const void *data)
 	if (e->delete_me) break;
      }
    /* free our old list of ins */
-   e->pointer.object.in = evas_list_free(e->pointer.object.in);
+   e->pointer.object.in = eina_list_free(e->pointer.object.in);
    /* and set up the new one */
    e->pointer.object.in = ins;
    evas_event_feed_mouse_move(e, e->pointer.x, e->pointer.y, timestamp, data);
@@ -792,25 +765,21 @@ evas_event_feed_mouse_out(Evas *e, unsigned int timestamp, const void *data)
    if (e->pointer.mouse_grabbed == 0)
      {
 	/* go thru old list of in objects */
-	Evas_List *l, *copy;
+	Eina_List *l, *copy;
+	Evas_Object *obj;
 
 	copy = evas_event_list_copy(e->pointer.object.in);
-	for (l = copy; l; l = l->next)
+	EINA_LIST_FOREACH(copy, l, obj)
 	  {
-	     Evas_Object *obj;
+	    obj->mouse_in = 0;
+	    if (e->events_frozen <= 0)
+	      evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
 
-	     obj = l->data;
-	       {
-
-		  obj->mouse_in = 0;
-		  if (e->events_frozen <= 0)
-		    evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
-	       }
-	     if (e->delete_me) break;
+	    if (e->delete_me) break;
 	  }
-	if (copy) copy = evas_list_free(copy);
+	if (copy) copy = eina_list_free(copy);
 	/* free our old list of ins */
-	e->pointer.object.in =  evas_list_free(e->pointer.object.in);
+	e->pointer.object.in =  eina_list_free(e->pointer.object.in);
      }
    _evas_unwalk(e);
 }
@@ -847,14 +816,12 @@ evas_event_feed_key_down(Evas *e, const char *keyname, const char *key, const ch
 	ev.event_flags = EVAS_EVENT_FLAG_NONE;
 	if (e->grabs)
 	  {
-	     Evas_List *l;
+	     Eina_List *l;
+	     Evas_Key_Grab *g;
 
 	     e->walking_grabs++;
-	     for (l = e->grabs; l; l= l->next)
+	     EINA_LIST_FOREACH(e->grabs, l, g)
 	       {
-		  Evas_Key_Grab *g;
-
-		  g = l->data;
 		  if (g->just_added)
 		    {
 		       g->just_added = 0;
@@ -879,15 +846,11 @@ evas_event_feed_key_down(Evas *e, const char *keyname, const char *key, const ch
 	       {
 		  while (e->delete_grabs > 0)
 		    {
-		       Evas_List *l;
-
 		       e->delete_grabs--;
 		       for (l = e->grabs; l;)
 			 {
-			    Evas_Key_Grab *g;
-
-			    g = l->data;
-			    l = l->next;
+			    g = eina_list_data_get(l);
+			    l = eina_list_next(l);
 			    if (g->delete_me)
 			      evas_key_grab_free(g->object, g->keyname, g->modifiers, g->not_modifiers);
 			 }
@@ -935,14 +898,12 @@ evas_event_feed_key_up(Evas *e, const char *keyname, const char *key, const char
 	ev.event_flags = EVAS_EVENT_FLAG_NONE;
 	if (e->grabs)
 	  {
-	     Evas_List *l;
+	     Eina_List *l;
+	     Evas_Key_Grab *g;
 
 	     e->walking_grabs++;
-	     for (l = e->grabs; l; l= l->next)
+	     EINA_LIST_FOREACH(e->grabs, l, g)
 	       {
-		  Evas_Key_Grab *g;
-
-		  g = l->data;
 		  if (g->just_added)
 		    {
 		       g->just_added = 0;
@@ -969,12 +930,12 @@ evas_event_feed_key_up(Evas *e, const char *keyname, const char *key, const char
 		       Evas_List *l;
 
 		       e->delete_grabs--;
-		       for (l = e->grabs; l; l= l->next)
+		       for (l = e->grabs; l;)
 			 {
 			    Evas_Key_Grab *g;
 
-			    g = l->data;
-			    l = l->next;
+			    g = eina_list_data_get(l);
+			    l = eina_list_next(l);
 			    if (g->delete_me)
 			      evas_key_grab_free(g->object, g->keyname, g->modifiers, g->not_modifiers);
 			 }
@@ -999,8 +960,9 @@ evas_event_feed_key_up(Evas *e, const char *keyname, const char *key, const char
 EAPI void
 evas_event_feed_hold(Evas *e, int hold, unsigned int timestamp, const void *data)
 {
-   Evas_List *l, *copy;
+   Eina_List *l, *copy;
    Evas_Event_Hold ev;
+   Evas_Object *obj;
 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
@@ -1016,23 +978,20 @@ evas_event_feed_hold(Evas *e, int hold, unsigned int timestamp, const void *data
 
    _evas_walk(e);
    copy = evas_event_list_copy(e->pointer.object.in);
-   for (l = copy; l; l = l->next)
+   EINA_LIST_FOREACH(copy, l, obj)
      {
-	Evas_Object *obj;
-
-	obj = l->data;
 	if (e->events_frozen <= 0)
 	  evas_object_event_callback_call(obj, EVAS_CALLBACK_HOLD, &ev);
 	if (e->delete_me) break;
      }
-   if (copy) copy = evas_list_free(copy);
+   if (copy) copy = eina_list_free(copy);
    _evas_unwalk(e);
 }
 
+
 /**
- * @defgroup Evas_Object_Event_Flags_Group Evas Object Event Flag Functions
- *
- * Functions that deal with how events on an Evas Object are processed.
+ * @addtogroup Evas_Object_Group
+ * @{
  */
 
 /**
@@ -1180,6 +1139,10 @@ evas_object_propagate_events_get(const Evas_Object *obj)
    return !(obj->no_propagate);
 }
 
+/**
+ * @}
+ */
+
 /**
  * Set pointer behavior.
  *
