@@ -1,16 +1,27 @@
-#include "evas_common.h"
-#include "evas_private.h"
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#ifdef HAVE_EVIL
+# include <Evil.h>
+#endif
+
 #ifdef BUILD_FONT_LOADER_EET
 #include <Eet.h>
 #endif
+
 #ifdef HAVE_FONTCONFIG
 #include <fontconfig/fontconfig.h>
 #endif
 
+#include "evas_common.h"
+#include "evas_private.h"
+
 /* font dir cache */
-static Evas_Hash *font_dirs = NULL;
-static Evas_List *fonts_cache = NULL;
-static Evas_List *fonts_zero = NULL;
+static Eina_Hash *font_dirs = NULL;
+static Eina_List *fonts_cache = NULL;
+static Eina_List *fonts_zero = NULL;
 
 typedef struct _Fndat Fndat;
 
@@ -24,7 +35,7 @@ struct _Fndat
 };
 
 /* private methods for font dir cache */
-static Evas_Bool font_cache_dir_free(const Evas_Hash *hash, const char *key, void *data, void *fdata);
+static Evas_Bool font_cache_dir_free(const Eina_Hash *hash, const void *key, void *data, void *fdata);
 static Evas_Font_Dir *object_text_font_cache_dir_update(char *dir, Evas_Font_Dir *fd);
 static Evas_Font *object_text_font_cache_font_find_x(Evas_Font_Dir *fd, char *font);
 static Evas_Font *object_text_font_cache_font_find_file(Evas_Font_Dir *fd, char *font);
@@ -34,22 +45,31 @@ static Evas_Font_Dir *object_text_font_cache_dir_add(char *dir);
 static void object_text_font_cache_dir_del(char *dir, Evas_Font_Dir *fd);
 static int evas_object_text_font_string_parse(char *buffer, char dest[14][256]);
 
+#ifdef HAVE_FONTCONFIG
+static int fc_init = 0;
+#endif
+
 void
 evas_font_dir_cache_free(void)
 {
+#ifdef HAVE_FONTCONFIG
+   fc_init--;
+   if (fc_init == 0) FcFini();
+#endif   
    if (!font_dirs) return;
 
-   evas_hash_foreach(font_dirs, font_cache_dir_free, NULL);
-   evas_hash_free(font_dirs);
+   eina_hash_foreach(font_dirs, font_cache_dir_free, NULL);
+   eina_hash_free(font_dirs);
    font_dirs = NULL;
 }
 
 const char *
 evas_font_dir_cache_find(char *dir, char *font)
 {
-   Evas_Font_Dir *fd;
+   Evas_Font_Dir *fd = NULL;
 
-   fd = evas_hash_find(font_dirs, dir);
+   if (!font_dirs) font_dirs = eina_hash_string_superfast_new(NULL);
+   else fd = eina_hash_find(font_dirs, dir);
    fd = object_text_font_cache_dir_update(dir, fd);
    if (fd)
      {
@@ -64,16 +84,16 @@ evas_font_dir_cache_find(char *dir, char *font)
    return NULL;
 }
 
-static Evas_List *
+static Eina_List *
 evas_font_set_get(const char *name)
 {
-   Evas_List *fonts = NULL;
+   Eina_List *fonts = NULL;
    char *p;
 
    p = strchr(name, ',');
    if (!p)
      {
-	fonts = evas_list_append(fonts, evas_stringshare_add(name));
+	fonts = eina_list_append(fonts, eina_stringshare_add(name));
      }
    else
      {
@@ -86,64 +106,96 @@ evas_font_set_get(const char *name)
 	     nm = alloca(p - pp + 1);
 	     strncpy(nm, pp, p - pp);
 	     nm[p - pp] = 0;
-	     fonts = evas_list_append(fonts, evas_stringshare_add(nm));
+	     fonts = eina_list_append(fonts, eina_stringshare_add(nm));
 	     pp = p + 1;
 	     p = strchr(pp, ',');
-	     if (!p) fonts = evas_list_append(fonts, evas_stringshare_add(pp));
+	     if (!p) fonts = eina_list_append(fonts, eina_stringshare_add(pp));
 	  }
      }
    return fonts;
 }
 
+void
+evas_fonts_zero_free(Evas *evas)
+{
+   Fndat *fd;
+
+   while (fonts_zero)
+     {
+	fd = eina_list_data_get(fonts_zero);
+
+	if (fd->name) eina_stringshare_del(fd->name);
+	if (fd->source) eina_stringshare_del(fd->source);
+	evas->engine.func->font_free(evas->engine.data.output, fd->font);
+	free(fd);
+
+	fonts_zero = eina_list_remove_list(fonts_zero, fonts_zero);
+     }
+}
+
 void
 evas_font_free(Evas *evas, void *font)
 {
-   Evas_List *l;
+   Eina_List *l;
+   Fndat *fd;
 
-   for (l = fonts_cache; l; l = l->next)
+   EINA_LIST_FOREACH(fonts_cache, l, fd)
      {
-	Fndat *fd;
-
-	fd = l->data;
 	if (fd->font == font)
 	  {
 	     fd->ref--;
 	     if (fd->ref == 0)
 	       {
-		  fonts_cache = evas_list_remove_list(fonts_cache, l);
-		  fonts_zero = evas_list_append(fonts_zero, fd);
+		  fonts_cache = eina_list_remove_list(fonts_cache, l);
+		  fonts_zero = eina_list_append(fonts_zero, fd);
 	       }
 	     break;
 	  }
      }
    while ((fonts_zero) &&
-	  (evas_list_count(fonts_zero) > 4)) /* 4 is arbitrary */
+	  (eina_list_count(fonts_zero) > 4)) /* 4 is arbitrary */
      {
-	Fndat *fd;
-
-	fd = evas_list_data(fonts_zero);
+	fd = eina_list_data_get(fonts_zero);
 	if (fd->ref != 0) break;
-	fonts_zero = evas_list_remove_list(fonts_zero, fonts_zero);
-	if (fd->name) evas_stringshare_del(fd->name);
-	if (fd->source) evas_stringshare_del(fd->source);
+	fonts_zero = eina_list_remove_list(fonts_zero, fonts_zero);
+	if (fd->name) eina_stringshare_del(fd->name);
+	if (fd->source) eina_stringshare_del(fd->source);
 	evas->engine.func->font_free(evas->engine.data.output, fd->font);
 	free(fd);
      }
 }
 
+static void
+evas_font_init(void)
+{
+   static int done = 0;
+   if (done) return;
+   done = 1;
+#ifdef HAVE_FONTCONFIG
+   fc_init++;
+   if (fc_init == 1)
+     {
+        FcInit();
+        FcConfigEnableHome(1);
+     }
+#endif   
+}
+
 void *
 evas_font_load(Evas *evas, const char *name, const char *source, int size)
 {
    void *font = NULL;
-   Evas_List *fonts, *l;
+   Eina_List *fonts, *l;
    Fndat *fd;
+   char *nm;
 
    if (!name) return NULL;
    if (name[0] == 0) return NULL;
+   
+   evas_font_init();
 
-   for (l = fonts_cache; l; l = l->next)
+   EINA_LIST_FOREACH(fonts_cache, l, fd)
      {
-	fd = l->data;
 	if (!strcmp(name, fd->name))
 	  {
 	     if (((!source) && (!fd->source)) ||
@@ -151,7 +203,7 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 	       {
 		  if (size == fd->size)
 		    {
-		       fonts_cache = evas_list_promote_list(fonts_cache, l);
+		       fonts_cache = eina_list_promote_list(fonts_cache, l);
 		       fd->ref++;
 		       return fd->font;
 		    }
@@ -159,9 +211,8 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 	  }
      }
 
-   for (l = fonts_zero; l; l = l->next)
+   EINA_LIST_FOREACH(fonts_zero, l, fd)
      {
-	fd = l->data;
 	if (!strcmp(name, fd->name))
 	  {
 	     if (((!source) && (!fd->source)) ||
@@ -169,8 +220,8 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 	       {
 		  if (size == fd->size)
 		    {
-		       fonts_zero = evas_list_remove_list(fonts_zero, l);
-		       fonts_cache = evas_list_prepend(fonts_cache, fd);
+		       fonts_zero = eina_list_remove_list(fonts_zero, l);
+		       fonts_cache = eina_list_prepend(fonts_cache, fd);
 		       fd->ref++;
 		       return fd->font;
 		    }
@@ -178,12 +229,9 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 	  }
      }
    fonts = evas_font_set_get(name);
-   for (l = fonts; l; l = l->next) /* Load each font in append */
+   EINA_LIST_FOREACH(fonts, l, nm) /* Load each font in append */
      {
-	char *nm;
-
-	nm = l->data;
-	if ((l == fonts) || (!font)) /* First iteration OR no font */
+	if (l == fonts || !font) /* First iteration OR no font */
 	  {
 #ifdef BUILD_FONT_LOADER_EET
 	     if (source) /* Load Font from "eet" source */
@@ -223,13 +271,14 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 		    font = evas->engine.func->font_load(evas->engine.data.output, (char *)nm, size);
 		  else /* search font path */
 		    {
-		       Evas_List *l;
+		       Eina_List *l;
+		       char *dir;
 
-		       for (l = evas->font_path; l; l = l->next)
+		       EINA_LIST_FOREACH(evas->font_path, l, dir)
 			 {
 			    const char *f_file;
 
-			    f_file = evas_font_dir_cache_find(l->data, (char *)nm);
+			    f_file = evas_font_dir_cache_find(dir, (char *)nm);
 			    if (f_file)
 			      {
 				 font = evas->engine.func->font_load(evas->engine.data.output, f_file, size);
@@ -285,13 +334,14 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 		    evas->engine.func->font_add(evas->engine.data.output, font, (char *)nm, size);
 		  else
 		    {
-		       Evas_List *l;
+		       Eina_List *l;
+		       char *dir;
 
-		       for (l = evas->font_path; l; l = l->next)
+		       EINA_LIST_FOREACH(evas->font_path, l, dir)
 			 {
 			    const char *f_file;
 
-			    f_file = evas_font_dir_cache_find(l->data, (char *)nm);
+			    f_file = evas_font_dir_cache_find(dir, (char *)nm);
 			    if (f_file)
 			      {
 				 if (evas->engine.func->font_add(evas->engine.data.output, font, f_file, size))
@@ -303,12 +353,11 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
 	       }
 #endif
 	  }
-	evas_stringshare_del(nm);
+	eina_stringshare_del(nm);
      }
-   evas_list_free(fonts);
+   fonts = eina_list_free(fonts);
 
 #ifdef HAVE_FONTCONFIG
-
    if (!font) /* Search using fontconfig */
      {
 	FcPattern *p_nm = NULL;
@@ -350,12 +399,12 @@ evas_font_load(Evas *evas, const char *name, const char *source, int size)
    fd = calloc(1, sizeof(Fndat));
    if (fd)
      {
-	fd->name = evas_stringshare_add(name);
-	if (source) fd->source = evas_stringshare_add(source);
+	fd->name = eina_stringshare_add(name);
+	if (source) fd->source = eina_stringshare_add(source);
 	fd->size = size;
 	fd->font = font;
 	fd->ref = 1;
-	fonts_cache = evas_list_prepend(fonts_cache, fd);
+	fonts_cache = eina_list_prepend(fonts_cache, fd);
      }
 
    if (font)
@@ -371,12 +420,13 @@ evas_font_load_hinting_set(Evas *evas, void *font, int hinting)
 				       hinting);
 }
 
-Evas_List *
+Eina_List *
 evas_font_dir_available_list(const Evas *evas)
 {
-   Evas_List *l;
-   Evas_List *ll;
-   Evas_List *available = NULL;
+   Eina_List *l;
+   Eina_List *ll;
+   Eina_List *available = NULL;
+   char *dir;
 
 #ifdef HAVE_FONTCONFIG
    /* Add font config fonts */
@@ -385,6 +435,8 @@ evas_font_dir_available_list(const Evas *evas)
    FcObjectSet *os;
    int i;
 
+   evas_font_init();
+   
    p = FcPatternCreate();
    os = FcObjectSetBuild(FC_FAMILY, FC_STYLE, NULL);
 
@@ -400,7 +452,7 @@ evas_font_dir_available_list(const Evas *evas)
 	     char *font;
 
 	     font = (char *)FcNameUnparse(set->fonts[i]);
-	     available = evas_list_append(available, evas_stringshare_add(font));
+	     available = eina_list_append(available, eina_stringshare_add(font));
 	     free(font);
 	  }
 
@@ -412,21 +464,20 @@ evas_font_dir_available_list(const Evas *evas)
    if (!evas->font_path)
      return available;
 
-   for (l = evas->font_path; l; l = l->next)
+   if (!font_dirs) font_dirs = eina_hash_string_superfast_new(NULL);
+
+   EINA_LIST_FOREACH(evas->font_path, l, dir)
      {
 	Evas_Font_Dir *fd;
 
-	fd = evas_hash_find(font_dirs, (char *)l->data);
-	fd = object_text_font_cache_dir_update((char *)l->data, fd);
+	fd = eina_hash_find(font_dirs, dir);
+	fd = object_text_font_cache_dir_update(dir, fd);
 	if (fd && fd->aliases)
 	  {
-	     for (ll = fd->aliases; ll; ll = ll->next)
-	       {
-		  Evas_Font_Alias *fa;
+	     Evas_Font_Alias *fa;
 
-		  fa = ll->data;
-		  available = evas_list_append(available, evas_stringshare_add((char *)fa->alias));
-	       }
+	     EINA_LIST_FOREACH(fd->aliases, ll, fa)
+	       available = eina_list_append(available, eina_stringshare_add((char *)fa->alias));
 	  }
      }
 
@@ -434,18 +485,18 @@ evas_font_dir_available_list(const Evas *evas)
 }
 
 void
-evas_font_dir_available_list_free(Evas_List *available)
+evas_font_dir_available_list_free(Eina_List *available)
 {
    while (available)
      {
-	evas_stringshare_del(available->data);
-	available = evas_list_remove(available, available->data);
+	eina_stringshare_del(available->data);
+	available = eina_list_remove(available, available->data);
      }
 }
 
 /* private stuff */
-static Evas_Bool
-font_cache_dir_free(const Evas_Hash *hash, const char *key, void *data, void *fdata)
+static Eina_Bool
+font_cache_dir_free(const Eina_Hash *hash __UNUSED__, const void *key, void *data, void *fdata __UNUSED__)
 {
    object_text_font_cache_dir_del((char *) key, data);
    return 1;
@@ -463,7 +514,7 @@ object_text_font_cache_dir_update(char *dir, Evas_Font_Dir *fd)
 	if (mt != fd->dir_mod_time)
 	  {
 	     object_text_font_cache_dir_del(dir, fd);
-	     font_dirs = evas_hash_del(font_dirs, dir, fd);
+	     eina_hash_del(font_dirs, dir, fd);
 	  }
 	else
 	  {
@@ -475,7 +526,7 @@ object_text_font_cache_dir_update(char *dir, Evas_Font_Dir *fd)
 		  if (mt != fd->fonts_dir_mod_time)
 		    {
 		       object_text_font_cache_dir_del(dir, fd);
-		       font_dirs = evas_hash_del(font_dirs, dir, fd);
+		       eina_hash_del(font_dirs, dir, fd);
 		    }
 		  else
 		    {
@@ -488,7 +539,7 @@ object_text_font_cache_dir_update(char *dir, Evas_Font_Dir *fd)
 		       if (mt != fd->fonts_alias_mod_time)
 			 {
 			    object_text_font_cache_dir_del(dir, fd);
-			    font_dirs = evas_hash_del(font_dirs, dir, fd);
+			    eina_hash_del(font_dirs, dir, fd);
 			 }
 		       else
 			 return fd;
@@ -502,17 +553,15 @@ object_text_font_cache_dir_update(char *dir, Evas_Font_Dir *fd)
 static Evas_Font *
 object_text_font_cache_font_find_x(Evas_Font_Dir *fd, char *font)
 {
-   Evas_List *l;
+   Eina_List *l;
    char font_prop[14][256];
    int num;
+   Evas_Font *fn;
 
    num = evas_object_text_font_string_parse(font, font_prop);
    if (num != 14) return NULL;
-   for (l = fd->fonts; l; l = l->next)
+   EINA_LIST_FOREACH(fd->fonts, l, fn)
      {
-	Evas_Font *fn;
-
-	fn = l->data;
 	if (fn->type == 1)
 	  {
 	     int i;
@@ -537,13 +586,11 @@ object_text_font_cache_font_find_x(Evas_Font_Dir *fd, char *font)
 static Evas_Font *
 object_text_font_cache_font_find_file(Evas_Font_Dir *fd, char *font)
 {
-   Evas_List *l;
+   Eina_List *l;
+   Evas_Font *fn;
 
-   for (l = fd->fonts; l; l = l->next)
+   EINA_LIST_FOREACH(fd->fonts, l, fn)
      {
-	Evas_Font *fn;
-
-	fn = l->data;
 	if (fn->type == 0)
 	  {
 	     if (!strcasecmp(font, fn->simple.name)) return fn;
@@ -555,15 +602,11 @@ object_text_font_cache_font_find_file(Evas_Font_Dir *fd, char *font)
 static Evas_Font *
 object_text_font_cache_font_find_alias(Evas_Font_Dir *fd, char *font)
 {
-   Evas_List *l;
+   Eina_List *l;
+   Evas_Font_Alias *fa;
 
-   for (l = fd->aliases; l; l = l->next)
-     {
-	Evas_Font_Alias *fa;
-
-	fa = l->data;
-	if (!strcasecmp(fa->alias, font)) return fa->fn;
-     }
+   EINA_LIST_FOREACH(fd->aliases, l, fa)
+     if (!strcasecmp(fa->alias, font)) return fa->fn;
    return NULL;
 }
 
@@ -572,13 +615,13 @@ object_text_font_cache_font_find(Evas_Font_Dir *fd, char *font)
 {
    Evas_Font *fn;
 
-   fn = evas_hash_find(fd->lookup, font);
+   fn = eina_hash_find(fd->lookup, font);
    if (fn) return fn;
    fn = object_text_font_cache_font_find_alias(fd, font);
    if (!fn) fn = object_text_font_cache_font_find_x(fd, font);
    if (!fn) fn = object_text_font_cache_font_find_file(fd, font);
    if (!fn) return NULL;
-   fd->lookup = evas_hash_add(fd->lookup, font, fn);
+   eina_hash_add(fd->lookup, font, fn);
    return fn;
 }
 
@@ -587,11 +630,14 @@ object_text_font_cache_dir_add(char *dir)
 {
    Evas_Font_Dir *fd;
    char *tmp, *tmp2;
-   Evas_List *fdir;
+   Eina_List *fdir;
+   Evas_Font *fn;
 
    fd = calloc(1, sizeof(Evas_Font_Dir));
    if (!fd) return NULL;
-   font_dirs = evas_hash_add(font_dirs, dir, fd);
+   fd->lookup = eina_hash_string_superfast_new(NULL);
+
+   eina_hash_add(font_dirs, dir, fd);
 
    /* READ fonts.alias, fonts.dir and directory listing */
 
@@ -601,7 +647,7 @@ object_text_font_cache_dir_add(char *dir)
      {
 	FILE *f;
 
-	f = fopen(tmp, "r");
+	f = fopen(tmp, "rb");
 	if (f)
 	  {
 	     int num;
@@ -620,21 +666,19 @@ object_text_font_cache_dir_add(char *dir)
 		  num = evas_object_text_font_string_parse((char *)fdef, font_prop);
 		  if (num == 14)
 		    {
-		       Evas_Font *fn;
-
 		       fn = calloc(1, sizeof(Evas_Font));
 		       if (fn)
 			 {
 			    fn->type = 1;
 			    for (i = 0; i < 14; i++)
-			      fn->x.prop[i] = evas_stringshare_add(font_prop[i]);
+			      fn->x.prop[i] = eina_stringshare_add(font_prop[i]);
 			    tmp2 = evas_file_path_join(dir, fname);
 			    if (tmp2)
 			      {
-				 fn->path = evas_stringshare_add(tmp2);
+				 fn->path = eina_stringshare_add(tmp2);
 				 free(tmp2);
 			      }
-			    fd->fonts = evas_list_append(fd->fonts, fn);
+			    fd->fonts = eina_list_append(fd->fonts, fn);
 			 }
 		    }
 	       }
@@ -651,8 +695,6 @@ object_text_font_cache_dir_add(char *dir)
 	tmp = evas_file_path_join(dir, fdir->data);
 	if (tmp)
 	  {
-	     Evas_Font *fn;
-
 	     fn = calloc(1, sizeof(Evas_Font));
 	     if (fn)
 	       {
@@ -663,19 +705,19 @@ object_text_font_cache_dir_add(char *dir)
 		  strcpy(tmp2, fdir->data);
 		  p = strrchr(tmp2, '.');
 		  if (p) *p = 0;
-		  fn->simple.name = evas_stringshare_add(tmp2);
+		  fn->simple.name = eina_stringshare_add(tmp2);
 		  tmp2 = evas_file_path_join(dir, fdir->data);
 		  if (tmp2)
 		    {
-		       fn->path = evas_stringshare_add(tmp2);
+		       fn->path = eina_stringshare_add(tmp2);
 		       free(tmp2);
 		    }
-		  fd->fonts = evas_list_append(fd->fonts, fn);
+		  fd->fonts = eina_list_append(fd->fonts, fn);
 	       }
 	     free(tmp);
 	  }
 	free(fdir->data);
-	fdir = evas_list_remove(fdir, fdir->data);
+	fdir = eina_list_remove(fdir, fdir->data);
      }
 
    /* fonts.alias */
@@ -684,7 +726,7 @@ object_text_font_cache_dir_add(char *dir)
      {
 	FILE *f;
 
-	f = fopen(tmp, "r");
+	f = fopen(tmp, "rb");
 	if (f)
 	  {
 	     char fname[4096], fdef[4096];
@@ -699,15 +741,15 @@ object_text_font_cache_dir_add(char *dir)
 		  fa = calloc(1, sizeof(Evas_Font_Alias));
 		  if (fa)
 		    {
-		       fa->alias = evas_stringshare_add(fname);
+		       fa->alias = eina_stringshare_add(fname);
 		       fa->fn = object_text_font_cache_font_find_x(fd, fdef);
 		       if ((!fa->alias) || (!fa->fn))
 			 {
-			    if (fa->alias) evas_stringshare_del(fa->alias);
+			    if (fa->alias) eina_stringshare_del(fa->alias);
 			    free(fa);
 			 }
 		       else
-			 fd->aliases = evas_list_append(fd->aliases, fa);
+			 fd->aliases = eina_list_append(fd->aliases, fa);
 		    }
 	       }
 	     fclose(f);
@@ -733,22 +775,22 @@ object_text_font_cache_dir_add(char *dir)
 }
 
 static void
-object_text_font_cache_dir_del(char *dir, Evas_Font_Dir *fd)
+object_text_font_cache_dir_del(char *dir __UNUSED__, Evas_Font_Dir *fd)
 {
-   if (fd->lookup) evas_hash_free(fd->lookup);
+   if (fd->lookup) eina_hash_free(fd->lookup);
    while (fd->fonts)
      {
 	Evas_Font *fn;
 	int i;
 
 	fn = fd->fonts->data;
-	fd->fonts = evas_list_remove(fd->fonts, fn);
+	fd->fonts = eina_list_remove(fd->fonts, fn);
 	for (i = 0; i < 14; i++)
 	  {
-	     if (fn->x.prop[i]) evas_stringshare_del(fn->x.prop[i]);
+	     if (fn->x.prop[i]) eina_stringshare_del(fn->x.prop[i]);
 	  }
-	if (fn->simple.name) evas_stringshare_del(fn->simple.name);
-	if (fn->path) evas_stringshare_del(fn->path);
+	if (fn->simple.name) eina_stringshare_del(fn->simple.name);
+	if (fn->path) eina_stringshare_del(fn->path);
 	free(fn);
      }
    while (fd->aliases)
@@ -756,8 +798,8 @@ object_text_font_cache_dir_del(char *dir, Evas_Font_Dir *fd)
 	Evas_Font_Alias *fa;
 
 	fa = fd->aliases->data;
-	fd->aliases = evas_list_remove(fd->aliases, fa);
-	if (fa->alias) evas_stringshare_del(fa->alias);
+	fd->aliases = eina_list_remove(fd->aliases, fa);
+	if (fa->alias) eina_stringshare_del(fa->alias);
 	free(fa);
      }
    free(fd);
