@@ -75,19 +75,35 @@ static const Evas_Object_Func object_func =
      evas_object_gradient_was_opaque,
      NULL,
      NULL,
+     NULL,
+     NULL,
+     NULL,
      NULL
 };
 
 /* the actual api call to add a gradient */
 
 /**
- * @defgroup Evas_Object_Gradient_Group Evas Gradient Object Functions
+ * @defgroup Evas_Object_Gradient_Group Gradient Object Functions
  *
  * Functions that work on evas gradient objects.
  *
  * The following example shows how
  */
 
+/**
+ * @defgroup Evas_Object_Specific Specific Object Functions
+ *
+ * Functions that work on specific objects.
+ *
+ */
+
+/**
+ * @addtogroup Evas_Object_Gradient_Group
+ * @{
+ * @ingroup Evas_Object_Specific
+ */
+
 /**
  * Adds a gradient object to the given evas.
  * @param   e The given evas.
@@ -132,7 +148,6 @@ evas_object_gradient_add(Evas *e)
  * @param   b        Blue component of the given color.
  * @param   a        Alpha component of the given color.
  * @param   delta    Proportion of the gradient object that is this color.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_color_stop_add(Evas_Object *obj, int r, int g, int b, int a, int delta)
@@ -166,7 +181,6 @@ evas_object_gradient_color_stop_add(Evas_Object *obj, int r, int g, int b, int a
  * @param   obj      The given evas gradient object.
  * @param   a        Alpha value.
  * @param   delta    Proportion of the gradient object that is this alpha.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_alpha_stop_add(Evas_Object *obj, int a, int delta)
@@ -191,7 +205,6 @@ evas_object_gradient_alpha_stop_add(Evas_Object *obj, int a, int delta)
 /**
  * Deletes all stops set for the given evas gradient object or any set data.
  * @param   obj The given evas gradient object.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_clear(Evas_Object *obj)
@@ -224,7 +237,6 @@ evas_object_gradient_clear(Evas_Object *obj)
  * @param   data      The color data to be set. Should be in argb32 pixel format.
  * @param   len       The length of the data pointer - multiple of the pixel size.
  * @param   has_alpha A flag indicating if the data has alpha or not.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_color_data_set(Evas_Object *obj, void *data, int len, Evas_Bool has_alpha)
@@ -256,7 +268,6 @@ evas_object_gradient_color_data_set(Evas_Object *obj, void *data, int len, Evas_
  * @param   obj       The given evas gradient object.
  * @param   data      The alpha data to be set, in a8 format.
  * @param   len       The length of the data pointer - multiple of the pixel size.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_alpha_data_set(Evas_Object *obj, void *data, int len)
@@ -279,13 +290,6 @@ evas_object_gradient_alpha_data_set(Evas_Object *obj, void *data, int len)
    evas_object_change(obj);
 }
 
-/**
- * @defgroup Evas_Object_Gradient_Fill_Group Gradient Object Fill Rectangle Functions
- *
- * Functions that deal with what areas of the gradient object are to be
- * tiled with the gradient.
- */
-
 /**
  * Sets the rectangle on the gradient object that the gradient will be
  * drawn to.
@@ -304,7 +308,6 @@ evas_object_gradient_alpha_data_set(Evas_Object *obj, void *data, int len)
  * @param   y   The Y coordinate for the top left corner of the rect.
  * @param   w   The width of the rect.
  * @param   h   The height of the rect.
- * @ingroup Evas_Object_Gradient_Fill_Group
  */
 EAPI void
 evas_object_gradient_fill_set(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h)
@@ -344,7 +347,6 @@ evas_object_gradient_fill_set(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas
  * @param   y   Pointer to an Evas_Coord to store the Y coordinate in.
  * @param   w   Pointer to an Evas_Coord to store the width in.
  * @param   h   Pointer to an Evas_Coord to store the height in.
- * @ingroup Evas_Object_Gradient_Fill_Group
  */
 EAPI void
 evas_object_gradient_fill_get(const Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
@@ -378,7 +380,6 @@ evas_object_gradient_fill_get(const Evas_Object *obj, Evas_Coord *x, Evas_Coord
  * from vertical.
  * @param   obj   The given evas gradient object.
  * @param   angle Angle in degrees.  Can be negative.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_fill_angle_set(Evas_Object *obj, Evas_Angle angle)
@@ -403,7 +404,6 @@ evas_object_gradient_fill_angle_set(Evas_Object *obj, Evas_Angle angle)
  * clockwise from the vertical.
  * @param   obj The given evas gradient object.
  * @return  The current angle if successful. @c 0.0 otherwise.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI Evas_Angle
 evas_object_gradient_fill_angle_get(const Evas_Object *obj)
@@ -426,7 +426,6 @@ evas_object_gradient_fill_angle_get(const Evas_Object *obj)
  * @param   spread One of EVAS_TEXTURE_REFLECT, EVAS_TEXTURE_REPEAT,
  * EVAS_TEXTURE_RESTRICT, EVAS_TEXTURE_RESTRICT_REFLECT, EVAS_TEXTURE_RESTRICT_REPEAT,
  * or EVAS_TEXTURE_PAD.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_fill_spread_set(Evas_Object *obj, int spread)
@@ -450,7 +449,6 @@ evas_object_gradient_fill_spread_set(Evas_Object *obj, int spread)
  * Retrieves the spread (tiling mode) for the given gradient object's fill.
  * @param   obj The given evas gradient object.
  * @return  The current spread mode of the gradient object.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI int
 evas_object_gradient_fill_spread_get(const Evas_Object *obj)
@@ -473,7 +471,6 @@ evas_object_gradient_fill_spread_get(const Evas_Object *obj)
  * Used mostly by 'linear' kinds of gradients.
  * @param   obj   The given evas gradient object.
  * @param   angle Angle in degrees.  Can be negative.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_angle_set(Evas_Object *obj, Evas_Angle angle)
@@ -499,7 +496,6 @@ evas_object_gradient_angle_set(Evas_Object *obj, Evas_Angle angle)
  * rel to its intrinsic orientation.
  * @param   obj The given evas gradient object.
  * @return  The current angle if successful. @c 0.0 otherwise.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI Evas_Angle
 evas_object_gradient_angle_get(const Evas_Object *obj)
@@ -520,7 +516,6 @@ evas_object_gradient_angle_get(const Evas_Object *obj)
  * Sets the offset of the given evas gradient object's spectrum.
  * @param   obj   The given evas gradient object.
  * @param   offset Values can be negative.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_offset_set(Evas_Object *obj, float offset)
@@ -544,7 +539,6 @@ evas_object_gradient_offset_set(Evas_Object *obj, float offset)
  * Retrieves the spectrum's offset
  * @param   obj The given evas gradient object.
  * @return  The current gradient offset if successful. @c 0.0 otherwise.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI float
 evas_object_gradient_offset_get(const Evas_Object *obj)
@@ -565,7 +559,6 @@ evas_object_gradient_offset_get(const Evas_Object *obj)
  * Sets the direction of the given evas gradient object's spectrum.
  * @param   obj   The given evas gradient object.
  * @param   direction Values are either 1 (the default) or -1.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI void
 evas_object_gradient_direction_set(Evas_Object *obj, int direction)
@@ -590,7 +583,6 @@ evas_object_gradient_direction_set(Evas_Object *obj, int direction)
  * Retrieves the evas gradient object's spectrum direction
  * @param   obj The given evas gradient object.
  * @return  The current gradient direction if successful. @c 1 otherwise.
- * @ingroup Evas_Object_Gradient_Group
  */
 EAPI int
 evas_object_gradient_direction_get(const Evas_Object *obj)
@@ -607,21 +599,12 @@ evas_object_gradient_direction_get(const Evas_Object *obj)
    return o->cur.map.direction;
 }
 
-/**
- * @defgroup Evas_Object_Gradient_Type_Group Gradient Object Type Functions
- *
- * Functions that set or get a gradient's geometric type. Examples are "linear",
- * "linear.diag", "linear.codiag", "radial", "rectangular", "angular", "sinusoidal", ...
- * Some types may accept additional parameters to further specify the look.
- */
-
 /**
  * Sets the geometric type displayed by the given gradient object.
  * @param   obj  The given gradient object.
  * @param   name Name of the geometric type that the gradient is to be drawn as.
  * @param   params List of allowable params that the given gradient type allows.
  * Can be NULL.
- * @ingroup Evas_Object_Gradient_Type_Group
  */
 EAPI void
 evas_object_gradient_type_set(Evas_Object *obj, const char *name, const char *params)
@@ -695,7 +678,6 @@ evas_object_gradient_type_set(Evas_Object *obj, const char *name, const char *pa
  *               name in.
  * @param   params  Pointer to a character pointer to store the pointer to the type
  *               params string in.
- * @ingroup Evas_Object_Gradient_Type_Group
  */
 EAPI void
 evas_object_gradient_type_get(const Evas_Object *obj, char **name, char **params)
@@ -718,6 +700,11 @@ evas_object_gradient_type_get(const Evas_Object *obj, char **name, char **params
 }
 
 
+
+/**
+ * @}
+ */
+
 /*
   these two functions don't really belong here as they can apply to other
   objs as well, but for now..
@@ -914,7 +901,6 @@ evas_object_gradient_render(Evas_Object *obj, void *output, void *context, void
 static void
 evas_object_gradient_render_pre(Evas_Object *obj)
 {
-   Evas_Rectangles rects = { 0, 0, NULL };
    Evas_Object_Gradient *o;
    int is_v, was_v;
 
@@ -1005,17 +991,17 @@ evas_object_gradient_render_pre(Evas_Object *obj)
    was_v = evas_object_was_visible(obj);
    if (is_v != was_v)
      {
-	evas_object_render_pre_visible_change(&rects, obj, is_v, was_v);
+	evas_object_render_pre_visible_change(&obj->layer->evas->clip_changes, obj, is_v, was_v);
 	goto done;
      }
    /* its not visible - we accounted for it appearing or not so just abort */
    if (!is_v) goto done;
    /* clipper changed this is in addition to anything else for obj */
-   evas_object_render_pre_clipper_change(&rects, obj);
+   evas_object_render_pre_clipper_change(&obj->layer->evas->clip_changes, obj);
    /* gradient changed */
    if (o->changed || obj->restack)
      {
-	evas_object_render_pre_prev_cur_add(&rects, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	goto done;
      }
    /* if it changed geometry */
@@ -1024,7 +1010,7 @@ evas_object_gradient_render_pre(Evas_Object *obj)
        (obj->cur.geometry.w != obj->prev.geometry.w) ||
        (obj->cur.geometry.h != obj->prev.geometry.h))
      {
-	evas_object_render_pre_prev_cur_add(&rects, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	goto done;
      }
    /* it obviously didn't change - add a NO obscure - this "unupdates"  this */
@@ -1040,7 +1026,7 @@ evas_object_gradient_render_pre(Evas_Object *obj)
 							    obj->cur.cache.clip.h);
    
    done:
-   evas_object_render_pre_effect_updates(&rects, obj, is_v, was_v);
+   evas_object_render_pre_effect_updates(&obj->layer->evas->clip_changes, obj, is_v, was_v);
 }
 
 static void
@@ -1053,14 +1039,7 @@ evas_object_gradient_render_post(Evas_Object *obj)
    /* data anymore we can free it if the object deems this is a good idea */
    o = (Evas_Object_Gradient *)(obj->object_data);
    /* remove those pesky changes */
-   while (obj->clip.changes)
-     {
-	Evas_Rectangle *r;
-
-	r = (Evas_Rectangle *)obj->clip.changes->data;
-	obj->clip.changes = evas_list_remove(obj->clip.changes, r);
-	free(r);
-     }
+   evas_object_clip_changes_clean(obj);
    /* move cur to prev safely for object data */
    obj->prev = obj->cur;
    if (o->prev.type.name && (o->prev.type.name != o->cur.type.name))
