@@ -9,8 +9,8 @@ struct _Evas_Object_Smart
    DATA32            magic;
    void             *engine_data;
    void             *data;
-   Evas_List        *callbacks;
-   Evas_Object_List *contained;
+   Eina_List        *callbacks;
+   Eina_Inlist *contained;
    int               walking_list;
    Evas_Bool         deletions_waiting : 1;
    Evas_Bool         need_recalculate : 1;
@@ -56,6 +56,9 @@ static const Evas_Object_Func object_func =
      NULL,
      NULL,
      NULL,
+     NULL,
+     NULL,
+     NULL,
      NULL
 };
 
@@ -181,19 +184,29 @@ evas_object_smart_member_add(Evas_Object *obj, Evas_Object *smart_obj)
 	abort();
 	return;
      }
+   if (obj->layer && smart_obj->layer
+       && obj->layer->evas != smart_obj->layer->evas)
+     {
+	printf("EVAS ERROR: Adding object %p from Evas (%p) from another Evas (%p)\n", obj, obj->layer->evas, smart_obj->layer->evas);
+	abort();
+	return;
+     }
+
    if (obj->smart.parent == smart_obj) return;
    
    if (obj->smart.parent) evas_object_smart_member_del(obj);
-   
+
    evas_object_release(obj, 1);
    obj->layer = smart_obj->layer;
    obj->cur.layer = obj->layer->layer;
    obj->layer->usage++;
    obj->smart.parent = smart_obj;
-   o->contained = evas_object_list_append(o->contained, obj);
+   o->contained = eina_inlist_append(o->contained, EINA_INLIST_GET(obj));
    evas_object_smart_member_cache_invalidate(obj);
    obj->restack = 1;
    evas_object_change(obj);
+   if (smart_obj->smart.smart->smart_class->member_add)
+     smart_obj->smart.smart->smart_class->member_add(smart_obj, obj);
 }
 
 /**
@@ -211,6 +224,7 @@ EAPI void
 evas_object_smart_member_del(Evas_Object *obj)
 {
    Evas_Object_Smart *o;
+   Evas_Object *smart_obj;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
@@ -218,8 +232,12 @@ evas_object_smart_member_del(Evas_Object *obj)
 
    if (!obj->smart.parent) return;
 
+   smart_obj = obj->smart.parent;
+   if (smart_obj->smart.smart->smart_class->member_del)
+     smart_obj->smart.smart->smart_class->member_del(smart_obj, obj);
+
    o = (Evas_Object_Smart *)(obj->smart.parent->object_data);
-   o->contained = evas_object_list_remove(o->contained, obj);
+   o->contained = eina_inlist_remove(o->contained, EINA_INLIST_GET(obj));
    obj->smart.parent = NULL;
    evas_object_smart_member_cache_invalidate(obj);
    obj->layer->usage--;
@@ -249,14 +267,14 @@ evas_object_smart_parent_get(const Evas_Object *obj)
  * Gets the list of the member objects of an Evas_Object
  * @param obj the Evas_Object you want to get the list of member objects
  * @return Returns the list of the member objects of @a obj.
- * The returned list should be freed with evas_list_free() when you no longer need it
+ * The returned list should be freed with eina_list_free() when you no longer need it
  */
-EAPI Evas_List *
+EAPI Eina_List *
 evas_object_smart_members_get(const Evas_Object *obj)
 {
    Evas_Object_Smart *o;
-   Evas_List *members;
-   Evas_Object_List *member;
+   Eina_List *members;
+   Eina_Inlist *member;
    
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return NULL;
@@ -268,12 +286,12 @@ evas_object_smart_members_get(const Evas_Object *obj)
 
    members = NULL;
    for (member = o->contained; member; member = member->next)
-      members = evas_list_append(members, member);
+      members = eina_list_append(members, member);
    
    return members;
 }
 
-const Evas_Object_List *
+const Eina_Inlist *
 evas_object_smart_members_get_direct(const Evas_Object *obj)
 {
    Evas_Object_Smart *o;
@@ -341,10 +359,10 @@ evas_object_smart_callback_add(Evas_Object *obj, const char *event, void (*func)
    if (!event) return;
    if (!func) return;
    cb = calloc(1, sizeof(Evas_Smart_Callback));
-   cb->event = evas_stringshare_add(event);
+   cb->event = eina_stringshare_add(event);
    cb->func = func;
    cb->func_data = (void *)data;
-   o->callbacks = evas_list_prepend(o->callbacks, cb);
+   o->callbacks = eina_list_prepend(o->callbacks, cb);
 }
 
 /**
@@ -362,7 +380,8 @@ EAPI void *
 evas_object_smart_callback_del(Evas_Object *obj, const char *event, void (*func) (void *data, Evas_Object *obj, void *event_info))
 {
    Evas_Object_Smart *o;
-   Evas_List *l;
+   Eina_List *l;
+   Evas_Smart_Callback *cb;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return NULL;
@@ -372,11 +391,8 @@ evas_object_smart_callback_del(Evas_Object *obj, const char *event, void (*func)
    return NULL;
    MAGIC_CHECK_END();
    if (!event) return NULL;
-   for (l = o->callbacks; l; l = l->next)
+   EINA_LIST_FOREACH(o->callbacks, l, cb)
      {
-	Evas_Smart_Callback *cb;
-
-	cb = l->data;
 	if ((!strcmp(cb->event, event)) && (cb->func == func))
 	  {
 	     void *data;
@@ -408,7 +424,8 @@ EAPI void
 evas_object_smart_callback_call(Evas_Object *obj, const char *event, void *event_info)
 {
    Evas_Object_Smart *o;
-   Evas_List *l;
+   Eina_List *l;
+   Evas_Smart_Callback *cb;
 
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
@@ -420,11 +437,8 @@ evas_object_smart_callback_call(Evas_Object *obj, const char *event, void *event
    if (!event) return;
    if (obj->delete_me) return;
    o->walking_list++;
-   for (l = o->callbacks; l; l = l->next)
+   EINA_LIST_FOREACH(o->callbacks, l, cb)
      {
-	Evas_Smart_Callback *cb;
-
-	cb = l->data;
 	if (!cb->delete_me)
 	  {
 	     if (!strcmp(cb->event, event))
@@ -485,7 +499,7 @@ evas_object_smart_need_recalculate_set(Evas_Object *obj, Evas_Bool value)
      {
 	Evas *e;
 	e = obj->layer->evas;
-	_evas_array_append(&e->calculate_objects, obj);
+	eina_array_push(&e->calculate_objects, obj);
      }
    /* TODO: else, remove from array */
 }
@@ -503,15 +517,15 @@ evas_object_smart_need_recalculate_set(Evas_Object *obj, Evas_Bool value)
  * @ingroup Evas_Smart_Object_Group
  */
 EAPI Evas_Bool
-evas_object_smart_need_recalculate_get(Evas_Object *obj)
+evas_object_smart_need_recalculate_get(const Evas_Object *obj)
 {
    Evas_Object_Smart *o;
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
-   return;
+   return 0;
    MAGIC_CHECK_END();
    o = obj->object_data;
    MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
-   return;
+   return 0;
    MAGIC_CHECK_END();
 
    return o->need_recalculate;
@@ -537,9 +551,11 @@ evas_object_smart_calculate(Evas_Object *obj)
    return;
    MAGIC_CHECK_END();
 
-   if (obj->smart.smart->smart_class->calculate)
-     obj->smart.smart->smart_class->calculate(obj);
+   if (!obj->smart.smart->smart_class->calculate)
+     return;
+
    o->need_recalculate = 0;
+   obj->smart.smart->smart_class->calculate(obj);
 }
 
 /**
@@ -550,7 +566,7 @@ evas_object_smart_calculate(Evas_Object *obj)
 void
 evas_call_smarts_calculate(Evas *e)
 {
-   Evas_Array *calculate;
+   Eina_Array *calculate;
    unsigned int i;
 
    calculate = &e->calculate_objects;
@@ -559,19 +575,19 @@ evas_call_smarts_calculate(Evas *e)
 	Evas_Object *obj;
 	Evas_Object_Smart *o;
 
-	obj = _evas_array_get(calculate, i);
+	obj = eina_array_data_get(calculate, i);
 	if (obj->delete_me)
 	  continue;
 
 	o = obj->object_data;
 	if (o->need_recalculate)
 	  {
-	     obj->smart.smart->smart_class->calculate(obj);
 	     o->need_recalculate = 0;
+	     obj->smart.smart->smart_class->calculate(obj);
 	  }
      }
 
-   evas_array_flush(calculate);
+   eina_array_flush(calculate);
 }
 
 /**
@@ -599,7 +615,8 @@ static void
 evas_object_smart_callbacks_clear(Evas_Object *obj)
 {
    Evas_Object_Smart *o;
-   Evas_List *l;
+   Eina_List *l;
+   Evas_Smart_Callback *cb;
 
    o = (Evas_Object_Smart *)(obj->object_data);
 
@@ -607,14 +624,12 @@ evas_object_smart_callbacks_clear(Evas_Object *obj)
    if (!o->deletions_waiting) return;
    for (l = o->callbacks; l;)
      {
-	Evas_Smart_Callback *cb;
-
-	cb = l->data;
-	l = l->next;
+	cb = eina_list_data_get(l);
+	l = eina_list_next(l);
 	if (cb->delete_me)
 	  {
-	     o->callbacks = evas_list_remove(o->callbacks, cb);
-	     if (cb->event) evas_stringshare_del(cb->event);
+	     o->callbacks = eina_list_remove(o->callbacks, cb);
+	     if (cb->event) eina_stringshare_del(cb->event);
 	     free(cb);
 	  }
      }
@@ -651,8 +666,8 @@ evas_object_smart_cleanup(Evas_Object *obj)
 	     Evas_Smart_Callback *cb;
 
 	     cb = o->callbacks->data;
-	     o->callbacks = evas_list_remove(o->callbacks, cb);
-	     if (cb->event) evas_stringshare_del(cb->event);
+	     o->callbacks = eina_list_remove(o->callbacks, cb);
+	     if (cb->event) eina_stringshare_del(cb->event);
 	     free(cb);
 	  }
 
@@ -667,7 +682,7 @@ void
 evas_object_smart_member_cache_invalidate(Evas_Object *obj)
 {
    Evas_Object_Smart *o;
-   Evas_Object_List *l;
+   Eina_Inlist *l;
 
    o = (Evas_Object_Smart *)(obj->object_data);
    if (o->magic != MAGIC_OBJ_SMART)
@@ -689,8 +704,7 @@ evas_object_smart_member_raise(Evas_Object *member)
    Evas_Object_Smart *o;
 
    o = (Evas_Object_Smart *)(member->smart.parent->object_data);
-   o->contained = evas_object_list_remove(o->contained, member);
-   o->contained = evas_object_list_append(o->contained, member);
+   o->contained = eina_inlist_demote(o->contained, EINA_INLIST_GET(member));
 }
 
 void
@@ -699,8 +713,7 @@ evas_object_smart_member_lower(Evas_Object *member)
    Evas_Object_Smart *o;
 
    o = (Evas_Object_Smart *)(member->smart.parent->object_data);
-   o->contained = evas_object_list_remove(o->contained, member);
-   o->contained = evas_object_list_prepend(o->contained, member);
+   o->contained = eina_inlist_promote(o->contained, EINA_INLIST_GET(member));
 }
 
 void
@@ -709,8 +722,8 @@ evas_object_smart_member_stack_above(Evas_Object *member, Evas_Object *other)
    Evas_Object_Smart *o;
 
    o = (Evas_Object_Smart *)(member->smart.parent->object_data);
-   o->contained = evas_object_list_remove(o->contained, member);
-   o->contained = evas_object_list_append_relative(o->contained, member, other);
+   o->contained = eina_inlist_remove(o->contained, EINA_INLIST_GET(member));
+   o->contained = eina_inlist_append_relative(o->contained, EINA_INLIST_GET(member), EINA_INLIST_GET(other));
 }
 
 void
@@ -719,8 +732,8 @@ evas_object_smart_member_stack_below(Evas_Object *member, Evas_Object *other)
    Evas_Object_Smart *o;
 
    o = (Evas_Object_Smart *)(member->smart.parent->object_data);
-   o->contained = evas_object_list_remove(o->contained, member);
-   o->contained = evas_object_list_prepend_relative(o->contained, member, other);
+   o->contained = eina_inlist_remove(o->contained, EINA_INLIST_GET(member));
+   o->contained = eina_inlist_prepend_relative(o->contained, EINA_INLIST_GET(member), EINA_INLIST_GET(other));
 }
 
 /* all nice and private */
@@ -772,11 +785,9 @@ evas_object_smart_free(Evas_Object *obj)
 }
 
 static void
-evas_object_smart_render(Evas_Object *obj, void *output, void *context, void *surface, int x, int y)
+evas_object_smart_render(Evas_Object *obj __UNUSED__, void *output __UNUSED__, void *context __UNUSED__, void *surface __UNUSED__, int x __UNUSED__, int y __UNUSED__)
 {
    return;
-   obj = output = context = surface = NULL;
-   x = y = 0;
 }
 
 static void
