@@ -50,6 +50,8 @@ static void *evas_object_text_engine_data_get(Evas_Object *obj);
 static int evas_object_text_is_opaque(Evas_Object *obj);
 static int evas_object_text_was_opaque(Evas_Object *obj);
 
+static void evas_object_text_scale_update(Evas_Object *obj);
+
 static const Evas_Object_Func object_func =
 {
    /* methods (compulsory) */
@@ -69,12 +71,27 @@ static const Evas_Object_Func object_func =
      evas_object_text_was_opaque,
      NULL,
      NULL,
+     NULL,
+     evas_object_text_scale_update,
+     NULL,
      NULL
 };
 
 /* the actual api call to add a rect */
 /* it has no other api calls as all properties are standard */
 
+/**
+ * @defgroup Evas_Object_Text Text Object Functions
+ *
+ * Functions that operate on evas text objects.
+ */
+
+/**
+ * @addtogroup Evas_Object_Text
+ * @{
+ * @ingroup Evas_Object_Specific
+ */
+
 /**
  * Creates a new text @c Evas_Object on the provided @c Evas canvas.
  *
@@ -122,8 +139,8 @@ evas_object_text_font_source_set(Evas_Object *obj, const char *font_source)
    if ((o->cur.source) && (font_source) &&
        (!strcmp(o->cur.source, font_source)))
      return;
-   if (o->cur.source) evas_stringshare_del(o->cur.source);
-   if (font_source) o->cur.source = evas_stringshare_add(font_source);
+   if (o->cur.source) eina_stringshare_del(o->cur.source);
+   if (font_source) o->cur.source = eina_stringshare_add(font_source);
    else o->cur.source = NULL;
 }
 
@@ -193,8 +210,8 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
      }
    if (!same_font)
      {
-	if (o->cur.font) evas_stringshare_del(o->cur.font);
-	if (font) o->cur.font = evas_stringshare_add(font);
+	if (o->cur.font) eina_stringshare_del(o->cur.font);
+	if (font) o->cur.font = eina_stringshare_add(font);
 	else o->cur.font = NULL;
 	o->prev.font = NULL;
      }
@@ -240,6 +257,7 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
      }
    o->changed = 1;
    evas_object_change(obj);
+   evas_object_clip_dirty(obj);
    evas_object_coords_recalc(obj);
    if (obj->layer->evas->events_frozen <= 0)
      {
@@ -315,8 +333,8 @@ evas_object_text_text_set(Evas_Object *obj, const char *text)
 				       obj->layer->evas->pointer.x,
 				       obj->layer->evas->pointer.y, 1, 1);
    /* DO II */
-   if (o->cur.text) evas_stringshare_del(o->cur.text);
-   if ((text) && (*text)) o->cur.text = evas_stringshare_add(text);
+   if (o->cur.text) eina_stringshare_del(o->cur.text);
+   if ((text) && (*text)) o->cur.text = eina_stringshare_add(text);
    else o->cur.text = NULL;
    o->prev.text = NULL;
    if ((o->engine_data) && (o->cur.text))
@@ -344,6 +362,7 @@ evas_object_text_text_set(Evas_Object *obj, const char *text)
      }
    o->changed = 1;
    evas_object_change(obj);
+   evas_object_clip_dirty(obj);
    evas_object_coords_recalc(obj);
    is = evas_object_is_in_output_rect(obj,
 				      obj->layer->evas->pointer.x,
@@ -681,6 +700,7 @@ evas_object_text_style_set(Evas_Object *obj, Evas_Text_Style_Type style)
      obj->cur.geometry.w = 0;
    obj->cur.geometry.h += (t - pt) + (b - pb);
    evas_object_change(obj);
+   evas_object_clip_dirty(obj);
 }
 
 /**
@@ -913,9 +933,12 @@ evas_object_text_outline_color_get(const Evas_Object *obj, int *r, int *g, int *
 }
 
 /**
- * To be documented.
+ * Gets the text style pad.
  *
- * FIXME: To be fixed.
+ * @param l The left pad (or NULL).
+ * @param r The right pad (or NULL).
+ * @param t The top pad (or NULL).
+ * @param b The bottom pad (or NULL).
  *
  */
 EAPI void
@@ -941,7 +964,11 @@ evas_object_text_style_pad_get(const Evas_Object *obj, int *l, int *r, int *t, i
 
 
 /**
- * @defgroup Evas_Font_Path_Group Font Path Functions
+ * @}
+ */
+
+/**
+ * @defgroup Evas_Font_Path_Group Evas Font Path Functions
  *
  * Functions that edit the paths being used to load fonts.
  */
@@ -959,8 +986,8 @@ evas_font_path_clear(Evas *e)
    MAGIC_CHECK_END();
    while (e->font_path)
      {
-	evas_stringshare_del(e->font_path->data);
-	e->font_path = evas_list_remove(e->font_path, e->font_path->data);
+	eina_stringshare_del(e->font_path->data);
+	e->font_path = eina_list_remove(e->font_path, e->font_path->data);
      }
 }
 
@@ -978,7 +1005,7 @@ evas_font_path_append(Evas *e, const char *path)
    MAGIC_CHECK_END();
    if (!path) return;
 
-   e->font_path = evas_list_append(e->font_path, evas_stringshare_add(path));
+   e->font_path = eina_list_append(e->font_path, eina_stringshare_add(path));
 }
 
 /**
@@ -995,7 +1022,7 @@ evas_font_path_prepend(Evas *e, const char *path)
    MAGIC_CHECK_END();
    
    if (!path) return;
-   e->font_path = evas_list_prepend(e->font_path, evas_stringshare_add(path));
+   e->font_path = eina_list_prepend(e->font_path, eina_stringshare_add(path));
 }
 
 /**
@@ -1004,7 +1031,7 @@ evas_font_path_prepend(Evas *e, const char *path)
  * @return  The list of font paths used.
  * @ingroup Evas_Font_Path_Group
  */
-EAPI const Evas_List *
+EAPI const Eina_List *
 evas_font_path_list(const Evas *e)
 {
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
@@ -1018,13 +1045,8 @@ evas_font_object_rehint(Evas_Object *obj)
 {
    if (obj->smart.smart)
      {
-	const Evas_Object_List *l3;
-	
-	for (l3 = evas_object_smart_members_get_direct(obj); l3; l3 = l3->next)
-	  {
-	     obj = (Evas_Object *)l3;
-	     evas_font_object_rehint(obj);
-	  }
+	EINA_INLIST_FOREACH(evas_object_smart_members_get_direct(obj), obj)
+	  evas_font_object_rehint(obj);
      }
    else
      {
@@ -1038,26 +1060,19 @@ evas_font_object_rehint(Evas_Object *obj)
 EAPI void
 evas_font_hinting_set(Evas *e, Evas_Font_Hinting_Flags hinting)
 {
-   Evas_Object_List *l;
-   
+   Evas_Layer *lay;
+
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
    MAGIC_CHECK_END();
    if (e->hinting == hinting) return;
    e->hinting = hinting;
-   for (l = (Evas_Object_List *)e->layers; l; l = l->next)
+   EINA_INLIST_FOREACH(e->layers, lay)
      {
-	Evas_Object_List *l2;
-	Evas_Layer *lay;
-	
-	lay = (Evas_Layer *)l;
-	for (l2 = (Evas_Object_List *)lay->objects; l2; l2 = l2->next)
-	  {
-	     Evas_Object *obj;
-	     
-	     obj = (Evas_Object *)l2;
-	     evas_font_object_rehint(obj);
-	  }
+	Evas_Object *obj;
+
+	EINA_INLIST_FOREACH(lay->objects, obj)
+	  evas_font_object_rehint(obj);
      }
 }
 
@@ -1141,7 +1156,7 @@ evas_font_cache_get(const Evas *e)
  * FIXME: To be fixed.
  *
  */
-EAPI Evas_List *
+EAPI Eina_List *
 evas_font_available_list(const Evas *e)
 {
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
@@ -1158,7 +1173,7 @@ evas_font_available_list(const Evas *e)
  *
  */
 EAPI void
-evas_font_available_list_free(Evas *e, Evas_List *available)
+evas_font_available_list_free(Evas *e, Eina_List *available)
 { 
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
@@ -1207,6 +1222,18 @@ evas_string_char_prev_get(const char *str, int pos, int *decoded)
    return p;
 }
 
+/**
+ * Get the length in characters of the string.
+ * @param  str The string to get the length of.
+ * @return The length in characters (not bytes)
+ */
+EAPI int
+evas_string_char_len_get(const char *str)
+{
+	if (!str) return 0;
+	return evas_common_font_utf8_get_len(str);
+}
+
 /**
  * Get the minimum padding a style adds to the text.
  * @param style The style to determine padding.
@@ -1342,9 +1369,9 @@ evas_object_text_free(Evas_Object *obj)
    return;
    MAGIC_CHECK_END();
    /* free obj */
-   if (o->cur.text) evas_stringshare_del(o->cur.text);
-   if (o->cur.font) evas_stringshare_del(o->cur.font);
-   if (o->cur.source) evas_stringshare_del(o->cur.source);
+   if (o->cur.text) eina_stringshare_del(o->cur.text);
+   if (o->cur.font) eina_stringshare_del(o->cur.font);
+   if (o->cur.source) eina_stringshare_del(o->cur.source);
    if (o->engine_data) evas_font_free(obj->layer->evas, o->engine_data);
    o->magic = 0;
    free(o);
@@ -1372,17 +1399,16 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
    ENFN->context_render_op_set(output, context, obj->cur.render_op);
 /*
    ENFN->context_color_set(output,
-						    context,
-						    230, 160, 30, 100);
+                           context,
+                           230, 160, 30, 100);
    ENFN->rectangle_draw(output,
-						 context,
-						 surface,
-						 obj->cur.cache.geometry.x + x,
-						 obj->cur.cache.geometry.y + y,
-						 obj->cur.cache.geometry.w,
-						 obj->cur.cache.geometry.h);
+                        context,
+                        surface,
+                        obj->cur.geometry.x + x,
+                        obj->cur.geometry.y + y,
+                        obj->cur.geometry.w,
+                        obj->cur.geometry.h);
  */
-
 #define COLOR_ONLY_SET(object, sub, col) \
 	ENFN->context_color_set(output, context, \
 				object->sub.col.r, \
@@ -1417,7 +1443,6 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
 				(((int)object->sub.col.g) * (amul)) / 255, \
 				(((int)object->sub.col.b) * (amul)) / 255, \
 				(((int)object->sub.col.a) * (amul)) / 255);
-          
 
 #define DRAW_TEXT(ox, oy) \
    if ((o->engine_data) && (o->cur.text)) \
@@ -1549,7 +1574,6 @@ evas_object_text_render(Evas_Object *obj, void *output, void *context, void *sur
 static void
 evas_object_text_render_pre(Evas_Object *obj)
 {
-   Evas_Rectangles rects = { 0, 0, NULL };
    Evas_Object_Text *o;
    int is_v, was_v;
 
@@ -1575,17 +1599,17 @@ evas_object_text_render_pre(Evas_Object *obj)
    was_v = evas_object_was_visible(obj);
    if (is_v != was_v)
      {
-	evas_object_render_pre_visible_change(&rects, obj, is_v, was_v);
+	evas_object_render_pre_visible_change(&obj->layer->evas->clip_changes, obj, is_v, was_v);
 	goto done;
      }
    /* its not visible - we accounted for it appearing or not so just abort */
    if (!is_v) goto done;
    /* clipper changed this is in addition to anything else for obj */
-   evas_object_render_pre_clipper_change(&rects, obj);
+   evas_object_render_pre_clipper_change(&obj->layer->evas->clip_changes, obj);
    /* if we restacked (layer or just within a layer) and dont clip anyone */
    if (obj->restack)
      {
-	evas_object_render_pre_prev_cur_add(&rects, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	goto done;
      }
    /* if it changed color */
@@ -1594,7 +1618,7 @@ evas_object_text_render_pre(Evas_Object *obj)
        (obj->cur.color.b != obj->prev.color.b) ||
        (obj->cur.color.a != obj->prev.color.a))
      {
-	evas_object_render_pre_prev_cur_add(&rects, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	goto done;
      }
    /* if it changed geometry - and obviously not visibility or color */
@@ -1605,17 +1629,17 @@ evas_object_text_render_pre(Evas_Object *obj)
        (obj->cur.geometry.w != obj->prev.geometry.w) ||
        (obj->cur.geometry.h != obj->prev.geometry.h))
      {
-	evas_object_render_pre_prev_cur_add(&rects, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	goto done;
      }
    if (obj->cur.render_op != obj->prev.render_op)
      {
-	evas_object_render_pre_prev_cur_add(&rects, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	goto done;
      }
    if (obj->cur.scale != obj->prev.scale)
      {
-	evas_object_render_pre_prev_cur_add(&rects, obj);
+	evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	goto done;
      }
    if (o->changed)
@@ -1645,12 +1669,12 @@ evas_object_text_render_pre(Evas_Object *obj)
 	    ((o->cur.glow2.b != o->prev.glow2.b)) ||
 	    ((o->cur.glow2.a != o->prev.glow2.a)))
 	  {
-	     evas_object_render_pre_prev_cur_add(&rects, obj);
+	     evas_object_render_pre_prev_cur_add(&obj->layer->evas->clip_changes, obj);
 	     goto done;
 	  }
      }
    done:
-   evas_object_render_pre_effect_updates(&rects, obj, is_v, was_v);
+   evas_object_render_pre_effect_updates(&obj->layer->evas->clip_changes, obj, is_v, was_v);
 }
 
 static void
@@ -1663,14 +1687,7 @@ evas_object_text_render_post(Evas_Object *obj)
    /* data anymore we can free it if the object deems this is a good idea */
    o = (Evas_Object_Text *)(obj->object_data);
    /* remove those pesky changes */
-   while (obj->clip.changes)
-     {
-	Evas_Rectangle *r;
-
-	r = (Evas_Rectangle *)obj->clip.changes->data;
-	obj->clip.changes = evas_list_remove(obj->clip.changes, r);
-	free(r);
-     }
+   evas_object_clip_changes_clean(obj);
    /* move cur to prev safely for object data */
    obj->prev = obj->cur;
    o->prev = o->cur;
@@ -1707,25 +1724,37 @@ static void *evas_object_text_engine_data_get(Evas_Object *obj)
 static int
 evas_object_text_is_opaque(Evas_Object *obj)
 {
-   Evas_Object_Text *o;
-
    /* this returns 1 if the internal object data implies that the object is */
    /* currently fulyl opque over the entire gradient it occupies */
-   o = (Evas_Object_Text *)(obj->object_data);
    return 0;
 }
 
 static int
 evas_object_text_was_opaque(Evas_Object *obj)
 {
-   Evas_Object_Text *o;
-
    /* this returns 1 if the internal object data implies that the object was */
    /* currently fulyl opque over the entire gradient it occupies */
-   o = (Evas_Object_Text *)(obj->object_data);
    return 0;
 }
 
+static void
+evas_object_text_scale_update(Evas_Object *obj)
+{
+   Evas_Object_Text *o;
+   int size;
+   const char *font;
+   
+   o = (Evas_Object_Text *)(obj->object_data);
+   font = eina_stringshare_add(o->cur.font);
+   size = o->cur.size;
+   if (o->cur.font) eina_stringshare_del(o->cur.font);
+   o->cur.font = NULL;
+   o->prev.font = NULL;
+   o->cur.size = 0;
+   o->prev.size = 0;
+   evas_object_text_font_set(obj, font, size);
+}
+
 void
 _evas_object_text_rehint(Evas_Object *obj)
 {
@@ -1766,6 +1795,7 @@ _evas_object_text_rehint(Evas_Object *obj)
      }
    o->changed = 1;
    evas_object_change(obj);
+   evas_object_clip_dirty(obj);
    evas_object_coords_recalc(obj);
    is = evas_object_is_in_output_rect(obj,
 				      obj->layer->evas->pointer.x,
