@@ -1,7 +1,7 @@
 #include "evas_common.h"
 #include "evas_private.h"
 
-static Evas_List *
+static Eina_List *
 evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char do_draw);
 
 /**
@@ -18,10 +18,9 @@ evas_damage_rectangle_add(Evas *e, int x, int y, int w, int h)
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
    MAGIC_CHECK_END();
-   r = malloc(sizeof(Evas_Rectangle));
+   NEW_RECT(r, x, y, w, h);
    if (!r) return;
-   r->x = x; r->y = y; r->w = w; r->h = h;
-   e->damages = evas_list_append(e->damages, r);
+   e->damages = eina_list_append(e->damages, r);
    e->changed = 1;
 }
 
@@ -39,10 +38,9 @@ evas_obscured_rectangle_add(Evas *e, int x, int y, int w, int h)
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
    MAGIC_CHECK_END();
-   r = malloc(sizeof(Evas_Rectangle));
+   NEW_RECT(r, x, y, w, h);
    if (!r) return;
-   r->x = x; r->y = y; r->w = w; r->h = h;
-   e->obscures = evas_list_append(e->obscures, r);
+   e->obscures = eina_list_append(e->obscures, r);
 }
 
 /**
@@ -54,21 +52,17 @@ evas_obscured_rectangle_add(Evas *e, int x, int y, int w, int h)
 EAPI void
 evas_obscured_clear(Evas *e)
 {
+   Evas_Rectangle *r;
+
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return;
    MAGIC_CHECK_END();
-   while (e->obscures)
-     {
-	Evas_Rectangle *r;
-
-	r = (Evas_Rectangle *)e->obscures->data;
-	e->obscures = evas_list_remove(e->obscures, r);
-	free(r);
-     }
+   EINA_LIST_FREE(e->obscures, r)
+     eina_mempool_free(_evas_rectangle_mp, r);
 }
 
 static void
-_evas_render_phase1_direct(Evas *e, Evas_Array *render_objects)
+_evas_render_phase1_direct(Evas *e, Eina_Array *render_objects)
 {
    unsigned int i;
 
@@ -76,7 +70,7 @@ _evas_render_phase1_direct(Evas *e, Evas_Array *render_objects)
      {
 	Evas_Object *obj;
 
-	obj = _evas_array_get(render_objects, i);
+	obj = eina_array_data_get(render_objects, i);
 	if (obj->changed) obj->func->render_pre(obj);
 	else
 	  {
@@ -96,7 +90,7 @@ _evas_render_phase1_direct(Evas *e, Evas_Array *render_objects)
 }
 
 static Evas_Bool
-_evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active_objects, Evas_Array *restack_objects, Evas_Array *delete_objects, Evas_Array *render_objects, int restack)
+_evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Eina_Array *active_objects, Eina_Array *restack_objects, Eina_Array *delete_objects, Eina_Array *render_objects, int restack)
 {
    int clean_them = 0;
    int is_active;
@@ -108,7 +102,7 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active
    evas_object_clip_recalc(obj);
    /* because of clip objects - delete 2 cycles later */
    if (obj->delete_me == 2)
-     _evas_array_append(delete_objects, obj);
+     eina_array_push(delete_objects, obj);
    else if (obj->delete_me != 0) obj->delete_me++;
    /* If the object will be removed, we should not cache anything during this run. */
    if (obj->delete_me != 0)
@@ -119,11 +113,11 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active
 
    obj->is_active = is_active;
    if ((is_active) || (obj->delete_me != 0))
-     _evas_array_append(active_objects, obj);
+     eina_array_push(active_objects, obj);
    if (restack)
      {
 	if (!obj->changed)
-	  _evas_array_append(&e->pending_objects, obj);
+	  eina_array_push(&e->pending_objects, obj);
 	obj->restack = 1;
 	obj->changed = 1;
 	clean_them = 1;
@@ -132,15 +126,12 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active
      {
 	if (obj->smart.smart)
 	  {
-	     const Evas_Object_List *l;
+	     Evas_Object *obj2;
 
-	     _evas_array_append(render_objects, obj);
+	     eina_array_push(render_objects, obj);
 	     obj->render_pre = 1;
-	     for (l = evas_object_smart_members_get_direct(obj); l; l = l->next)
+	     EINA_INLIST_FOREACH(evas_object_smart_members_get_direct(obj), obj2)
 	       {
-		  Evas_Object *obj2;
-
-		  obj2 = (Evas_Object *)l;
 		  _evas_render_phase1_object_process(e, obj2,
 						     active_objects,
 						     restack_objects,
@@ -154,12 +145,12 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active
 	     if ((is_active) && (obj->restack) && (!obj->clip.clipees) &&
 		 ((evas_object_is_visible(obj) && (!obj->cur.have_clipees)) ||
 		  (evas_object_was_visible(obj) && (!obj->prev.have_clipees))))
-               _evas_array_append(restack_objects, obj);
+               eina_array_push(restack_objects, obj);
 	     else if ((is_active) && (!obj->clip.clipees) &&
 		      ((evas_object_is_visible(obj) && (!obj->cur.have_clipees)) ||
 		       (evas_object_was_visible(obj) && (!obj->prev.have_clipees))))
 	       {
-		  _evas_array_append(render_objects, obj);
+		  eina_array_push(render_objects, obj);
 		  obj->render_pre = 1;
 	       }
 	  }
@@ -172,15 +163,12 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active
 	  {
 	     if (obj->smart.smart)
 	       {
-		  const Evas_Object_List *l;
+		  Evas_Object *obj2;
 
-		  _evas_array_append(render_objects, obj);
+		  eina_array_push(render_objects, obj);
 		  obj->render_pre = 1;
-		  for (l = evas_object_smart_members_get_direct(obj); l; l = l->next)
+		  EINA_INLIST_FOREACH(evas_object_smart_members_get_direct(obj), obj2)
 		    {
-		       Evas_Object *obj2;
-
-		       obj2 = (Evas_Object *)l;
 		       _evas_render_phase1_object_process(e, obj2,
 							  active_objects,
 							  restack_objects,
@@ -194,7 +182,7 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active
 		  if (evas_object_is_opaque(obj) &&
 		      evas_object_is_visible(obj))
 		    {
-		       _evas_array_append(render_objects, obj);
+		       eina_array_push(render_objects, obj);
 		       obj->rect_del = 1;
 		    }
 	       }
@@ -205,22 +193,17 @@ _evas_render_phase1_object_process(Evas *e, Evas_Object *obj, Evas_Array *active
 }
 
 static Evas_Bool
-_evas_render_phase1_process(Evas *e, Evas_Array *active_objects, Evas_Array *restack_objects, Evas_Array *delete_objects, Evas_Array *render_objects)
+_evas_render_phase1_process(Evas *e, Eina_Array *active_objects, Eina_Array *restack_objects, Eina_Array *delete_objects, Eina_Array *render_objects)
 {
-   Evas_Object_List *l;
+   Evas_Layer *lay;
    int clean_them = 0;
 
-   for (l = (Evas_Object_List *)e->layers; l; l = l->next)
+   EINA_INLIST_FOREACH(e->layers, lay)
      {
-	Evas_Object_List *l2;
-	Evas_Layer *lay;
+	Evas_Object *obj;
 
-	lay = (Evas_Layer *)l;
-	for (l2 = (Evas_Object_List *)lay->objects; l2; l2 = l2->next)
+	EINA_INLIST_FOREACH(lay->objects, obj)
 	  {
-	     Evas_Object *obj;
-
-	     obj = (Evas_Object *)l2;
 	     clean_them |= _evas_render_phase1_object_process(e, obj,
 							      active_objects, restack_objects,
 							      delete_objects, render_objects,
@@ -232,7 +215,7 @@ _evas_render_phase1_process(Evas *e, Evas_Array *active_objects, Evas_Array *res
 }
 
 static void
-_evas_render_check_pending_objects(Evas_Array *pending_objects, Evas *e)
+_evas_render_check_pending_objects(Eina_Array *pending_objects, Evas *e)
 {
    unsigned int i;
 
@@ -242,7 +225,7 @@ _evas_render_check_pending_objects(Evas_Array *pending_objects, Evas *e)
 	int ok = 0;
 	int is_active;
 
-	obj = _evas_array_get(pending_objects, i);
+	obj = eina_array_data_get(pending_objects, i);
 
 	if (!obj->layer) goto clean_stuff;
 
@@ -302,11 +285,11 @@ _evas_render_check_pending_objects(Evas_Array *pending_objects, Evas *e)
      clean_stuff:
 	if (!ok)
 	  {
-	     evas_array_clean(&e->active_objects);
-	     evas_array_clean(&e->render_objects);
+	     eina_array_clean(&e->active_objects);
+	     eina_array_clean(&e->render_objects);
 
-	     evas_array_clean(&e->restack_objects);
-	     evas_array_clean(&e->delete_objects);
+	     eina_array_clean(&e->restack_objects);
+	     eina_array_clean(&e->delete_objects);
 
 	     e->invalidate = 1;
 
@@ -315,23 +298,24 @@ _evas_render_check_pending_objects(Evas_Array *pending_objects, Evas *e)
      }
 }
 
-Evas_Bool pending_change(void *data, void *gdata)
+Eina_Bool pending_change(void *data, __UNUSED__ void *gdata)
 {
    Evas_Object *obj;
 
    obj = data;
-   if (obj->delete_me) return 0;
+   if (obj->delete_me) return EINA_FALSE;
    if (!obj->layer) obj->changed = 0;
-   return obj->changed;
+   return obj->changed ? EINA_TRUE : EINA_FALSE;
 }
 
-static Evas_List *
+static Eina_List *
 evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char do_draw)
 {
-   Evas_List *updates = NULL;
-   Evas_List *ll;
+   Eina_List *updates = NULL;
+   Eina_List *ll;
    void *surface;
    Evas_Bool clean_them = 0;
+   Evas_Rectangle *r;
    int ux, uy, uw, uh;
    int cx, cy, cw, ch;
    unsigned int i, j;
@@ -358,7 +342,7 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
      {
 	Evas_Object *obj;
 
-	obj = _evas_array_get(&e->restack_objects, i);
+	obj = eina_array_data_get(&e->restack_objects, i);
 	obj->func->render_pre(obj);
 	e->engine.func->output_redraws_rect_add(e->engine.data.output,
 						obj->prev.cache.clip.x,
@@ -371,17 +355,13 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 						obj->cur.cache.clip.w,
 						obj->cur.cache.clip.h);
      }
-   evas_array_clean(&e->restack_objects);
+   eina_array_clean(&e->restack_objects);
    /* phase 3. add exposes */
-   while (e->damages)
+   EINA_LIST_FREE(e->damages, r)
      {
-	Evas_Rectangle *r;
-
-	r = e->damages->data;
-	e->damages = evas_list_remove(e->damages, r);
 	e->engine.func->output_redraws_rect_add(e->engine.data.output,
 					       r->x, r->y, r->w, r->h);
-	free(r);
+	eina_mempool_free(_evas_rectangle_mp, r);
      }
    /* phase 4. output & viewport changes */
    if (e->viewport.changed)
@@ -403,29 +383,26 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 	printf("EVAS: error: viewport size != output size!\n");
      }
    /* phase 5. add obscures */
-   for (ll = e->obscures; ll; ll = ll->next)
-     {
-	Evas_Rectangle *r;
-
-	r = ll->data;
-	e->engine.func->output_redraws_rect_del(e->engine.data.output,
+   EINA_LIST_FOREACH(e->obscures, ll, r)
+        e->engine.func->output_redraws_rect_del(e->engine.data.output,
 					       r->x, r->y, r->w, r->h);
-     }
    /* build obscure objects list of active objects that obscure */
    for (i = 0; i < e->active_objects.count; ++i)
      {
 	Evas_Object *obj;
 
-	obj = _evas_array_get(&e->active_objects, i);
-	if (UNLIKELY(evas_object_is_opaque(obj) &&
+	obj = eina_array_data_get(&e->active_objects, i);
+	if (UNLIKELY((evas_object_is_opaque(obj) ||
+                      ((obj->func->has_opaque_rect) &&
+                       (obj->func->has_opaque_rect(obj)))) &&
                      evas_object_is_visible(obj) &&
                      (!obj->clip.clipees) &&
                      (obj->cur.visible) &&
                      (!obj->delete_me) &&
                      (obj->cur.cache.clip.visible) &&
                      (!obj->smart.smart)))
-/* 	  obscuring_objects = evas_list_append(obscuring_objects, obj); */
-	  _evas_array_append(&e->obscuring_objects, obj);
+/* 	  obscuring_objects = eina_list_append(obscuring_objects, obj); */
+	  eina_array_push(&e->obscuring_objects, obj);
      }
    /* save this list */
 /*    obscuring_objects_orig = obscuring_objects; */
@@ -446,12 +423,9 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 	       {
 		  Evas_Rectangle *rect;
 
-		  rect = malloc(sizeof(Evas_Rectangle));
+		  NEW_RECT(rect, ux, uy, uw, uh);
 		  if (rect)
-		    {
-		       rect->x = ux; rect->y = uy; rect->w = uw; rect->h = uh;
-		       updates = evas_list_append(updates, rect);
-		    }
+		    updates = eina_list_append(updates, rect);
 	       }
 	     off_x = cx - ux;
 	     off_y = cy - uy;
@@ -460,16 +434,16 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 	       {
 		  Evas_Object *obj;
 
-		  obj = (Evas_Object *) _evas_array_get(&e->obscuring_objects, i);
+		  obj = (Evas_Object *) eina_array_data_get(&e->obscuring_objects, i);
 		  if (evas_object_is_in_output_rect(obj, ux, uy, uw, uh))
-		    _evas_array_append(&e->temporary_objects, obj);
+		    eina_array_push(&e->temporary_objects, obj);
 	       }
 	     /* render all object that intersect with rect */
              for (i = 0; i < e->active_objects.count; ++i)
 	       {
 		  Evas_Object *obj;
 
-		  obj = _evas_array_get(&e->active_objects, i);
+		  obj = eina_array_data_get(&e->active_objects, i);
 
 		  /* if it's in our outpout rect and it doesn't clip anything */
 		  if (evas_object_is_in_output_rect(obj, ux, uy, uw, uh) &&
@@ -483,7 +457,7 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 		       int x, y, w, h;
 
 		       if ((e->temporary_objects.count > offset) &&
-			   (_evas_array_get(&e->temporary_objects, offset) == obj))
+			   (eina_array_data_get(&e->temporary_objects, offset) == obj))
 			 offset++;
 		       x = cx; y = cy; w = cw; h = ch;
 		       RECTS_CLIP_TO_RECT(x, y, w, h,
@@ -493,21 +467,6 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 					  obj->cur.cache.clip.h);
 		       if ((w > 0) && (h > 0))
 			 {
-///		       printf("CLIP: %p | %i %i, %ix%i | %p %i %i %ix%i\n",
-///			      obj,
-///			      x, y, w, h,
-///			      obj->cur.clipper,
-///			      obj->cur.cache.clip.x + off_x,
-///			      obj->cur.cache.clip.y + off_y,
-///			      obj->cur.cache.clip.w,
-///			      obj->cur.cache.clip.h
-///			      );
-///			    if (((obj->cur.cache.clip.x + off_x) == 0) &&
-///				((obj->cur.cache.clip.w) == 960))
-///			      {
-///				 abort();
-///			      }
-
 			    e->engine.func->context_clip_set(e->engine.data.output,
 							     e->engine.data.context,
 							     x, y, w, h);
@@ -516,13 +475,38 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 			      {
 				 Evas_Object *obj2;
 
-				 obj2 = (Evas_Object *) _evas_array_get(&e->temporary_objects, j);
-				 e->engine.func->context_cutout_add(e->engine.data.output,
-								    e->engine.data.context,
-								    obj2->cur.cache.clip.x + off_x,
-								    obj2->cur.cache.clip.y + off_y,
-								    obj2->cur.cache.clip.w,
-								    obj2->cur.cache.clip.h);
+				 obj2 = (Evas_Object *) eina_array_data_get(&e->temporary_objects, j);
+                                 if (evas_object_is_opaque(obj2))
+                                   e->engine.func->context_cutout_add(e->engine.data.output,
+                                                                      e->engine.data.context,
+                                                                      obj2->cur.cache.clip.x + off_x,
+                                                                      obj2->cur.cache.clip.y + off_y,
+                                                                      obj2->cur.cache.clip.w,
+                                                                      obj2->cur.cache.clip.h);
+                                 else
+                                   {
+                                      if (obj2->func->get_opaque_rect)
+                                        {
+                                           Evas_Coord obx, oby, obw, obh;
+                                           
+                                           obj2->func->get_opaque_rect
+                                             (obj2, &obx, &oby, &obw, &obh);
+                                           if ((obw > 0) && (obh > 0))
+                                             {
+                                                obx += off_x;
+                                                oby += off_y;
+                                                RECTS_CLIP_TO_RECT(obx, oby, obw, obh,
+                                                                   obj2->cur.cache.clip.x + off_x,
+                                                                   obj2->cur.cache.clip.y + off_y,
+                                                                   obj2->cur.cache.clip.w,
+                                                                   obj2->cur.cache.clip.h);
+                                                e->engine.func->context_cutout_add(e->engine.data.output,
+                                                                                   e->engine.data.context,
+                                                                                   obx, oby,
+                                                                                   obw, obh);
+                                             }
+                                        }
+                                   }
 			      }
 #endif
 			    obj->func->render(obj,
@@ -540,7 +524,7 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 							     surface,
 							     ux, uy, uw, uh);
 	     /* free obscuring objects list */
-	     evas_array_clean(&e->temporary_objects);
+	     eina_array_clean(&e->temporary_objects);
 	  }
 	/* flush redraws */
 	e->engine.func->output_flush(e->engine.data.output);
@@ -552,7 +536,7 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
      {
 	Evas_Object *obj;
 
-	obj = _evas_array_get(&e->active_objects, i);
+	obj = eina_array_data_get(&e->active_objects, i);
 	obj->pre_render_done = 0;
 	if ((obj->changed) && (do_draw))
 	  {
@@ -563,27 +547,27 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
 /* moved to other pre-process phase 1
 	if (obj->delete_me == 2)
 	  {
-	     delete_objects = evas_list_append(delete_objects, obj);
+	     delete_objects = eina_list_append(delete_objects, obj);
 	  }
 	else if (obj->delete_me != 0) obj->delete_me++;
  */
      }
    /* free our obscuring object list */
-   evas_array_clean(&e->obscuring_objects);
+   eina_array_clean(&e->obscuring_objects);
 
    /* If some object are still marked as changed, do not remove
       them from the pending list. */
-   evas_array_remove(&e->pending_objects, pending_change, NULL);
+   eina_array_remove(&e->pending_objects, pending_change, NULL);
 
    /* delete all objects flagged for deletion now */
    for (i = 0; i < e->delete_objects.count; ++i)
      {
 	Evas_Object *obj;
 
-	obj = _evas_array_get(&e->delete_objects, i);
+	obj = eina_array_data_get(&e->delete_objects, i);
 	evas_object_free(obj, 1);
      }
-   evas_array_clean(&e->delete_objects);
+   eina_array_clean(&e->delete_objects);
 
    e->changed = 0;
    e->viewport.changed = 0;
@@ -593,8 +577,8 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
    /* If their are some object to restack or some object to delete, it's useless to keep the render object list around. */
    if (clean_them)
      {
-	evas_array_clean(&e->active_objects);
-	evas_array_clean(&e->render_objects);
+	eina_array_clean(&e->active_objects);
+	eina_array_clean(&e->render_objects);
 
 	e->invalidate = 1;
      }
@@ -611,13 +595,12 @@ evas_render_updates_internal(Evas *e, unsigned char make_updates, unsigned char
  *
  */
 EAPI void
-evas_render_updates_free(Evas_List *updates)
+evas_render_updates_free(Eina_List *updates)
 {
-   while (updates)
-     {
-	free(updates->data);
-	updates = evas_list_remove(updates, updates->data);
-     }
+   Evas_Rectangle *r;
+
+   EINA_LIST_FREE(updates, r)
+     eina_mempool_free(_evas_rectangle_mp, r);
 }
 
 /**
@@ -626,7 +609,7 @@ evas_render_updates_free(Evas_List *updates)
  * FIXME: To be fixed.
  *
  */
-EAPI Evas_List *
+EAPI Eina_List *
 evas_render_updates(Evas *e)
 {
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
@@ -688,10 +671,11 @@ evas_render_idle_flush(Evas *e)
        (e->engine.data.output))
      e->engine.func->output_idle_flush(e->engine.data.output);
 
-   evas_array_flush(&e->delete_objects);
-   evas_array_flush(&e->active_objects);
-   evas_array_flush(&e->restack_objects);
-   evas_array_flush(&e->render_objects);
+   eina_array_flush(&e->delete_objects);
+   eina_array_flush(&e->active_objects);
+   eina_array_flush(&e->restack_objects);
+   eina_array_flush(&e->render_objects);
+   eina_array_flush(&e->clip_changes);
 
    e->invalidate = 1;
 }
@@ -703,11 +687,11 @@ evas_render_invalidate(Evas *e)
    return;
    MAGIC_CHECK_END();
 
-   evas_array_clean(&e->active_objects);
-   evas_array_clean(&e->render_objects);
+   eina_array_clean(&e->active_objects);
+   eina_array_clean(&e->render_objects);
 
-   evas_array_flush(&e->restack_objects);
-   evas_array_flush(&e->delete_objects);
+   eina_array_flush(&e->restack_objects);
+   eina_array_flush(&e->delete_objects);
 
    e->invalidate = 1;
 }
@@ -727,7 +711,7 @@ evas_render_object_recalc(Evas_Object *obj)
 	if (!e || e->cleanup) return ;
 
 	if (!obj->changed)
-	  _evas_array_append(&e->pending_objects, obj);
+	  eina_array_push(&e->pending_objects, obj);
 	obj->changed = 1;
      }
 }
