@@ -8,9 +8,10 @@
 #include "evas_mmx.h"
 #endif
 
+#ifndef _WIN32
+#include <signal.h>
 #include <setjmp.h>
 
-#ifndef _WIN32
 static sigjmp_buf detect_buf;
 #endif
 
@@ -21,13 +22,13 @@ static void evas_common_cpu_catch_ill(int sig);
 static void evas_common_cpu_catch_segv(int sig);
 
 static void
-evas_common_cpu_catch_ill(int sig)
+evas_common_cpu_catch_ill(int sig __UNUSED__)
 {
    siglongjmp(detect_buf, 1);
 }
 
 static void
-evas_common_cpu_catch_segv(int sig)
+evas_common_cpu_catch_segv(int sig __UNUSED__)
 {
    siglongjmp(detect_buf, 1);
 }
@@ -75,6 +76,18 @@ evas_common_cpu_altivec_test(void)
 #endif /* __POWERPC__ */
 }
 
+void
+evas_common_cpu_neon_test(void)
+{
+#if defined(__ARM_ARCH__) && (__ARM_ARCH__ >= 70)
+#ifdef BUILD_NEON
+   asm volatile (
+                 "vqadd.u8 d0, d1, d0\n"
+                 );
+#endif
+#endif
+}
+
 void
 evas_common_cpu_vis_test(void)
 {
@@ -127,13 +140,19 @@ evas_common_cpu_init(void)
    cpu_feature_mask |= CPU_FEATURE_MMX *
      evas_common_cpu_feature_test(evas_common_cpu_mmx_test);
    evas_common_cpu_end_opt();
+   if (getenv("EVAS_CPU_NO_MMX"))
+     cpu_feature_mask &= ~CPU_FEATURE_MMX;
    cpu_feature_mask |= CPU_FEATURE_MMX2 *
      evas_common_cpu_feature_test(evas_common_cpu_mmx2_test);
    evas_common_cpu_end_opt();
+   if (getenv("EVAS_CPU_NO_MMX2"))
+     cpu_feature_mask &= ~CPU_FEATURE_MMX2;
 #ifdef BUILD_SSE
    cpu_feature_mask |= CPU_FEATURE_SSE *
      evas_common_cpu_feature_test(evas_common_cpu_sse_test);
    evas_common_cpu_end_opt();
+   if (getenv("EVAS_CPU_NO_SSE"))
+     cpu_feature_mask &= ~CPU_FEATURE_SSE;
 #endif /* BUILD_SSE */
 #endif /* BUILD_MMX */
 #ifdef __POWERPC__
@@ -141,13 +160,24 @@ evas_common_cpu_init(void)
    cpu_feature_mask |= CPU_FEATURE_ALTIVEC *
      evas_common_cpu_feature_test(evas_common_cpu_altivec_test);
    evas_common_cpu_end_opt();
+   if (getenv("EVAS_CPU_NO_ALTIVEC"))
+     cpu_feature_mask &= ~CPU_FEATURE_ALTIVEC;
 #endif /* __VEC__ */
 #endif /* __POWERPC__ */
 #ifdef __SPARC__
    cpu_feature_mask |= CPU_FEATURE_VIS *
      evas_common_cpu_feature_test(evas_common_cpu_vis_test);
    evas_common_cpu_end_opt();
+   if (getenv("EVAS_CPU_NO_VIS"))
+     cpu_feature_mask &= ~CPU_FEATURE_VIS;
 #endif /* __SPARC__ */
+#if defined(__ARM_ARCH__) && (__ARM_ARCH__ >= 70)
+#ifdef BUILD_NEON
+   cpu_feature_mask |= CPU_FEATURE_NEON *
+     evas_common_cpu_feature_test(evas_common_cpu_neon_test);
+   evas_common_cpu_end_opt();
+#endif
+#endif
 }
 
 int
