@@ -17,9 +17,10 @@ EAPI int               evas_common_font_max_ascent_get       (RGBA_Font *fn);
 EAPI int               evas_common_font_max_descent_get      (RGBA_Font *fn);
 EAPI int               evas_common_font_get_line_advance     (RGBA_Font *fn);
 
-EAPI int               evas_common_font_utf8_get_next        (unsigned char *buf, int *iindex);
-EAPI int               evas_common_font_utf8_get_prev        (unsigned char *buf, int *iindex);
-EAPI int               evas_common_font_utf8_get_last        (unsigned char *buf, int buflen);
+EAPI int               evas_common_font_utf8_get_next        (const unsigned char *buf, int *iindex);
+EAPI int               evas_common_font_utf8_get_prev        (const unsigned char *buf, int *iindex);
+EAPI int               evas_common_font_utf8_get_last        (const unsigned char *buf, int buflen);
+EAPI int               evas_common_font_utf8_get_len         (const unsigned char *buf);
 
 /* draw */
 
@@ -59,11 +60,12 @@ EAPI RGBA_Font_Int    *evas_common_font_int_find             (const char *name,
 
 /* query */
 
+EAPI int               evas_common_font_query_kerning        (RGBA_Font_Int* fi, FT_UInt prev, FT_UInt index, int* kerning);
 EAPI void              evas_common_font_query_size           (RGBA_Font *fn, const char *text, int *w, int *h);
 EAPI int               evas_common_font_query_inset          (RGBA_Font *fn, const char *text);
 EAPI void              evas_common_font_query_advance        (RGBA_Font *fn, const char *text, int *h_adv, int *v_adv);
 EAPI int               evas_common_font_query_char_coords    (RGBA_Font *fn, const char *text, int pos, int *cx, int *cy, int *cw, int *ch);
 EAPI int               evas_common_font_query_text_at_pos    (RGBA_Font *fn, const char *text, int x, int y, int *cx, int *cy, int *cw, int *ch);
-
+EAPI int               evas_common_font_query_last_up_to_pos (RGBA_Font *fn, const char *text, int x, int y);
 
 #endif /* _EVAS_FONT_H */
