@@ -9,11 +9,11 @@ extern FT_Library         evas_ft_lib;
 
 static int                font_cache_usage = 0;
 static int                font_cache = 0;
-static Evas_Object_List * fonts_src = NULL;
-static Evas_Object_List * fonts = NULL;
+static Eina_Inlist * fonts_src = NULL;
+static Eina_Inlist * fonts = NULL;
 
-static Evas_Bool font_modify_cache_cb(const Evas_Hash *hash, const char *key, void *data, void *fdata);
-static Evas_Bool font_flush_free_glyph_cb(const Evas_Hash *hash, const char *key, void *data, void *fdata);
+static Evas_Bool font_modify_cache_cb(const Eina_Hash *hash, const void *key, void *data, void *fdata);
+static Evas_Bool font_flush_free_glyph_cb(const Eina_Hash *hash, const void *key, void *data, void *fdata);
 
 EAPI RGBA_Font_Source *
 evas_common_font_source_memory_load(const char *name, const void *data, int data_size)
@@ -33,12 +33,12 @@ evas_common_font_source_memory_load(const char *name, const void *data, int data
 	free(fs);
 	return NULL;
      }
-   fs->name = evas_stringshare_add(name);
+   fs->name = eina_stringshare_add(name);
    fs->file = NULL;
    error = FT_Select_Charmap(fs->ft.face, ft_encoding_unicode);
    fs->ft.orig_upem = fs->ft.face->units_per_EM;
    fs->references = 1;
-   fonts_src = evas_object_list_prepend(fonts_src, fs);
+   fonts_src = eina_inlist_prepend(fonts_src, EINA_INLIST_GET(fs));
    return fs;
 }
 
@@ -54,13 +54,13 @@ evas_common_font_source_load(const char *name)
    fs->current_size = 0;
    fs->ft.face = NULL;
 
-   fs->name = evas_stringshare_add(name);
+   fs->name = eina_stringshare_add(name);
    fs->file = fs->name;
 
    fs->ft.orig_upem = 0;
 
    fs->references = 1;
-   fonts_src = evas_object_list_prepend(fonts_src, fs);
+   fonts_src = eina_inlist_prepend(fonts_src, EINA_INLIST_GET(fs));
    return fs;
 }
 
@@ -91,19 +91,15 @@ evas_common_font_source_load_complete(RGBA_Font_Source *fs)
 EAPI RGBA_Font_Source *
 evas_common_font_source_find(const char *name)
 {
-   Evas_Object_List *l;
+   RGBA_Font_Source *fs;
 
    if (!name) return NULL;
-   for (l = fonts_src; l; l = l->next)
+   EINA_INLIST_FOREACH(fonts_src, fs)
      {
-	RGBA_Font_Source *fs;
-
-	fs = (RGBA_Font_Source *)l;
 	if ((fs->name) && (!strcmp(name, fs->name)))
 	  {
 	     fs->references++;
-	     fonts_src = evas_object_list_remove(fonts_src, fs);
-	     fonts_src = evas_object_list_prepend(fonts_src, fs);
+	     fonts_src = eina_inlist_demote(fonts_src, EINA_INLIST_GET(fs));
 	     return fs;
 	  }
      }
@@ -116,17 +112,17 @@ evas_common_font_source_free(RGBA_Font_Source *fs)
    fs->references--;
    if (fs->references > 0) return;
 
-   fonts_src = evas_object_list_remove(fonts_src, fs);
+   fonts_src = eina_inlist_remove(fonts_src, EINA_INLIST_GET(fs));
    FT_Done_Face(fs->ft.face);
    if (fs->charmap) evas_array_hash_free(fs->charmap);
-   if (fs->name) evas_stringshare_del(fs->name);
+   if (fs->name) eina_stringshare_del(fs->name);
    free(fs);
 }
 
 EAPI void
 evas_common_font_size_use(RGBA_Font *fn)
 {
-   Evas_List *l;
+   Eina_List *l;
 
    for (l = fn->fonts; l; l = l->next)
      {
@@ -142,6 +138,63 @@ evas_common_font_size_use(RGBA_Font *fn)
      }
 }
 
+static unsigned int
+_evas_common_font_int_length(const void *key __UNUSED__)
+{
+   return sizeof (int);
+}
+
+static unsigned int
+_evas_common_font_double_int_length(const void *key __UNUSED__)
+{
+   return sizeof (int) * 2;
+}
+
+static int
+_evas_common_font_int_cmp(const int *key1, __UNUSED__ int key1_length,
+			  const int *key2, __UNUSED__ int key2_length)
+{
+   return *key1 - *key2;
+}
+
+static int
+_evas_common_font_double_int_cmp(const int *key1, __UNUSED__ int key1_length,
+				 const int *key2, __UNUSED__ int key2_length)
+{
+   if (key1[0] - key2[0] == 0)
+     return key1[1] - key2[1];
+   return key1[0] - key2[0];
+}
+
+static int
+_evas_common_font_double_int_hash(const unsigned int key[2], int key_length)
+{
+   int tmp;
+
+   tmp = eina_hash_int32(&key[0], key_length);
+   tmp ^= eina_hash_int32(&key[1], key_length);
+
+   return tmp;
+}
+
+static void
+_evas_commont_font_int_cache_init(RGBA_Font_Int *fi)
+{
+   /* Add some font kerning cache. */
+   fi->indexes = eina_hash_new(EINA_KEY_LENGTH(_evas_common_font_int_length),
+			       EINA_KEY_CMP(_evas_common_font_int_cmp),
+			       EINA_KEY_HASH(eina_hash_int32),
+			       free, 3);
+   fi->kerning = eina_hash_new(EINA_KEY_LENGTH(_evas_common_font_double_int_length),
+			       EINA_KEY_CMP(_evas_common_font_double_int_cmp),
+			       EINA_KEY_HASH(_evas_common_font_double_int_hash),
+			       free, 3);
+#ifdef HAVE_PTHREAD
+   pthread_mutex_init(&fi->ft_mutex, NULL);
+#endif
+}
+
+
 EAPI RGBA_Font_Int *
 evas_common_font_int_memory_load(const char *name, int size, const void *data, int data_size)
 {
@@ -165,6 +218,8 @@ evas_common_font_int_memory_load(const char *name, int size, const void *data, i
 
    fi->size = size;
 
+   _evas_commont_font_int_cache_init(fi);
+
    fi = evas_common_font_int_load_init(fi);
    evas_common_font_int_load_complete(fi);
 
@@ -194,6 +249,8 @@ evas_common_font_int_load(const char *name, int size)
 
    fi->size = size;
 
+   _evas_commont_font_int_cache_init(fi);
+
    return evas_common_font_int_load_init(fi);
 }
 
@@ -201,10 +258,11 @@ EAPI RGBA_Font_Int *
 evas_common_font_int_load_init(RGBA_Font_Int *fi)
 {
    fi->ft.size = NULL;
-   fi->glyphs = NULL;
+   fi->glyphs = eina_hash_int32_new(NULL);
    fi->usage = 0;
    fi->references = 1;
-   fonts = evas_object_list_prepend(fonts, fi);
+   fonts = eina_inlist_prepend(fonts, EINA_INLIST_GET(fi));
+
    return fi;
 }
 
@@ -274,7 +332,7 @@ evas_common_font_memory_load(const char *name, int size, const void *data, int d
 	free(fi);
 	return NULL;
      }
-   fn->fonts = evas_list_append(fn->fonts, fi);
+   fn->fonts = eina_list_append(fn->fonts, fi);
    fn->hinting = FONT_BYTECODE_HINT;
    fi->hinting = fn->hinting;
    fn->references = 1;
@@ -321,7 +379,7 @@ evas_common_font_load(const char *name, int size)
 	  }
 	return NULL;
      }
-   fn->fonts = evas_list_append(fn->fonts, fi);
+   fn->fonts = eina_list_append(fn->fonts, fi);
    fn->hinting = FONT_BYTECODE_HINT;
    fi->hinting = fn->hinting;
    fn->references = 1;
@@ -339,7 +397,7 @@ evas_common_font_add(RGBA_Font *fn, const char *name, int size)
    fi = evas_common_font_int_load(name, size);
    if (fi)
      {
-	fn->fonts = evas_list_append(fn->fonts, fi);
+	fn->fonts = eina_list_append(fn->fonts, fi);
 	fi->hinting = fn->hinting;
 	return fn;
      }
@@ -356,7 +414,7 @@ evas_common_font_memory_add(RGBA_Font *fn, const char *name, int size, const voi
    fi = evas_common_font_int_memory_load(name, size, data, data_size);
    if (fi)
      {
-	fn->fonts = evas_list_append(fn->fonts, fi);
+	fn->fonts = eina_list_append(fn->fonts, fi);
 	fi->hinting = fn->hinting;
 	return fn;
      }
@@ -366,17 +424,15 @@ evas_common_font_memory_add(RGBA_Font *fn, const char *name, int size, const voi
 EAPI void
 evas_common_font_free(RGBA_Font *fn)
 {
-   Evas_List *l;
+   Eina_List *l;
+   RGBA_Font_Int *fi;
 
    if (!fn)
       return;
    fn->references--;
    if (fn->references > 0) return;
-   for (l = fn->fonts; l; l = l->next)
+   EINA_LIST_FOREACH(fn->fonts, l, fi)
      {
-	RGBA_Font_Int *fi;
-
-	fi = l->data;
 	fi->references--;
 	if (fi->references == 0)
 	  {
@@ -384,7 +440,7 @@ evas_common_font_free(RGBA_Font *fn)
 	     evas_common_font_flush();
 	  }
      }
-   evas_list_free(fn->fonts);
+   eina_list_free(fn->fonts);
    LKD(fn->lock);
    free(fn);
 }
@@ -392,18 +448,14 @@ evas_common_font_free(RGBA_Font *fn)
 EAPI void
 evas_common_font_hinting_set(RGBA_Font *fn, Font_Hint_Flags hinting)
 {
-   Evas_List *l;
+   Eina_List *l;
+   RGBA_Font_Int *fi;
 
    if (!fn)
      return;
    fn->hinting = hinting;
-   for (l = fn->fonts; l; l = l->next)
-     {
-	RGBA_Font_Int *fi;
-
-	fi = l->data;
-	fi->hinting = fn->hinting;
-     }
+   EINA_LIST_FOREACH(fn->fonts, l, fi)
+     fi->hinting = fn->hinting;
 }
 
 EAPI Evas_Bool
@@ -477,8 +529,8 @@ evas_common_font_memory_hinting_add(RGBA_Font *fn, const char *name, int size, c
    return fn;
 }
 
-static Evas_Bool
-font_modify_cache_cb(const Evas_Hash *hash, const char *key, void *data, void *fdata)
+static Eina_Bool
+font_modify_cache_cb(const Eina_Hash *hash, const void *key, void *data, void *fdata)
 {
    int *dir;
    RGBA_Font_Glyph *fg;
@@ -487,7 +539,7 @@ font_modify_cache_cb(const Evas_Hash *hash, const char *key, void *data, void *f
    dir = fdata;
    font_cache_usage += (*dir) *
      ((fg->glyph_out->bitmap.width * fg->glyph_out->bitmap.rows) +
-      sizeof(RGBA_Font_Glyph) + sizeof(Evas_List) + 400); /* fudge values */
+      sizeof(RGBA_Font_Glyph) + sizeof(Eina_List) + 400); /* fudge values */
    return 1;
    hash = 0;
    key = 0;
@@ -502,8 +554,8 @@ evas_common_font_int_modify_cache_by(RGBA_Font_Int *fi, int dir)
 {
    int sz_hash = 0;
 
-   if (fi->glyphs) sz_hash = sizeof(Evas_Hash);
-   evas_hash_foreach(fi->glyphs, font_modify_cache_cb, &dir);
+   if (fi->glyphs) sz_hash = eina_hash_population(fi->glyphs);
+   eina_hash_foreach(fi->glyphs, font_modify_cache_cb, &dir);
    font_cache_usage += dir * (sizeof(RGBA_Font) + sz_hash +
 			      sizeof(FT_FaceRec) + 16384); /* fudge values */
 }
@@ -528,8 +580,8 @@ evas_common_font_flush(void)
    while (font_cache_usage > font_cache) evas_common_font_flush_last();
 }
 
-static Evas_Bool
-font_flush_free_glyph_cb(const Evas_Hash *hash, const char *key, void *data, void *fdata)
+static Eina_Bool
+font_flush_free_glyph_cb(const Eina_Hash *hash, const void *key, void *data, void *fdata)
 {
    RGBA_Font_Glyph *fg;
 
@@ -550,25 +602,29 @@ font_flush_free_glyph_cb(const Evas_Hash *hash, const char *key, void *data, voi
 EAPI void
 evas_common_font_flush_last(void)
 {
-   Evas_Object_List *l;
+   RGBA_Font_Int *fi_tmp;
    RGBA_Font_Int *fi = NULL;
 
-   for (l = fonts; l; l = l->next)
+   EINA_INLIST_FOREACH(fonts, fi_tmp)
      {
-	RGBA_Font_Int *fi_tmp;
-
-	fi_tmp = (RGBA_Font_Int *)l;
 	if (fi_tmp->references == 0) fi = fi_tmp;
      }
    if (!fi) return;
 
    FT_Done_Size(fi->ft.size);
 
-   fonts = evas_object_list_remove(fonts, fi);
+   fonts = eina_inlist_remove(fonts, EINA_INLIST_GET(fi));
    evas_common_font_int_modify_cache_by(fi, -1);
 
-   evas_hash_foreach(fi->glyphs, font_flush_free_glyph_cb, NULL);
-   evas_hash_free(fi->glyphs);
+   eina_hash_foreach(fi->glyphs, font_flush_free_glyph_cb, NULL);
+   eina_hash_free(fi->glyphs);
+
+   eina_hash_free(fi->kerning);
+   eina_hash_free(fi->indexes);
+
+#ifdef HAVE_PTHREAD
+   pthread_mutex_destroy(&fi->ft_mutex);
+#endif
 
    evas_common_font_source_free(fi->src);
 
@@ -578,19 +634,15 @@ evas_common_font_flush_last(void)
 EAPI RGBA_Font_Int *
 evas_common_font_int_find(const char *name, int size)
 {
-   Evas_Object_List *l;
+   RGBA_Font_Int *fi;
 
-   for (l = fonts; l; l = l->next)
+   EINA_INLIST_FOREACH(fonts, fi)
      {
-	RGBA_Font_Int *fi;
-
-	fi = (RGBA_Font_Int *)l;
 	if ((fi->size == size) && (!strcmp(name, fi->src->name)))
 	  {
 	     if (fi->references == 0) evas_common_font_int_modify_cache_by(fi, -1);
 	     fi->references++;
-	     fonts = evas_object_list_remove(fonts, fi);
-	     fonts = evas_object_list_prepend(fonts, fi);
+	     fonts = eina_inlist_promote(fonts, EINA_INLIST_GET(fi));
 	     return fi;
 	  }
      }
