@@ -9,17 +9,85 @@
 # include "config.h"  /* so that EAPI in Evas.h is correctly defined */
 #endif
 
+#include <Eina.h>
 #include "Evas.h"
 
 /*****************************************************************************/
 
 #include "evas_options.h"
 
+#if defined(__ARM_ARCH_3M__)
+# define __ARM_ARCH__ 40
+#endif
+#if defined(__ARM_ARCH_4__)
+# define __ARM_ARCH__ 40
+#endif
+#if defined(__ARM_ARCH_4T__)
+# define __ARM_ARCH__ 41
+#endif
+
+#if defined(__ARM_ARCH_5__)
+# define __ARM_ARCH__ 50
+#endif
+#if defined(__ARM_ARCH_5T__)
+# define __ARM_ARCH__ 51
+#endif
+#if defined(__ARM_ARCH_5E__)
+# define __ARM_ARCH__ 52
+#endif
+#if defined(__ARM_ARCH_5TE__)
+# define __ARM_ARCH__ 53
+#endif
+#if defined(__ARM_ARCH_5TEJ__)
+# define __ARM_ARCH__ 54
+#endif
+
+#if defined(__ARM_ARCH_6__)
+# define __ARM_ARCH__ 60
+#endif
+#if defined(__ARM_ARCH_6J__)
+# define __ARM_ARCH__ 61
+#endif
+#if defined(__ARM_ARCH_6K__)
+# define __ARM_ARCH__ 62
+#endif
+#if defined(__ARM_ARCH_6Z__)
+# define __ARM_ARCH__ 63
+#endif
+#if defined(__ARM_ARCH_6ZK__)
+# define __ARM_ARCH__ 64
+#endif
+#if defined(__ARM_ARCH_6T2__)
+# define __ARM_ARCH__ 65
+#endif
+
+#if defined(__ARM_ARCH_7__)
+# define __ARM_ARCH__ 70
+#endif
+#if defined(__ARM_ARCH_7A__)
+# define __ARM_ARCH__ 71
+#endif
+#if defined(__ARM_ARCH_7R__)
+# define __ARM_ARCH__ 72
+#endif
+#if defined(__ARM_ARCH_7M__)
+# define __ARM_ARCH__ 73
+#endif
+
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
 
+#ifndef BUILD_PTHREAD
+#undef BUILD_PIPE_RENDER
+#endif
+
 #ifdef BUILD_PTHREAD
+
+#ifndef __USE_GNU
+#define __USE_GNU
+#endif
+
 # include <pthread.h>
 # include <sched.h>
 # define LK(x)  pthread_mutex_t x
@@ -49,8 +117,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <signal.h>
-#include <setjmp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -123,6 +189,95 @@ void *alloca (size_t);
 
 /*****************************************************************************/
 
+#if defined(__ARM_ARCH_3M__)
+# define __ARM_ARCH__ 40
+#endif
+#if defined(__ARM_ARCH_4__)
+# define __ARM_ARCH__ 40
+#endif
+#if defined(__ARM_ARCH_4T__)
+# define __ARM_ARCH__ 41
+#endif
+
+#if defined(__ARM_ARCH_5__)
+# define __ARM_ARCH__ 50
+#endif
+#if defined(__ARM_ARCH_5T__)
+# define __ARM_ARCH__ 51
+#endif
+#if defined(__ARM_ARCH_5E__)
+# define __ARM_ARCH__ 52
+#endif
+#if defined(__ARM_ARCH_5TE__)
+# define __ARM_ARCH__ 53
+#endif
+#if defined(__ARM_ARCH_5TEJ__)
+# define __ARM_ARCH__ 54
+#endif
+
+#if defined(__ARM_ARCH_6__)
+# define __ARM_ARCH__ 60
+#endif
+#if defined(__ARM_ARCH_6J__)
+# define __ARM_ARCH__ 61
+#endif
+#if defined(__ARM_ARCH_6K__)
+# define __ARM_ARCH__ 62
+#endif
+#if defined(__ARM_ARCH_6Z__)
+# define __ARM_ARCH__ 63
+#endif
+#if defined(__ARM_ARCH_6ZK__)
+# define __ARM_ARCH__ 64
+#endif
+#if defined(__ARM_ARCH_6T2__)
+# define __ARM_ARCH__ 65
+#endif
+
+#if defined(__ARM_ARCH_7__)
+# define __ARM_ARCH__ 70
+#endif
+#if defined(__ARM_ARCH_7A__)
+# define __ARM_ARCH__ 71
+#endif
+#if defined(__ARM_ARCH_7R__)
+# define __ARM_ARCH__ 72
+#endif
+#if defined(__ARM_ARCH_7M__)
+# define __ARM_ARCH__ 73
+#endif
+
+#if defined(__ARM_ARCH__) && (__ARM_ARCH__ >= 52)
+/* tested on ARMv6 (arm1136j-s), Nokia N800 CPU */
+#define pld(addr, off)                                                  \
+   __asm__("pld [%[address], %[offset]]"::                              \
+           [address] "r" (addr), [offset] "i" (off))
+#else
+#define pld(addr, off)
+#endif /* __ARMEL__ */
+
+/*****************************************************************************/
+
+#define UNROLL2(op...) op op
+#define UNROLL4(op...) UNROLL2(op) UNROLL2(op)
+#define UNROLL8(op...) UNROLL4(op) UNROLL4(op)
+#define UNROLL16(op...) UNROLL8(op) UNROLL8(op)
+
+#define UNROLL8_PLD_WHILE(start, size, end, op)         \
+    pld(start, 0);                                      \
+    end = start + (size & ~7);                          \
+    while (start < end)                                 \
+        {                                               \
+            pld(start, 32);                             \
+            UNROLL8(op);                                \
+        }                                               \
+    end += (size & 7);                                  \
+    pld(start, 32);                                     \
+    while (start <  end)                                \
+        {                                               \
+        op;                                             \
+        }
+
 /*****************************************************************************/
 
 typedef unsigned long long		DATA64;
@@ -133,11 +288,14 @@ typedef unsigned char                   DATA8;
 typedef struct _Image_Entry             Image_Entry;
 typedef struct _Image_Entry_Flags	Image_Entry_Flags;
 typedef struct _Engine_Image_Entry      Engine_Image_Entry;
+typedef struct _Evas_Cache_Target       Evas_Cache_Target;
 
 typedef struct _RGBA_Image_Loadopts   RGBA_Image_Loadopts;
+#ifdef BUILD_PIPE_RENDER
 typedef struct _RGBA_Pipe_Op          RGBA_Pipe_Op;
 typedef struct _RGBA_Pipe             RGBA_Pipe;
 typedef struct _RGBA_Pipe_Thread_Info RGBA_Pipe_Thread_Info;
+#endif
 typedef struct _RGBA_Image            RGBA_Image;
 typedef struct _RGBA_Image_Span       RGBA_Image_Span;
 typedef struct _RGBA_Draw_Context     RGBA_Draw_Context;
@@ -230,7 +388,8 @@ typedef enum _CPU_Features
    CPU_FEATURE_SSE     = (1 << 2),
    CPU_FEATURE_ALTIVEC = (1 << 3),
    CPU_FEATURE_VIS     = (1 << 4),
-   CPU_FEATURE_VIS2    = (1 << 5)
+   CPU_FEATURE_VIS2    = (1 << 5),
+   CPU_FEATURE_NEON    = (1 << 6)
 } CPU_Features;
 
 typedef enum _Font_Hint_Flags
@@ -264,49 +423,63 @@ struct _Image_Entry_Flags
 #endif
 };
 
+struct _Evas_Cache_Target
+{
+  EINA_INLIST;
+  const void *target;
+};
+
 struct _Image_Entry
 {
-  Evas_Object_List       _list_data;
+   EINA_INLIST;
 
-  Evas_Cache_Image      *cache;
+   Evas_Cache_Image      *cache;
+   
+   const char            *cache_key;
+   
+   const char            *file;
+   const char            *key;
+   
+   Evas_Cache_Target     *targets;
+   
+   time_t                 timestamp;
+   time_t                 laststat;
+   
+   int                    references;
+   
+   unsigned char          scale;
+   
+   RGBA_Image_Loadopts    load_opts;
+   int                    space;
+   int                    w;
+   int                    h;
+   
+   struct
+     {
+        int		 w;
+        int		 h;
+     } allocated;
+   
+   struct
+     {
+        void		*module;
+        void		*loader;
+     } info;
+   
+#ifdef BUILD_ASYNC_PRELOAD
+   LK(lock);
+#endif
 
-  const char            *cache_key;
-
-  const char            *file;
-  const char            *key;
-
-  const void            *target;
-
-  time_t                 timestamp;
-  time_t                 laststat;
-
-  int                    references;
-
-  unsigned char          scale;
-
-  RGBA_Image_Loadopts    load_opts;
-  int                    space;
-  int                    w;
-  int                    h;
-
-  struct
-  {
-     int		 w;
-     int		 h;
-  } allocated;
-
-  struct
-  {
-     void		*module;
-     void		*loader;
-  } info;
-
-  Image_Entry_Flags      flags;
+   Image_Entry_Flags      flags;
+   Evas_Image_Scale_Hint  scale_hint;
+   void                  *data1, *data2;
+   int                    server_id;
+   int                    connect_num;
 };
 
 struct _Engine_Image_Entry
 {
-   Evas_Object_List              _list_data;
+   EINA_INLIST;
 
    /* Upper Engine data. */
    Image_Entry                  *src;
@@ -380,6 +553,7 @@ struct _RGBA_Draw_Context
    Evas_Bool anti_alias : 1;
 };
 
+#ifdef BUILD_PIPE_RENDER
 struct _RGBA_Pipe_Op
 {
    RGBA_Draw_Context         context;
@@ -422,7 +596,7 @@ struct _RGBA_Pipe_Op
 
 struct _RGBA_Pipe
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
    int               op_num;
    RGBA_Pipe_Op      op[PIPE_LEN];
 };
@@ -432,6 +606,7 @@ struct _RGBA_Pipe_Thread_Info
    RGBA_Image *im;
    int         x, y, w, h;
 };
+#endif
 
 struct _RGBA_Image
 {
@@ -448,7 +623,9 @@ struct _RGBA_Image
      } info;
 
    void                *extended_info;
+#ifdef BUILD_PIPE_RENDER
    RGBA_Pipe           *pipe;
+#endif   
    int                  ref;
 
 /*    unsigned char        scale; */
@@ -461,23 +638,32 @@ struct _RGBA_Image
    } cs;
 
    /* RGBA stuff */
-   struct
-   {
+   struct {
       DATA32            *data;
       Evas_Bool          no_free : 1;
    } image;
+   
+   struct {
+      LK(lock);
+      Eina_List *list;
+      unsigned long long orig_usage;
+      unsigned long long usage_count;
+      int populate_count;
+      unsigned long long newest_usage;
+      unsigned long long newest_usage_count;
+   } cache;
 };
 
 struct _RGBA_Gradient_Color_Stop
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
    int               r, g, b, a;
    int               dist;
 };
 
 struct _RGBA_Gradient_Alpha_Stop
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
    int               a;
    int               dist;
 };
@@ -495,13 +681,13 @@ struct _RGBA_Gradient
      } map;
 
    struct {
-	Evas_Object_List *stops;
+	Eina_Inlist *stops;
 	DATA32           *data;
 	int               nstops;
 	int               len;
    }  color;
    struct {
-	Evas_Object_List *stops;
+	Eina_Inlist *stops;
 	DATA8            *data;
 	int               nstops;
 	int               len;
@@ -543,7 +729,7 @@ struct _RGBA_Gradient_Type
 
 struct _RGBA_Gradient2_Color_Np_Stop
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
    int               r, g, b, a;
    float             pos;
    int               dist;
@@ -559,7 +745,7 @@ struct _RGBA_Gradient2
      } map;
 
    struct {
-	Evas_Object_List *stops;
+	Eina_Inlist *stops;
 	int               nstops;
 	DATA32           *cdata;
 	DATA8            *adata;
@@ -600,13 +786,13 @@ struct _RGBA_Gradient2_Type
 
 struct _RGBA_Polygon_Point
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
    int               x, y;
 };
 
 struct _RGBA_Font
 {
-   Evas_List *fonts;
+   Eina_List *fonts;
    Font_Hint_Flags hinting;
    int references;
    LK(lock);
@@ -614,7 +800,7 @@ struct _RGBA_Font
 
 struct _RGBA_Font_Int
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    RGBA_Font_Source *src;
 
@@ -625,7 +811,12 @@ struct _RGBA_Font_Int
       FT_Size       size;
    } ft;
 
-   Evas_Hash       *glyphs;
+   Eina_Hash       *glyphs;
+
+   LK(ft_mutex);
+   
+   Eina_Hash       *kerning;
+   Eina_Hash       *indexes;
 
    int              usage;
    Font_Hint_Flags hinting;
@@ -635,7 +826,7 @@ struct _RGBA_Font_Int
 
 struct _RGBA_Font_Source
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    const char       *name;
    const char       *file;
@@ -778,7 +969,7 @@ struct _Tilebuf_Tile
 
 struct _Tilebuf_Rect
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
    int               x, y, w, h;
 };
 /*
@@ -790,7 +981,7 @@ struct _Regionbuf
 
 struct _Regionspan
 {
-   Evas_Object_List  _list_data;
+  EINA_INLIST;
    int x1, x2;
 };
 */
@@ -936,14 +1127,22 @@ Tilebuf_Rect *evas_common_regionbuf_rects_get (Regionbuf *rb);
 #include "../engines/common/evas_draw.h"
 
 /****/
-#include "../engines/common/evas_pipe.h"
-
+#ifdef BUILD_PIPE_RENDER
+# include "../engines/common/evas_pipe.h"
+#endif
+   
 void              evas_font_dir_cache_free(void);
 
 /****/
 
 /*****************************************************************************/
 
+#if defined(__ARM_ARCH__) && (__ARM_ARCH__ >= 70)
+#ifdef BUILD_NEON
+# include <arm_neon.h>
+#endif
+#endif   
+   
 #ifdef __cplusplus
 }
 #endif
