@@ -9,6 +9,8 @@
 # include <config.h>
 #endif
 
+#include <Eina.h>
+#include <eina_safety_checks.h>
 #include "Evas.h"
 
 #include "../file/evas_module.h"
@@ -24,6 +26,8 @@
 typedef struct _Evas_Layer                  Evas_Layer;
 typedef struct _Evas_Size                   Evas_Size;
 typedef struct _Evas_Aspect                 Evas_Aspect;
+typedef struct _Evas_Border                 Evas_Border;
+typedef struct _Evas_Double_Pair            Evas_Double_Pair;
 typedef struct _Evas_Size_Hints             Evas_Size_Hints;
 typedef struct _Evas_Font_Dir               Evas_Font_Dir;
 typedef struct _Evas_Font                   Evas_Font;
@@ -81,12 +85,12 @@ MAGIC_CHECK_FAILED(o, t, m)
 #endif
 
 #define NEW_RECT(_r, _x, _y, _w, _h) \
-{(_r) = malloc(sizeof(Evas_Rectangle)); \
-if (_r) \
-{ \
-   (_r)->x = (_x); (_r)->y = (_y); \
-   (_r)->w = (_w); (_r)->h = (_h); \
-}}
+{(_r) = (Evas_Rectangle *)eina_mempool_alloc(_evas_rectangle_mp, sizeof(Evas_Rectangle)); \
+   if (_r)								\
+     {									\
+	(_r)->x = (_x); (_r)->y = (_y);					\
+	(_r)->w = (_w); (_r)->h = (_h);					\
+     }}
 
 #define MERR_NONE() _evas_alloc_error = EVAS_ALLOC_ERROR_NONE
 #define MERR_FATAL() _evas_alloc_error = EVAS_ALLOC_ERROR_FATAL
@@ -95,26 +99,26 @@ if (_r) \
 #define EVAS_OBJECT_IMAGE_FREE_FILE_AND_KEY(o)                              \
    if ((o)->cur.file)                                                       \
      {                                                                      \
-         evas_stringshare_del((o)->cur.file);                               \
+         eina_stringshare_del((o)->cur.file);                               \
 	 if ((o)->prev.file == (o)->cur.file)                               \
 	       (o)->prev.file = NULL;                                       \
 	 (o)->cur.file = NULL;                                              \
      }                                                                      \
    if ((o)->cur.key)                                                        \
      {                                                                      \
-         evas_stringshare_del((o)->cur.key);                                \
+         eina_stringshare_del((o)->cur.key);                                \
 	 if ((o)->prev.key == (o)->cur.key)                                 \
 	       (o)->prev.key = NULL;                                        \
 	 (o)->cur.key = NULL;                                               \
      }                                                                      \
    if ((o)->prev.file)                                                      \
      {                                                                      \
-         evas_stringshare_del((o)->prev.file);                              \
+         eina_stringshare_del((o)->prev.file);                              \
 	 (o)->prev.file = NULL;                                             \
      }                                                                      \
    if ((o)->prev.key)                                                       \
      {                                                                      \
-         evas_stringshare_del((o)->prev.key);                               \
+         eina_stringshare_del((o)->prev.key);                               \
 	 (o)->prev.key = NULL;                                              \
      }
 
@@ -208,32 +212,32 @@ struct _Evas_Lock
 
 struct _Evas_Callbacks
 {
-   Evas_Object_List *callbacks;
+   Eina_Inlist *callbacks;
    int               walking_list;
    unsigned char     deletions_waiting : 1;
 /*
-   Evas_Object_List *down;
-   Evas_Object_List *up;
-   Evas_Object_List *move;
-   Evas_Object_List *in;
-   Evas_Object_List *out;
-   Evas_Object_List *wheel;
-   Evas_Object_List *key_down;
-   Evas_Object_List *key_up;
-   Evas_Object_List *free;
-   Evas_Object_List *obj_focus_in;
-   Evas_Object_List *obj_focus_out;
-   Evas_Object_List *obj_show;
-   Evas_Object_List *obj_hide;
-   Evas_Object_List *obj_move;
-   Evas_Object_List *obj_resize;
-   Evas_Object_List *obj_restack;
+   Eina_Inlist *down;
+   Eina_Inlist *up;
+   Eina_Inlist *move;
+   Eina_Inlist *in;
+   Eina_Inlist *out;
+   Eina_Inlist *wheel;
+   Eina_Inlist *key_down;
+   Eina_Inlist *key_up;
+   Eina_Inlist *free;
+   Eina_Inlist *obj_focus_in;
+   Eina_Inlist *obj_focus_out;
+   Eina_Inlist *obj_show;
+   Eina_Inlist *obj_hide;
+   Eina_Inlist *obj_move;
+   Eina_Inlist *obj_resize;
+   Eina_Inlist *obj_restack;
  */
 };
 
 struct _Evas
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    DATA32            magic;
 
@@ -246,7 +250,7 @@ struct _Evas
 ////      Evas_Coord         canvas_x, canvas_y;
 
       struct {
-	 Evas_List *in;
+	 Eina_List *in;
       } object;
 
    } pointer;
@@ -262,12 +266,12 @@ struct _Evas
       unsigned char  changed : 1;
    } output;
 
-   Evas_List        *damages;
-   Evas_List        *obscures;
+   Eina_List        *damages;
+   Eina_List        *obscures;
 
    Evas_Layer       *layers;
 
-   Evas_Hash        *name_hash;
+   Eina_Hash        *name_hash;
 
    int               output_validity;
 
@@ -287,20 +291,21 @@ struct _Evas
       int   info_magic;
    } engine;
 
-   Evas_Array     delete_objects;
-   Evas_Array     active_objects;
-   Evas_Array     restack_objects;
-   Evas_Array	  render_objects;
-   Evas_Array	  pending_objects;
-   Evas_Array     obscuring_objects;
-   Evas_Array     temporary_objects;
-   Evas_Array     calculate_objects;
+   Eina_Array     delete_objects;
+   Eina_Array     active_objects;
+   Eina_Array     restack_objects;
+   Eina_Array	  render_objects;
+   Eina_Array	  pending_objects;
+   Eina_Array     obscuring_objects;
+   Eina_Array     temporary_objects;
+   Eina_Array     calculate_objects;
+   Eina_Array     clip_changes;
 
    int            delete_grabs;
    int            walking_grabs;
-   Evas_List     *grabs;
+   Eina_List     *grabs;
 
-   Evas_List     *font_path;
+   Eina_List     *font_path;
 
    Evas_Object   *focused;
    void          *attach_data;
@@ -318,7 +323,7 @@ struct _Evas
 
 struct _Evas_Layer
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    short             layer;
    Evas_Object      *objects;
@@ -348,15 +353,27 @@ struct _Evas_Aspect
    Evas_Size size;
 };
 
+struct _Evas_Border
+{
+   Evas_Coord l, r, t, b;
+};
+
+struct _Evas_Double_Pair
+{
+   double x, y;
+};
+
 struct _Evas_Size_Hints
 {
    Evas_Size min, max, request;
    Evas_Aspect aspect;
+   Evas_Double_Pair align, weight;
+   Evas_Border padding;
 };
 
 struct _Evas_Object
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    DATA32            magic;
 
@@ -397,16 +414,16 @@ struct _Evas_Object
    Evas_Intercept_Func *interceptors;
 
    struct {
-      Evas_List *elements;
+      Eina_List *elements;
    } data;
 
-   Evas_List *grabs;
+   Eina_List *grabs;
 
    Evas_Callbacks *callbacks;
 
    struct {
-      Evas_List   *clipees;
-      Evas_List   *changes;
+      Eina_List   *clipees;
+      Eina_List   *changes;
    } clip;
 
    const Evas_Object_Func *func;
@@ -449,7 +466,7 @@ struct _Evas_Object
 
 struct _Evas_Func_Node
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
    void (*func) (void *data, Evas *e, Evas_Object *obj, void *event_info);
    void *data;
    Evas_Callback_Type type;
@@ -464,9 +481,9 @@ struct _Evas_Data_Node
 
 struct _Evas_Font_Dir
 {
-   Evas_Hash *lookup;
-   Evas_List *fonts;
-   Evas_List *aliases;
+   Eina_Hash *lookup;
+   Eina_List *fonts;
+   Eina_List *aliases;
    DATA64     dir_mod_time;
    DATA64     fonts_dir_mod_time;
    DATA64     fonts_alias_mod_time;
@@ -514,13 +531,18 @@ struct _Evas_Object_Func
    int  (*was_inside) (Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 
    void (*coords_recalc) (Evas_Object *obj);
+   
+   void (*scale_update) (Evas_Object *obj);
+
+   int (*has_opaque_rect) (Evas_Object *obj);
+   int (*get_opaque_rect) (Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h);
 };
 
 struct _Evas_Func
 {
    void *(*info)                           (Evas *e);
    void (*info_free)                       (Evas *e, void *info);
-   void (*setup)                           (Evas *e, void *info);
+   int  (*setup)                           (Evas *e, void *info);
 
    void (*output_free)                     (void *data);
    void (*output_resize)                   (void *data, int w, int h);
@@ -615,7 +637,7 @@ struct _Evas_Func
    void *(*image_data_get)                 (void *data, void *image, int to_write, DATA32 **image_data);
    void *(*image_data_put)                 (void *data, void *image, DATA32 *image_data);
    void  (*image_data_preload_request)     (void *data, void *image, const void *target);
-   void  (*image_data_preload_cancel)      (void *data, void *image);
+   void  (*image_data_preload_cancel)      (void *data, void *image, const void *target);
    void *(*image_alpha_set)                (void *data, void *image, int has_alpha);
    int  (*image_alpha_get)                 (void *data, void *image);
    void *(*image_border_set)               (void *data, void *image, int l, int r, int t, int b);
@@ -660,6 +682,8 @@ struct _Evas_Func
 
 /*    void (*image_rotation_set)              (void *data, void *image); */
 
+   void (*image_scale_hint_set)            (void *data, void *image, int hint);
+   int  (*image_scale_hint_get)            (void *data, void *image);
 };
 
 struct _Evas_Image_Load_Func
@@ -682,11 +706,12 @@ void evas_object_free(Evas_Object *obj, int clean_layer);
 void evas_object_inject(Evas_Object *obj, Evas *e);
 void evas_object_release(Evas_Object *obj, int clean_layer);
 void evas_object_change(Evas_Object *obj);
-void evas_object_render_pre_visible_change(Evas_Rectangles *rects, Evas_Object *obj, int is_v, int was_v);
-void evas_object_render_pre_clipper_change(Evas_Rectangles *rects, Evas_Object *obj);
-void evas_object_render_pre_prev_cur_add(Evas_Rectangles *rects, Evas_Object *obj);
-void evas_object_render_pre_effect_updates(Evas_Rectangles *rects, Evas_Object *obj, int is_v, int was_v);
-void evas_rects_return_difference_rects(Evas_Rectangles *rects, int x, int y, int w, int h, int xx, int yy, int ww, int hh);
+void evas_object_clip_changes_clean(Evas_Object *obj);
+void evas_object_render_pre_visible_change(Eina_Array *rects, Evas_Object *obj, int is_v, int was_v);
+void evas_object_render_pre_clipper_change(Eina_Array *rects, Evas_Object *obj);
+void evas_object_render_pre_prev_cur_add(Eina_Array *rects, Evas_Object *obj);
+void evas_object_render_pre_effect_updates(Eina_Array *rects, Evas_Object *obj, int is_v, int was_v);
+void evas_rects_return_difference_rects(Eina_Array *rects, int x, int y, int w, int h, int xx, int yy, int ww, int hh);
 
 void evas_object_clip_dirty(Evas_Object *obj);
 void evas_object_recalc_clippees(Evas_Object *obj);
@@ -697,7 +722,6 @@ Evas_Layer *evas_layer_find(Evas *e, short layer_num);
 void evas_layer_add(Evas_Layer *lay);
 void evas_layer_del(Evas_Layer *lay);
 
-int evas_object_was_visible(Evas_Object *obj);
 int evas_object_was_in_output_rect(Evas_Object *obj, int x, int y, int w, int h);
 
 int evas_object_was_opaque(Evas_Object *obj);
@@ -705,7 +729,7 @@ int evas_object_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 int evas_object_was_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 int evas_object_clippers_was_visible(Evas_Object *obj);
 void evas_object_event_callback_call(Evas_Object *obj, Evas_Callback_Type type, void *event_info);
-Evas_List *evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y);
+Eina_List *evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y);
 int evas_mem_free(int mem_required);
 int evas_mem_degrade(int mem_required);
 void evas_debug_error(void);
@@ -722,7 +746,7 @@ void evas_object_smart_member_raise(Evas_Object *member);
 void evas_object_smart_member_lower(Evas_Object *member);
 void evas_object_smart_member_stack_above(Evas_Object *member, Evas_Object *other);
 void evas_object_smart_member_stack_below(Evas_Object *member, Evas_Object *other);
-const Evas_Object_List *evas_object_smart_members_get_direct(const Evas_Object *obj);
+const Eina_Inlist *evas_object_smart_members_get_direct(const Evas_Object *obj);
 void evas_call_smarts_calculate(Evas *e);
 void *evas_mem_calloc(int size);
 void evas_object_event_callback_all_del(Evas_Object *obj);
@@ -733,6 +757,7 @@ void evas_object_inform_call_move(Evas_Object *obj);
 void evas_object_inform_call_resize(Evas_Object *obj);
 void evas_object_inform_call_restack(Evas_Object *obj);
 void evas_object_inform_call_changed_size_hints(Evas_Object *obj);
+void evas_object_inform_call_image_preloaded(Evas_Object *obj);
 void evas_object_intercept_cleanup(Evas_Object *obj);
 int evas_object_intercept_call_show(Evas_Object *obj);
 int evas_object_intercept_call_hide(Evas_Object *obj);
@@ -750,9 +775,10 @@ void evas_object_grabs_cleanup(Evas_Object *obj);
 void evas_key_grab_free(Evas_Object *obj, const char *keyname, Evas_Modifier_Mask modifiers, Evas_Modifier_Mask not_modifiers);
 void evas_font_dir_cache_free(void);
 const char *evas_font_dir_cache_find(char *dir, char *font);
-Evas_List *evas_font_dir_available_list(const Evas* evas);
-void evas_font_dir_available_list_free(Evas_List *available);
+Eina_List *evas_font_dir_available_list(const Evas* evas);
+void evas_font_dir_available_list_free(Eina_List *available);
 void evas_font_free(Evas *evas, void *font);
+void evas_fonts_zero_free(Evas *evas);
 void *evas_font_load(Evas *evas, const char *name, const char *source, int size);
 void evas_font_load_hinting_set(Evas *evas, void *font, int hinting);
 void evas_object_smart_member_cache_invalidate(Evas_Object *obj);
@@ -761,6 +787,7 @@ void _evas_object_text_rehint(Evas_Object *obj);
 void _evas_object_textblock_rehint(Evas_Object *obj);
 
 extern int _evas_alloc_error;
+extern Eina_Mempool *_evas_rectangle_mp;
 
 struct _Evas_Imaging_Image
 {
