@@ -1,6 +1,8 @@
 #ifndef EVAS_GL_COMMON_H
 #define EVAS_GL_COMMON_H
 
+#define EVAS_GL_COMMON_NOCUTOUTS 1
+
 /* FIXME: need to handle memory errors */
 /* FIXME: need to handle list errors */
 /* FIXME: need to handle gl errors */
@@ -20,18 +22,20 @@
 #include <sys/time.h>
 #include <unistd.h>
 
-#ifdef BUILD_ENGINE_GL_QUARTZ
-#include <OpenGL/gl.h>
-#include <OpenGL/glu.h>
-#else
-#include <GL/gl.h>
-#include <GL/glu.h>
-#endif /* BUILD_ENGINE_GL_QUARTZ */
-
 #ifdef BUILD_ENGINE_GL_GLEW
-#include <GL/glew.h>
+# include <GL/glew.h>
+#else
+# define GL_GLEXT_PROTOTYPES
 #endif /* BUILD_ENGINE_GL_GLEW */
 
+#ifdef BUILD_ENGINE_GL_QUARTZ
+# include <OpenGL/gl.h>
+# include <OpenGL/glu.h>
+#else
+# include <GL/gl.h>
+# include <GL/glu.h>
+#endif /* BUILD_ENGINE_GL_QUARTZ */
+
 typedef struct _Evas_GL_Context                      Evas_GL_Context;
 typedef struct _Evas_GL_Texture                      Evas_GL_Texture;
 typedef struct _Evas_GL_Image                        Evas_GL_Image;
@@ -88,8 +92,8 @@ struct _Evas_GL_Context
 
    int             references;
 
-   Evas_List      *images;
-   Evas_List      *tex_pool;
+   Eina_List      *images;
+   Eina_List      *tex_pool;
 
    RGBA_Draw_Context  *dc;
    
@@ -137,7 +141,7 @@ struct _Evas_GL_Image
 
 struct _Evas_GL_Polygon
 {
-   Evas_List *points;
+   Eina_List *points;
    GLuint     dl;
    Evas_Bool  changed : 1;
 };
@@ -173,7 +177,7 @@ struct _Evas_GL_Font_Texture_Pool
    GLuint           texture;
    int              references;
    unsigned char    rectangle : 1;
-   Evas_List       *allocations;
+   Eina_List       *allocations;
 };
 
 struct _Evas_GL_Font_Texture_Pool_Allocation
