@@ -150,7 +150,7 @@ evas_gl_common_texture_new(Evas_GL_Context *gc, RGBA_Image *im, int smooth)
 }
 
 void
-evas_gl_common_texture_update(Evas_GL_Texture *tex, RGBA_Image *im, int smooth)
+evas_gl_common_texture_update(Evas_GL_Texture *tex, RGBA_Image *im, int smooth __UNUSED__)
 {
    DATA32 *im_data;
    int im_w, im_h;
@@ -425,7 +425,7 @@ Evas_GL_Texture *
 evas_gl_common_ycbcr601pl_texture_new(Evas_GL_Context *gc, unsigned char **rows, int w, int h, int smooth)
 {
    Evas_GL_Texture *tex;
-   int im_w, im_h, tw, th, y;
+   int tw, th;
    GLenum texfmt;
   
 // on an nv 6600gt this is fast - but on a 5500fx its DEAD SLOW!!!!!   
@@ -543,11 +543,10 @@ evas_gl_common_ycbcr601pl_texture_new(Evas_GL_Context *gc, unsigned char **rows,
 }
 
 void
-evas_gl_common_ycbcr601pl_texture_update(Evas_GL_Texture *tex, unsigned char **rows, int w, int h, int smooth)
+evas_gl_common_ycbcr601pl_texture_update(Evas_GL_Texture *tex, unsigned char **rows, int w __UNUSED__, int h __UNUSED__, int smooth)
 {
    int texfmt;
-   int y;
-   
+
    /* FIXME: should use subimage */
    glEnable(GL_TEXTURE_2D);
    texfmt = GL_LUMINANCE;
