@@ -15,37 +15,37 @@
            __FUNCTION__, __FILE__, __LINE__)
 
 static void *
-eng_context_new(void *data)
+eng_context_new(void *data __UNUSED__)
 {
    return evas_common_draw_context_new();
 }
 
 static void
-eng_context_free(void *data, void *context)
+eng_context_free(void *data __UNUSED__, void *context)
 {
    evas_common_draw_context_free(context);
 }
 
 static void
-eng_context_clip_set(void *data, void *context, int x, int y, int w, int h)
+eng_context_clip_set(void *data __UNUSED__, void *context, int x, int y, int w, int h)
 {
    evas_common_draw_context_set_clip(context, x, y, w, h);
 }
 
 static void
-eng_context_clip_clip(void *data, void *context, int x, int y, int w, int h)
+eng_context_clip_clip(void *data __UNUSED__, void *context, int x, int y, int w, int h)
 {
    evas_common_draw_context_clip_clip(context, x, y, w, h);
 }
 
 static void
-eng_context_clip_unset(void *data, void *context)
+eng_context_clip_unset(void *data __UNUSED__, void *context)
 {
    evas_common_draw_context_unset_clip(context);
 }
 
 static int
-eng_context_clip_get(void *data, void *context, int *x, int *y, int *w, int *h)
+eng_context_clip_get(void *data __UNUSED__, void *context, int *x, int *y, int *w, int *h)
 {
    *x = ((RGBA_Draw_Context *)context)->clip.x;
    *y = ((RGBA_Draw_Context *)context)->clip.y;
@@ -55,13 +55,13 @@ eng_context_clip_get(void *data, void *context, int *x, int *y, int *w, int *h)
 }
 
 static void
-eng_context_color_set(void *data, void *context, int r, int g, int b, int a)
+eng_context_color_set(void *data __UNUSED__, void *context, int r, int g, int b, int a)
 {
    evas_common_draw_context_set_color(context, r, g, b, a);
 }
 
 static int
-eng_context_color_get(void *data, void *context, int *r, int *g, int *b, int *a)
+eng_context_color_get(void *data __UNUSED__, void *context, int *r, int *g, int *b, int *a)
 {
    *r = (int)(R_VAL(&((RGBA_Draw_Context *)context)->col.col));
    *g = (int)(G_VAL(&((RGBA_Draw_Context *)context)->col.col));
@@ -71,19 +71,19 @@ eng_context_color_get(void *data, void *context, int *r, int *g, int *b, int *a)
 }
 
 static void
-eng_context_multiplier_set(void *data, void *context, int r, int g, int b, int a)
+eng_context_multiplier_set(void *data __UNUSED__, void *context, int r, int g, int b, int a)
 {
    evas_common_draw_context_set_multiplier(context, r, g, b, a);
 }
 
 static void
-eng_context_multiplier_unset(void *data, void *context)
+eng_context_multiplier_unset(void *data __UNUSED__, void *context)
 {
    evas_common_draw_context_unset_multiplier(context);
 }
 
 static int
-eng_context_multiplier_get(void *data, void *context, int *r, int *g, int *b, int *a)
+eng_context_multiplier_get(void *data __UNUSED__, void *context, int *r, int *g, int *b, int *a)
 {
    *r = (int)(R_VAL(&((RGBA_Draw_Context *)context)->mul.col));
    *g = (int)(G_VAL(&((RGBA_Draw_Context *)context)->mul.col));
@@ -93,49 +93,49 @@ eng_context_multiplier_get(void *data, void *context, int *r, int *g, int *b, in
 }
 
 static void
-eng_context_cutout_add(void *data, void *context, int x, int y, int w, int h)
+eng_context_cutout_add(void *data __UNUSED__, void *context, int x, int y, int w, int h)
 {
    evas_common_draw_context_add_cutout(context, x, y, w, h);
 }
 
 static void
-eng_context_cutout_clear(void *data, void *context)
+eng_context_cutout_clear(void *data __UNUSED__, void *context)
 {
    evas_common_draw_context_clear_cutouts(context);
 }
 
 static void
-eng_context_anti_alias_set(void *data, void *context, unsigned char aa)
+eng_context_anti_alias_set(void *data __UNUSED__, void *context, unsigned char aa)
 {
    evas_common_draw_context_set_anti_alias(context, aa);
 }
 
 static unsigned char
-eng_context_anti_alias_get(void *data, void *context)
+eng_context_anti_alias_get(void *data __UNUSED__, void *context)
 {
    return ((RGBA_Draw_Context *)context)->anti_alias;
 }
 
 static void
-eng_context_color_interpolation_set(void *data, void *context, int color_space)
+eng_context_color_interpolation_set(void *data __UNUSED__, void *context, int color_space)
 {
    evas_common_draw_context_set_color_interpolation(context, color_space);
 }
 
 static int
-eng_context_color_interpolation_get(void *data, void *context)
+eng_context_color_interpolation_get(void *data __UNUSED__, void *context)
 {
    return ((RGBA_Draw_Context *)context)->interpolation.color_space;
 }
 
 static void
-eng_context_render_op_set(void *data, void *context, int op)
+eng_context_render_op_set(void *data __UNUSED__, void *context, int op)
 {
    evas_common_draw_context_set_render_op(context, op);
 }
 
 static int
-eng_context_render_op_get(void *data, void *context)
+eng_context_render_op_get(void *data __UNUSED__, void *context)
 {
    return ((RGBA_Draw_Context *)context)->render_op;
 }
@@ -143,143 +143,144 @@ eng_context_render_op_get(void *data, void *context)
 
 
 static void
-eng_rectangle_draw(void *data, void *context, void *surface, int x, int y, int w, int h)
+eng_rectangle_draw(void *data __UNUSED__, void *context, void *surface, int x, int y, int w, int h)
 {
    soft16_rectangle_draw(surface, context, x, y, w, h);
 }
 
 static void
-eng_line_draw(void *data, void *context, void *surface, int x1, int y1, int x2, int y2)
+eng_line_draw(void *data __UNUSED__, void *context, void *surface, int x1, int y1, int x2, int y2)
 {
    soft16_line_draw(surface, context, x1, y1, x2, y2);
 }
 
 static void *
-eng_polygon_point_add(void *data, void *context, void *polygon, int x, int y)
+eng_polygon_point_add(void *data __UNUSED__, void *context __UNUSED__, void *polygon, int x, int y)
 {
    return evas_common_polygon_point_add(polygon, x, y);
 }
 
 static void *
-eng_polygon_points_clear(void *data, void *context, void *polygon)
+eng_polygon_points_clear(void *data __UNUSED__, void *context __UNUSED__, void *polygon)
 {
    return evas_common_polygon_points_clear(polygon);
 }
 
 static void
-eng_polygon_draw(void *data, void *context, void *surface, void *polygon)
+eng_polygon_draw(void *data __UNUSED__, void *context, void *surface, void *polygon)
 {
    soft16_polygon_draw(surface, context, polygon);
 }
 
+
 static void
-eng_gradient2_color_np_stop_insert(void *data, void *gradient, int r, int g, int b, int a, float pos)
+eng_gradient2_color_np_stop_insert(void *data __UNUSED__, void *gradient __UNUSED__, int r __UNUSED__, int g __UNUSED__, int b __UNUSED__, int a __UNUSED__, float pos __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_clear(void *data, void *gradient)
+eng_gradient2_clear(void *data __UNUSED__, void *gradient __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_fill_transform_set(void *data, void *gradient, void *transform)
+eng_gradient2_fill_transform_set(void *data __UNUSED__, void *gradient __UNUSED__, void *transform __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_fill_spread_set(void *data, void *gradient, int spread)
+eng_gradient2_fill_spread_set(void *data __UNUSED__, void *gradient __UNUSED__, int spread __UNUSED__)
 {
 }
 
 static void *
-eng_gradient2_linear_new(void *data)
+eng_gradient2_linear_new(void *data __UNUSED__)
 {
    return NULL;
 }
 
 static void
-eng_gradient2_linear_free(void *data, void *linear_gradient)
+eng_gradient2_linear_free(void *data __UNUSED__, void *linear_gradient __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_linear_fill_set(void *data, void *linear_gradient, int x0, int y0, int x1, int y1)
+eng_gradient2_linear_fill_set(void *data __UNUSED__, void *linear_gradient __UNUSED__, int x0 __UNUSED__, int y0 __UNUSED__, int x1 __UNUSED__, int y1 __UNUSED__)
 {
 }
 
 static int
-eng_gradient2_linear_is_opaque(void *data, void *context, void *linear_gradient, int x, int y, int w, int h)
+eng_gradient2_linear_is_opaque(void *data __UNUSED__, void *context __UNUSED__, void *linear_gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    return 1;
 }
 
 static int
-eng_gradient2_linear_is_visible(void *data, void *context, void *linear_gradient, int x, int y, int w, int h)
+eng_gradient2_linear_is_visible(void *data __UNUSED__, void *context __UNUSED__, void *linear_gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    return 1;
 }
 
 static void
-eng_gradient2_linear_render_pre(void *data, void *context, void *linear_gradient)
+eng_gradient2_linear_render_pre(void *data __UNUSED__, void *context __UNUSED__, void *linear_gradient __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_linear_render_post(void *data, void *linear_gradient)
+eng_gradient2_linear_render_post(void *data __UNUSED__, void *linear_gradient __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_linear_draw(void *data, void *context, void *surface, void *linear_gradient, int x, int y, int w, int h)
+eng_gradient2_linear_draw(void *data __UNUSED__, void *context __UNUSED__, void *surface __UNUSED__, void *linear_gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
 }
 
 static void *
-eng_gradient2_radial_new(void *data)
+eng_gradient2_radial_new(void *data __UNUSED__)
 {
    return NULL;
 }
 
 static void
-eng_gradient2_radial_free(void *data, void *radial_gradient)
+eng_gradient2_radial_free(void *data __UNUSED__, void *radial_gradient __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_radial_fill_set(void *data, void *radial_gradient, float cx, float cy, float rx, float ry)
+eng_gradient2_radial_fill_set(void *data __UNUSED__, void *radial_gradient __UNUSED__, float cx __UNUSED__, float cy __UNUSED__, float rx __UNUSED__, float ry __UNUSED__)
 {
 }
 
 static int
-eng_gradient2_radial_is_opaque(void *data, void *context, void *radial_gradient, int x, int y, int w, int h)
+eng_gradient2_radial_is_opaque(void *data __UNUSED__, void *context __UNUSED__, void *radial_gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    return 1;
 }
 
 static int
-eng_gradient2_radial_is_visible(void *data, void *context, void *radial_gradient, int x, int y, int w, int h)
+eng_gradient2_radial_is_visible(void *data __UNUSED__, void *context __UNUSED__, void *radial_gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    return 1;
 }
 
 static void
-eng_gradient2_radial_render_pre(void *data, void *context, void *radial_gradient)
+eng_gradient2_radial_render_pre(void *data __UNUSED__, void *context __UNUSED__, void *radial_gradient __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_radial_render_post(void *data, void *radial_gradient)
+eng_gradient2_radial_render_post(void *data __UNUSED__, void *radial_gradient __UNUSED__)
 {
 }
 
 static void
-eng_gradient2_radial_draw(void *data, void *context, void *surface, void *radial_gradient, int x, int y, int w, int h)
+eng_gradient2_radial_draw(void *data __UNUSED__, void *context __UNUSED__, void *surface __UNUSED__, void *radial_gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
 }
 
 static void *
-eng_gradient_new(void *data)
+eng_gradient_new(void *data __UNUSED__)
 {
    NOT_IMPLEMENTED();
    return NULL;
@@ -287,98 +288,98 @@ eng_gradient_new(void *data)
 }
 
 static void
-eng_gradient_free(void *data, void *gradient)
+eng_gradient_free(void *data __UNUSED__, void *gradient __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_free(gradient);
 }
 
 static void
-eng_gradient_color_stop_add(void *data, void *gradient, int r, int g, int b, int a, int delta)
+eng_gradient_color_stop_add(void *data __UNUSED__, void *gradient __UNUSED__, int r __UNUSED__, int g __UNUSED__, int b __UNUSED__, int a __UNUSED__, int delta __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_color_stop_add(gradient, r, g, b, a, delta);
 }
 
 static void
-eng_gradient_alpha_stop_add(void *data, void *gradient, int a, int delta)
+eng_gradient_alpha_stop_add(void *data __UNUSED__, void *gradient __UNUSED__, int a __UNUSED__, int delta __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_alpha_stop_add(gradient, a, delta);
 }
 
 static void
-eng_gradient_color_data_set(void *data, void *gradient, void *map, int len, int has_alpha)
+eng_gradient_color_data_set(void *data __UNUSED__, void *gradient __UNUSED__, void *map __UNUSED__, int len __UNUSED__, int has_alpha __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_color_data_set(gradient, map, len, has_alpha);
 }
 
 static void
-eng_gradient_alpha_data_set(void *data, void *gradient, void *alpha_map, int len)
+eng_gradient_alpha_data_set(void *data __UNUSED__, void *gradient __UNUSED__, void *alpha_map __UNUSED__, int len __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_alpha_data_set(gradient, alpha_map, len);
 }
 
 static void
-eng_gradient_clear(void *data, void *gradient)
+eng_gradient_clear(void *data __UNUSED__, void *gradient __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_clear(gradient);
 }
 
 static void
-eng_gradient_fill_set(void *data, void *gradient, int x, int y, int w, int h)
+eng_gradient_fill_set(void *data __UNUSED__, void *gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_fill_set(gradient, x, y, w, h);
 }
 
 static void
-eng_gradient_fill_angle_set(void *data, void *gradient, double angle)
+eng_gradient_fill_angle_set(void *data __UNUSED__, void *gradient __UNUSED__, double angle __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_fill_angle_set(gradient, angle);
 }
 
 static void
-eng_gradient_fill_spread_set(void *data, void *gradient, int spread)
+eng_gradient_fill_spread_set(void *data __UNUSED__, void *gradient __UNUSED__, int spread __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_fill_spread_set(gradient, spread);
 }
 
 static void
-eng_gradient_angle_set(void *data, void *gradient, double angle)
+eng_gradient_angle_set(void *data __UNUSED__, void *gradient __UNUSED__, double angle __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_map_angle_set(gradient, angle);
 }
 
 static void
-eng_gradient_offset_set(void *data, void *gradient, float offset)
+eng_gradient_offset_set(void *data __UNUSED__, void *gradient __UNUSED__, float offset __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_map_offset_set(gradient, offset);
 }
 
 static void
-eng_gradient_direction_set(void *data, void *gradient, int direction)
+eng_gradient_direction_set(void *data __UNUSED__, void *gradient __UNUSED__, int direction __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_map_direction_set(gradient, direction);
 }
 
 static void
-eng_gradient_type_set(void *data, void *gradient, char *name, char *params)
+eng_gradient_type_set(void *data __UNUSED__, void *gradient __UNUSED__, char *name __UNUSED__, char *params __UNUSED__)
 {
    NOT_IMPLEMENTED();
 //   evas_common_gradient_type_set(gradient, name, params);
 }
 
 static int
-eng_gradient_is_opaque(void *data, void *context, void *gradient, int x, int y, int w, int h)
+eng_gradient_is_opaque(void *data __UNUSED__, void *context __UNUSED__, void *gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    NOT_IMPLEMENTED();
    return 0;
@@ -391,7 +392,7 @@ eng_gradient_is_opaque(void *data, void *context, void *gradient, int x, int y,
 }
 
 static int
-eng_gradient_is_visible(void *data, void *context, void *gradient, int x, int y, int w, int h)
+eng_gradient_is_visible(void *data __UNUSED__, void *context __UNUSED__, void *gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    NOT_IMPLEMENTED();
    return 0;
@@ -402,7 +403,7 @@ eng_gradient_is_visible(void *data, void *context, void *gradient, int x, int y,
 }
 
 static void
-eng_gradient_render_pre(void *data, void *context, void *gradient)
+eng_gradient_render_pre(void *data __UNUSED__, void *context __UNUSED__, void *gradient __UNUSED__)
 {
 //   RGBA_Draw_Context *dc = (RGBA_Draw_Context *)context;
 //   RGBA_Gradient *gr = (RGBA_Gradient *)gradient;
@@ -416,13 +417,13 @@ eng_gradient_render_pre(void *data, void *context, void *gradient)
 }
 
 static void
-eng_gradient_render_post(void *data, void *gradient)
+eng_gradient_render_post(void *data __UNUSED__, void *gradient __UNUSED__)
 {
    NOT_IMPLEMENTED();
 }
 
 static void
-eng_gradient_draw(void *data, void *context, void *surface, void *gradient, int x, int y, int w, int h)
+eng_gradient_draw(void *data __UNUSED__, void *context __UNUSED__, void *surface __UNUSED__, void *gradient __UNUSED__, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
 //   evas_common_gradient_draw(surface, context, x, y, w, h, gradient);
 //   evas_common_cpu_end_opt();
@@ -430,7 +431,7 @@ eng_gradient_draw(void *data, void *context, void *surface, void *gradient, int
 }
 
 static int
-eng_image_alpha_get(void *data, void *image)
+eng_image_alpha_get(void *data __UNUSED__, void *image)
 {
    Soft16_Image *im;
 
@@ -440,13 +441,13 @@ eng_image_alpha_get(void *data, void *image)
 }
 
 static int
-eng_image_colorspace_get(void *data, void *image)
+eng_image_colorspace_get(void *data __UNUSED__, void *image __UNUSED__)
 {
    return EVAS_COLORSPACE_RGB565_A5P;
 }
 
 static void *
-eng_image_alpha_set(void *data, void *image, int have_alpha)
+eng_image_alpha_set(void *data __UNUSED__, void *image, int have_alpha)
 {
    if (!image) return NULL;
    have_alpha = !!have_alpha;
@@ -455,56 +456,56 @@ eng_image_alpha_set(void *data, void *image, int have_alpha)
 }
 
 static void *
-eng_image_border_set(void *data, void *image, int l, int r, int t, int b)
+eng_image_border_set(void *data __UNUSED__, void *image, int l __UNUSED__, int r __UNUSED__, int t __UNUSED__, int b __UNUSED__)
 {
    return image;
 }
 
 static void
-eng_image_border_get(void *data, void *image, int *l, int *r, int *t, int *b)
+eng_image_border_get(void *data __UNUSED__, void *image __UNUSED__, int *l __UNUSED__, int *r __UNUSED__, int *t __UNUSED__, int *b __UNUSED__)
 {
 }
 
 static char *
-eng_image_comment_get(void *data, void *image, char *key)
+eng_image_comment_get(void *data __UNUSED__, void *image __UNUSED__, char *key __UNUSED__)
 {
    return NULL;
 }
 
 static char *
-eng_image_format_get(void *data, void *image)
+eng_image_format_get(void *data __UNUSED__, void *image __UNUSED__)
 {
    NOT_IMPLEMENTED();
    return NULL;
 }
 
 static void
-eng_image_colorspace_set(void *data, void *image, int cspace)
+eng_image_colorspace_set(void *data __UNUSED__, void *image __UNUSED__, int cspace __UNUSED__)
 {
    NOT_IMPLEMENTED();
 }
 
 static void
-eng_image_native_set(void *data, void *image, void *native)
+eng_image_native_set(void *data __UNUSED__, void *image __UNUSED__, void *native __UNUSED__)
 {
    NOT_IMPLEMENTED();
 }
 
 static void *
-eng_image_native_get(void *data, void *image)
+eng_image_native_get(void *data __UNUSED__, void *image __UNUSED__)
 {
    NOT_IMPLEMENTED();
    return NULL;
 }
 
 static void *
-eng_image_load(void *data, const char *file, const char *key, int *error, Evas_Image_Load_Opts *lo)
+eng_image_load(void *data __UNUSED__, const char *file, const char *key, int *error, Evas_Image_Load_Opts *lo)
 {
    return evas_cache_image_request(evas_common_soft16_image_cache_get(), file, key, lo, error);
 }
 
 static void *
-eng_image_new_from_data(void *data, int w, int h, DATA32 *image_data, int alpha, int cspace)
+eng_image_new_from_data(void *data __UNUSED__, int w, int h, DATA32 *image_data, int alpha, int cspace)
 {
    if ((image_data) && (cspace != EVAS_COLORSPACE_RGB565_A5P))
      {
@@ -516,7 +517,7 @@ eng_image_new_from_data(void *data, int w, int h, DATA32 *image_data, int alpha,
 }
 
 static void *
-eng_image_new_from_copied_data(void *data, int w, int h, DATA32 *image_data, int alpha, int cspace)
+eng_image_new_from_copied_data(void *data __UNUSED__, int w, int h, DATA32 *image_data, int alpha, int cspace)
 {
    if ((image_data) && (cspace != EVAS_COLORSPACE_RGB565_A5P))
      {
@@ -528,13 +529,13 @@ eng_image_new_from_copied_data(void *data, int w, int h, DATA32 *image_data, int
 }
 
 static void
-eng_image_free(void *data, void *image)
+eng_image_free(void *data __UNUSED__, void *image)
 {
    evas_cache_image_drop((Image_Entry *) image);
 }
 
 static void
-eng_image_size_get(void *data, void *image, int *w, int *h)
+eng_image_size_get(void *data __UNUSED__, void *image, int *w, int *h)
 {
    Soft16_Image *im;
 
@@ -547,7 +548,7 @@ eng_image_size_get(void *data, void *image, int *w, int *h)
 }
 
 static void *
-eng_image_size_set(void *data, void *image, int w, int h)
+eng_image_size_set(void *data __UNUSED__, void *image, int w, int h)
 {
    if (!image) return NULL;
    if ((w <= 0) || (h <= 0))
@@ -559,7 +560,7 @@ eng_image_size_set(void *data, void *image, int w, int h)
 }
 
 static void
-eng_image_stride_get(void *data, void *image, int *stride)
+eng_image_stride_get(void *data __UNUSED__, void *image, int *stride)
 {
    Soft16_Image *im;
 
@@ -570,7 +571,7 @@ eng_image_stride_get(void *data, void *image, int *stride)
 }
 
 static void *
-eng_image_dirty_region(void *data, void *image, int x, int y, int w, int h)
+eng_image_dirty_region(void *data __UNUSED__, void *image, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
 {
    /* FIXME: is this required? */
    //NOT_IMPLEMENTED();
@@ -578,7 +579,7 @@ eng_image_dirty_region(void *data, void *image, int x, int y, int w, int h)
 }
 
 static void *
-eng_image_data_get(void *data, void *image, int to_write, DATA32 **image_data)
+eng_image_data_get(void *data __UNUSED__, void *image, int to_write, DATA32 **image_data)
 {
    Soft16_Image *im;
 
@@ -600,7 +601,7 @@ eng_image_data_get(void *data, void *image, int to_write, DATA32 **image_data)
 }
 
 static void *
-eng_image_data_put(void *data, void *image, DATA32 *image_data)
+eng_image_data_put(void *data __UNUSED__, void *image, DATA32 *image_data)
 {
    Soft16_Image *old_im, *new_im;
 
@@ -609,13 +610,13 @@ eng_image_data_put(void *data, void *image, DATA32 *image_data)
    old_im = image;
    if ((DATA16 *)image_data == old_im->pixels) return old_im;
 
-   new_im = (Soft16_Image *) evas_cache_image_copied_data(evas_common_soft16_image_cache_get(), old_im->cache_entry.w, old_im->cache_entry.h, image_data, old_im->cache_entry.flags.alpha, EVAS_COLORSPACE_RGB565_A5P);
+   new_im = (Soft16_Image *) evas_cache_image_data(evas_common_soft16_image_cache_get(), old_im->cache_entry.w, old_im->cache_entry.h, image_data, old_im->cache_entry.flags.alpha, EVAS_COLORSPACE_RGB565_A5P);
    evas_cache_image_drop(&old_im->cache_entry);
    return new_im;
 }
 
 static void
-eng_image_data_preload_request(void *data, void *image, void *target)
+eng_image_data_preload_request(void *data __UNUSED__, void *image, const void *target)
 {
    Soft16_Image *im = image;
 
@@ -624,16 +625,16 @@ eng_image_data_preload_request(void *data, void *image, void *target)
 }
 
 static void
-eng_image_data_preload_cancel(void *data, void *image)
+eng_image_data_preload_cancel(void *data __UNUSED__, void *image, const void *target)
 {
    Soft16_Image *im = image;
 
    if (!im) return ;
-   evas_cache_image_preload_cancel(&im->cache_entry);
+   evas_cache_image_preload_cancel(&im->cache_entry, target);
 }
 
 static void
-eng_image_draw(void *data, void *context, void *surface, void *image, int src_x, int src_y, int src_w, int src_h, int dst_x, int dst_y, int dst_w, int dst_h, int smooth)
+eng_image_draw(void *data __UNUSED__, void *context, void *surface, void *image, int src_x, int src_y, int src_w, int src_h, int dst_x, int dst_y, int dst_w, int dst_h, int smooth)
 {
    Soft16_Image *im;
 
@@ -647,91 +648,102 @@ eng_image_draw(void *data, void *context, void *surface, void *image, int src_x,
 }
 
 static void
-eng_image_cache_flush(void *data)
+eng_image_scale_hint_set(void *data __UNUSED__, void *image, int hint)
+{
+}
+
+static int
+eng_image_scale_hint_get(void *data __UNUSED__, void *image)
+{
+   return EVAS_IMAGE_SCALE_HINT_NONE;
+}
+
+static void
+eng_image_cache_flush(void *data __UNUSED__)
 {
    evas_cache_image_flush(evas_common_soft16_image_cache_get());
 }
 
 static void
-eng_image_cache_set(void *data, int bytes)
+eng_image_cache_set(void *data __UNUSED__, int bytes)
 {
    evas_cache_image_set(evas_common_soft16_image_cache_get(), bytes);
 }
 
 static int
-eng_image_cache_get(void *data)
+eng_image_cache_get(void *data __UNUSED__)
 {
    return evas_cache_image_get(evas_common_soft16_image_cache_get());
 }
 
 static void *
-eng_font_load(void *data, const char *name, int size)
+eng_font_load(void *data __UNUSED__, const char *name, int size)
 {
    return evas_common_font_load(name, size);
 }
 
 static void *
-eng_font_memory_load(void *data, char *name, int size, const void *fdata, int fdata_size)
+eng_font_memory_load(void *data __UNUSED__, char *name, int size, const void *fdata, int fdata_size)
 {
    return evas_common_font_memory_load(name, size, fdata, fdata_size);
 }
 
 static void *
-eng_font_add(void *data, void *font, const char *name, int size)
+eng_font_add(void *data __UNUSED__, void *font, const char *name, int size)
 {
    return evas_common_font_add(font, name, size);
 }
 
 static void *
-eng_font_memory_add(void *data, void *font, char *name, int size, const void *fdata, int fdata_size)
+eng_font_memory_add(void *data __UNUSED__, void *font, char *name, int size, const void *fdata, int fdata_size)
 {
    return evas_common_font_memory_add(font, name, size, fdata, fdata_size);
 }
 
 static void
-eng_font_free(void *data, void *font)
+eng_font_free(void *data __UNUSED__, void *font)
 {
    evas_common_font_free(font);
 }
 
 static int
-eng_font_ascent_get(void *data, void *font)
+eng_font_ascent_get(void *data __UNUSED__, void *font)
 {
    return evas_common_font_ascent_get(font);
 }
 
 static int
-eng_font_descent_get(void *data, void *font)
+eng_font_descent_get(void *data __UNUSED__, void *font)
 {
    return evas_common_font_descent_get(font);
 }
 
 static int
-eng_font_max_ascent_get(void *data, void *font)
+eng_font_max_ascent_get(void *data __UNUSED__, void *font)
 {
    return evas_common_font_max_ascent_get(font);
 }
 
 static int
-eng_font_max_descent_get(void *data, void *font)
+eng_font_max_descent_get(void *data __UNUSED__, void *font)
 {
    return evas_common_font_max_descent_get(font);
 }
 
 static void
-eng_font_string_size_get(void *data, void *font, const char *text, int *w, int *h)
+eng_font_string_size_get(void *data __UNUSED__, void *font, const char *text, int *w, int *h)
 {
    evas_common_font_query_size(font, text, w, h);
 }
 
 static int
-eng_font_inset_get(void *data, void *font, const char *text)
+eng_font_inset_get(void *data __UNUSED__, void *font, const char *text)
 {
    return evas_common_font_query_inset(font, text);
 }
 
 static int
-eng_font_h_advance_get(void *data, void *font, const char *text)
+eng_font_h_advance_get(void *data __UNUSED__, void *font, const char *text)
 {
    int h, v;
 
@@ -740,7 +752,7 @@ eng_font_h_advance_get(void *data, void *font, const char *text)
 }
 
 static int
-eng_font_v_advance_get(void *data, void *font, const char *text)
+eng_font_v_advance_get(void *data __UNUSED__, void *font, const char *text)
 {
    int h, v;
 
@@ -749,19 +761,19 @@ eng_font_v_advance_get(void *data, void *font, const char *text)
 }
 
 static int
-eng_font_char_coords_get(void *data, void *font, const char *text, int pos, int *cx, int *cy, int *cw, int *ch)
+eng_font_char_coords_get(void *data __UNUSED__, void *font, const char *text, int pos, int *cx, int *cy, int *cw, int *ch)
 {
    return evas_common_font_query_char_coords(font, text, pos, cx, cy, cw, ch);
 }
 
 static int
-eng_font_char_at_coords_get(void *data, void *font, const char *text, int x, int y, int *cx, int *cy, int *cw, int *ch)
+eng_font_char_at_coords_get(void *data __UNUSED__, void *font, const char *text, int x, int y, int *cx, int *cy, int *cw, int *ch)
 {
    return evas_common_font_query_text_at_pos(font, text, x, y, cx, cy, cw, ch);
 }
 
 static void
-eng_font_draw(void *data, void *context, void *surface, void *font, int x, int y, int w, int h, int ow, int oh, const char *text)
+eng_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const char *text)
 {
    static RGBA_Image    *im = NULL;
    Soft16_Image         *dst = surface;
@@ -783,31 +795,31 @@ eng_font_draw(void *data, void *context, void *surface, void *font, int x, int y
 }
 
 static void
-eng_font_cache_flush(void *data)
+eng_font_cache_flush(void *data __UNUSED__)
 {
    evas_common_font_flush();
 }
 
 static void
-eng_font_cache_set(void *data, int bytes)
+eng_font_cache_set(void *data __UNUSED__, int bytes)
 {
    evas_common_font_cache_set(bytes);
 }
 
 static int
-eng_font_cache_get(void *data)
+eng_font_cache_get(void *data __UNUSED__)
 {
    return evas_common_font_cache_get();
 }
 
 static void
-eng_font_hinting_set(void *data, void *font, int hinting)
+eng_font_hinting_set(void *data __UNUSED__, void *font, int hinting)
 {
    evas_common_font_hinting_set(font, hinting);
 }
 
 static int
-eng_font_hinting_can_hint(void *data, int hinting)
+eng_font_hinting_can_hint(void *data __UNUSED__, int hinting)
 {
    return evas_common_hinting_available(hinting);
 }
@@ -958,7 +970,9 @@ static Evas_Func func =
      eng_font_cache_get,
      /* font hinting functions */
      eng_font_hinting_set,
-     eng_font_hinting_can_hint
+     eng_font_hinting_can_hint,
+     eng_image_scale_hint_set,
+     eng_image_scale_hint_get
      /* FUTURE software generic calls go here */
 };
 
