@@ -61,12 +61,10 @@ static const Evas_Cache_Engine_Image_Func       _sdl16_cache_engine_image_cb = {
 
 /* engine api this module provides */
 static void *
-evas_engine_sdl16_info(Evas *e)
+evas_engine_sdl16_info(Evas *e __UNUSED__)
 {
    Evas_Engine_Info_SDL_16      *info;
 
-   (void) e;
-
    info = calloc(1, sizeof(Evas_Engine_Info_SDL_16));
    if (!info) return NULL;
    info->magic.magic = rand();
@@ -74,7 +72,7 @@ evas_engine_sdl16_info(Evas *e)
 }
 
 static void
-evas_engine_sdl16_info_free(Evas *e, void *info)
+evas_engine_sdl16_info_free(Evas *e __UNUSED__, void *info)
 {
    Evas_Engine_Info_SDL_16 *in;
 
@@ -88,7 +86,7 @@ _tmp_out_alloc(Render_Engine *re)
    Tilebuf_Rect *r;
    int w = 0, h = 0;
 
-   for (r = re->rects; r; r = (Tilebuf_Rect *)(r->_list_data.next))
+   EINA_INLIST_FOREACH(re->rects, r)
      {
 	if (r->w > w) w = r->w;
 	if (r->h > h) h = r->h;
@@ -118,9 +116,12 @@ _tmp_out_alloc(Render_Engine *re)
 static void*
 _sdl16_output_setup(int w, int h, int rotation, int fullscreen, int noframe, int hwsurface)
 {
-   Render_Engine	*re = calloc(1, sizeof(Render_Engine));
+   Render_Engine	*re;
    SDL_Surface          *surface;
 
+   re = calloc(1, sizeof(Render_Engine));
+   if (!re)
+     return NULL;
    /* if we haven't initialized - init (automatic abort if already done) */
    evas_common_cpu_init();
    evas_common_blend_init();
@@ -143,7 +144,8 @@ _sdl16_output_setup(int w, int h, int rotation, int fullscreen, int noframe, int
    if (!re->cache)
      {
         fprintf(stderr, "Evas_Cache_Engine_Image allocation failed!\n");
-        exit(-1);
+        free(re);
+        return NULL;
      }
 
    re->tb = evas_common_tilebuf_new(w, h);
@@ -168,7 +170,9 @@ _sdl16_output_setup(int w, int h, int rotation, int fullscreen, int noframe, int
    if (!surface)
      {
         fprintf(stderr, "SDL_SetVideoMode [ %i x %i x 16 ] failed\n", w, h);
-        exit(-1);
+        evas_cache_engine_image_shutdown(re->cache);
+        free(re);
+        return NULL;
      }
 
    SDL_SetAlpha(surface, SDL_RLEACCEL, 0);
@@ -178,27 +182,30 @@ _sdl16_output_setup(int w, int h, int rotation, int fullscreen, int noframe, int
    if (!re->soft16_engine_image)
      {
         fprintf(stderr, "Soft16_Image allocation from SDL failed\n");
-        exit(-1);
+        evas_cache_engine_image_shutdown(re->cache);
+        free(re);
+        return NULL;
      }
 
    return re;
 }
 
 
-static void
+static int
 evas_engine_sdl16_setup(Evas *e, void *in)
 {
    Evas_Engine_Info_SDL_16      *info = (Evas_Engine_Info_SDL_16 *) in;
 
    if (evas_output_method_get(e) != evas_render_method_lookup("software_16_sdl"))
-     return ;
+     return 0;
 
    SDL_Init(SDL_INIT_NOPARACHUTE);
 
    if (SDL_InitSubSystem(SDL_INIT_VIDEO) < 0)
      {
         fprintf(stderr, "SDL_Init failed with %s\n", SDL_GetError());
-        exit(-1);
+        SDL_Quit();
+        return 0;
      }
 
    e->engine.data.output = _sdl16_output_setup(e->output.w, e->output.h,
@@ -207,10 +214,12 @@ evas_engine_sdl16_setup(Evas *e, void *in)
                                                info->info.noframe,
                                                info->info.hwsurface);
    if (!e->engine.data.output)
-     return;
+     return 0;
 
    e->engine.func = &func;
    e->engine.data.context = e->engine.func->context_new(e->engine.data.output);
+
+   return 1;
 }
 
 static void
@@ -225,7 +234,7 @@ evas_engine_sdl16_output_free(void *data)
    if (re->soft16_engine_image) evas_cache_engine_image_drop(&re->soft16_engine_image->cache_entry);
    free(re);
 
-/*    evas_common_font_shutdown(); */
+   evas_common_font_shutdown();
    evas_common_image_shutdown();
    evas_common_soft16_image_shutdown();
 }
@@ -340,7 +349,7 @@ evas_engine_sdl16_output_redraws_next_update_get(void *data,
    *cy = *y = tb_rect->y;
    *cw = *w = tb_rect->w;
    *ch = *h = tb_rect->h;
-   re->cur_rect = (Tilebuf_Rect *)(re->cur_rect->_list_data.next);
+   re->cur_rect = (Tilebuf_Rect *)((EINA_INLIST_GET(re->cur_rect))->next);
    if (!re->cur_rect)
      {
         evas_common_tilebuf_free_render_rects(re->rects);
@@ -478,7 +487,7 @@ _tmp_out_process(Render_Engine *re, int out_x, int out_y, int w, int h)
 }
 
 static void
-evas_engine_sdl16_output_redraws_next_update_push(void *data, void *surface,
+evas_engine_sdl16_output_redraws_next_update_push(void *data, void *surface __UNUSED__,
                                                   int x, int y, int w, int h)
 {
    Render_Engine        *re = data;
@@ -561,13 +570,11 @@ evas_engine_sdl16_image_load(void *data, const char *file, const char *key, int
 }
 
 static int
-evas_engine_sdl16_image_alpha_get(void *data, void *image)
+evas_engine_sdl16_image_alpha_get(void *data __UNUSED__, void *image)
 {
    SDL_Engine_Image_Entry       *eim = image;
    Soft16_Image                 *im;
 
-   (void) data;
-
    if (!eim) return 1;
    im = (Soft16_Image *) eim->cache_entry.src;
    if (im->cache_entry.flags.alpha) return 1;
@@ -575,32 +582,24 @@ evas_engine_sdl16_image_alpha_get(void *data, void *image)
 }
 
 static void
-evas_engine_sdl16_image_size_get(void *data, void *image, int *w, int *h)
+evas_engine_sdl16_image_size_get(void *data __UNUSED__, void *image, int *w, int *h)
 {
    SDL_Engine_Image_Entry       *eim;
 
-   (void) data;
-
    eim = image;
    if (w) *w = eim->cache_entry.w;
    if (h) *h = eim->cache_entry.h;
 }
 
 static int
-evas_engine_sdl16_image_colorspace_get(void *data, void *image)
+evas_engine_sdl16_image_colorspace_get(void *data __UNUSED__, void *image __UNUSED__)
 {
-   (void) data;
-   (void) image;
-
    return EVAS_COLORSPACE_RGB565_A5P;
 }
 
 static void
-evas_engine_sdl16_image_colorspace_set(void *data, void *image, int cspace)
+evas_engine_sdl16_image_colorspace_set(void *data __UNUSED__, void *image __UNUSED__, int cspace __UNUSED__)
 {
-   (void) data;
-   (void) image;
-
    /* FIXME: Not implemented. */
 }
 
@@ -646,46 +645,38 @@ evas_engine_sdl16_image_new_from_data(void *data, int w, int h, DATA32* image_da
 }
 
 static void
-evas_engine_sdl16_image_free(void *data, void *image)
+evas_engine_sdl16_image_free(void *data __UNUSED__, void *image)
 {
    SDL_Engine_Image_Entry       *eim = image;
 
-   (void) data;
-
    evas_cache_engine_image_drop(&eim->cache_entry);
 }
 
 static void*
-evas_engine_sdl16_image_size_set(void *data, void *image, int w, int h)
+evas_engine_sdl16_image_size_set(void *data __UNUSED__, void *image, int w, int h)
 {
    SDL_Engine_Image_Entry       *eim = image;
 
-   (void) data;
-
    return evas_cache_engine_image_size_set(&eim->cache_entry, w, h);
 }
 
 static void*
-evas_engine_sdl16_image_dirty_region(void *data,
+evas_engine_sdl16_image_dirty_region(void *data __UNUSED__,
                                      void *image,
                                      int x, int y, int w, int h)
 {
    SDL_Engine_Image_Entry       *eim = image;
 
-   (void) data;
-
    return evas_cache_engine_image_dirty(&eim->cache_entry, x, y, w, h);
 }
 
 static void*
-evas_engine_sdl16_image_data_get(void *data, void *image,
+evas_engine_sdl16_image_data_get(void *data __UNUSED__, void *image,
                                  int to_write, DATA32** image_data)
 {
    SDL_Engine_Image_Entry       *eim = image;
    Soft16_Image                 *im;
 
-   (void) data;
-
    if (!eim)
      {
         *image_data = NULL;
@@ -730,7 +721,7 @@ evas_engine_sdl16_image_data_put(void *data, void *image, DATA32* image_data)
 }
 
 static void
-evas_engine_sdl16_image_data_preload_request(void *data, void *image, const void *target)
+evas_engine_sdl16_image_data_preload_request(void *data __UNUSED__, void *image, const void *target)
 {
    SDL_Engine_Image_Entry       *eim = image;
    Soft16_Image                 *im;
@@ -742,7 +733,7 @@ evas_engine_sdl16_image_data_preload_request(void *data, void *image, const void
 }
 
 static void
-evas_engine_sdl16_image_data_preload_cancel(void *data, void *image)
+evas_engine_sdl16_image_data_preload_cancel(void *data __UNUSED__, void *image, const void *target)
 {
    SDL_Engine_Image_Entry       *eim = image;
    Soft16_Image                 *im;
@@ -750,17 +741,15 @@ evas_engine_sdl16_image_data_preload_cancel(void *data, void *image)
    if (!eim) return ;
    im = (Soft16_Image *) eim->cache_entry.src;
    if (!im) return ;
-   evas_cache_image_preload_cancel(&im->cache_entry);
+   evas_cache_image_preload_cancel(&im->cache_entry, target);
 }
 
 static void*
-evas_engine_sdl16_image_alpha_set(void *data, void *image, int has_alpha)
+evas_engine_sdl16_image_alpha_set(void *data __UNUSED__, void *image, int has_alpha)
 {
    SDL_Engine_Image_Entry       *eim = image;
    Soft16_Image                 *im;
 
-   (void) data;
-
    if (!eim) return NULL;
 
    im = (Soft16_Image *) eim->cache_entry.src;
@@ -779,19 +768,19 @@ evas_engine_sdl16_image_alpha_set(void *data, void *image, int has_alpha)
 }
 
 static void*
-evas_engine_sdl16_image_border_set(void *data, void *image, int l, int r, int t, int b)
+evas_engine_sdl16_image_border_set(void *data __UNUSED__, void *image, int l __UNUSED__, int r __UNUSED__, int t __UNUSED__, int b __UNUSED__)
 {
    return image;
 }
 
 static void
-evas_engine_sdl16_image_border_get(void *data, void *image, int *l, int *r, int *t, int *b)
+evas_engine_sdl16_image_border_get(void *data __UNUSED__, void *image __UNUSED__, int *l __UNUSED__, int *r __UNUSED__, int *t __UNUSED__, int *b __UNUSED__)
 {
    /* FIXME: need to know what evas expect from this call */
 }
 
 static void
-evas_engine_sdl16_image_draw(void *data, void *context, void *surface, void *image,
+evas_engine_sdl16_image_draw(void *data __UNUSED__, void *context, void *surface, void *image,
                              int src_region_x, int src_region_y, int src_region_w, int src_region_h,
                              int dst_region_x, int dst_region_y, int dst_region_w, int dst_region_h,
                              int smooth)
@@ -801,8 +790,6 @@ evas_engine_sdl16_image_draw(void *data, void *context, void *surface, void *ima
    int                           mustlock_im = 0;
    int                           mustlock_dst = 0;
 
-   (void) data;
-
    evas_cache_engine_image_load_data(&eim->cache_entry);
 
    /* Fallback to software method */
@@ -834,6 +821,18 @@ evas_engine_sdl16_image_draw(void *data, void *context, void *surface, void *ima
      SDL_UnlockSurface(dst->surface);
 }
 
+static void
+evas_engine_sdl16_image_scale_hint_set(void *data __UNUSED__, void *image, int hint)
+{
+}
+
+static int
+evas_engine_sdl16_image_scale_hint_get(void *data __UNUSED__, void *image)
+{
+   return EVAS_IMAGE_SCALE_HINT_NONE;
+}
+
+
 static void
 evas_engine_sdl16_image_cache_flush(void *data)
 {
@@ -862,24 +861,20 @@ evas_engine_sdl16_image_cache_get(void *data)
 }
 
 static char*
-evas_engine_sdl16_image_comment_get(void *data, void *image, char *key)
+evas_engine_sdl16_image_comment_get(void *data __UNUSED__, void *image __UNUSED__, char *key __UNUSED__)
 {
-   (void) data;
-   (void) image;
-   (void) key;
-
    return NULL;
 }
 
 static char*
-evas_engine_sdl16_image_format_get(void *data, void *image)
+evas_engine_sdl16_image_format_get(void *data __UNUSED__, void *image __UNUSED__)
 {
    /* FIXME: need to know what evas expect from this call */
    return NULL;
 }
 
 static void
-evas_engine_sdl16_font_draw(void *data, void *context, void *surface, void *font, int x, int y, int w, int h, int ow, int oh, const char *text)
+evas_engine_sdl16_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const char *text)
 {
    static RGBA_Image            *im = NULL;
    SDL_Engine_Image_Entry       *eim = surface;
@@ -913,7 +908,7 @@ evas_engine_sdl16_font_draw(void *data, void *context, void *surface, void *font
 }
 
 static void
-evas_engine_sdl16_line_draw(void *data, void *context, void *surface, int x1, int y1, int x2, int y2)
+evas_engine_sdl16_line_draw(void *data __UNUSED__, void *context, void *surface, int x1, int y1, int x2, int y2)
 {
    SDL_Engine_Image_Entry       *eim = surface;
    int                           mustlock_im = 0;
@@ -934,7 +929,7 @@ evas_engine_sdl16_line_draw(void *data, void *context, void *surface, int x1, in
 }
 
 static void
-evas_engine_sdl16_rectangle_draw(void *data, void *context, void *surface, int x, int y, int w, int h)
+evas_engine_sdl16_rectangle_draw(void *data __UNUSED__, void *context, void *surface, int x, int y, int w, int h)
 {
    SDL_Engine_Image_Entry       *eim = surface;
 #if ENGINE_SDL_PRIMITIVE
@@ -995,7 +990,7 @@ evas_engine_sdl16_rectangle_draw(void *data, void *context, void *surface, int x
 }
 
 static void
-evas_engine_sdl16_polygon_draw(void *data, void *context, void *surface, void *polygon)
+evas_engine_sdl16_polygon_draw(void *data __UNUSED__, void *context, void *surface, void *polygon)
 {
    SDL_Engine_Image_Entry       *eim = surface;
    int                           mustlock_im = 0;
@@ -1014,7 +1009,7 @@ evas_engine_sdl16_polygon_draw(void *data, void *context, void *surface, void *p
 }
 
 static void
-evas_engine_sdl16_image_stride_get(void *data, void *image, int *stride)
+evas_engine_sdl16_image_stride_get(void *data __UNUSED__, void *image, int *stride)
 {
    SDL_Engine_Image_Entry       *eim = image;
 
@@ -1075,6 +1070,10 @@ module_open(Evas_Module *em)
    ORD(line_draw);
    ORD(rectangle_draw);
    ORD(polygon_draw);
+   
+   ORD(image_scale_hint_set);
+   ORD(image_scale_hint_get);
+   
    /* now advertise out own api */
    em->functions = (void *)(&func);
    return 1;
@@ -1110,7 +1109,7 @@ _sdl16_image_delete(Engine_Image_Entry *eim)
 }
 
 static int
-_sdl16_image_constructor(Engine_Image_Entry *ie, void* data)
+_sdl16_image_constructor(Engine_Image_Entry *ie, void* data __UNUSED__)
 {
    SDL_Surface                  *sdl = NULL;
    SDL_Engine_Image_Entry       *eim = (SDL_Engine_Image_Entry *) ie;
@@ -1157,7 +1156,7 @@ _sdl16_image_dirty_region(Engine_Image_Entry *eim, int x, int y, int w, int h)
 }
 
 static int
-_sdl16_image_dirty(Engine_Image_Entry *dst, const Engine_Image_Entry *src)
+_sdl16_image_dirty(Engine_Image_Entry *dst, const Engine_Image_Entry *src __UNUSED__)
 {
    SDL_Engine_Image_Entry       *eim = (SDL_Engine_Image_Entry *) dst;
    SDL_Surface                  *sdl = NULL;
@@ -1177,7 +1176,7 @@ _sdl16_image_dirty(Engine_Image_Entry *dst, const Engine_Image_Entry *src)
 }
 
 static int
-_sdl16_image_size_set(Engine_Image_Entry *dst, const Engine_Image_Entry *src)
+_sdl16_image_size_set(Engine_Image_Entry *dst, const Engine_Image_Entry *src __UNUSED__)
 {
    SDL_Engine_Image_Entry       *eim = (SDL_Engine_Image_Entry *) dst;
    SDL_Surface                  *sdl;
