@@ -30,7 +30,7 @@ struct _Render_Engine
 
 static void *eng_info(Evas *e);
 static void eng_info_free(Evas *e, void *info);
-static void eng_setup(Evas *e, void *info);
+static int eng_setup(Evas *e, void *info);
 static void eng_output_free(void *data);
 static void eng_output_resize(void *data, int w, int h);
 static void eng_output_tile_size_set(void *data, int w, int h);
@@ -56,7 +56,7 @@ eng_info(Evas *e)
 }
 
 static void
-eng_info_free(Evas *e, void *info)
+eng_info_free(Evas *e __UNUSED__, void *info)
 {
    Evas_Engine_Info_Software_16_X11 *in;
 
@@ -70,7 +70,7 @@ _tmp_out_alloc(Render_Engine *re)
    Tilebuf_Rect *r;
    int w = 0, h = 0;
 
-   for (r = re->rects; r; r = (Tilebuf_Rect *)(r->_list_data.next))
+   EINA_INLIST_FOREACH(re->rects, r)
      {
 	if (r->w > w) w = r->w;
 	if (r->h > h) h = r->h;
@@ -98,7 +98,7 @@ _tmp_out_alloc(Render_Engine *re)
 }
 
 
-static void
+static int
 eng_setup(Evas *e, void *in)
 {
    Render_Engine *re;
@@ -136,6 +136,8 @@ eng_setup(Evas *e, void *in)
 
 	/* render engine specific data */
 	re = calloc(1, sizeof(Render_Engine));
+        if (!re)
+          return 0;
 	e->engine.data.output = re;
 	re->disp = info->info.display;
 	re->draw = info->info.drawable;
@@ -172,13 +174,15 @@ eng_setup(Evas *e, void *in)
 	     re->tmp_out = NULL;
 	  }
      }
-   if (!e->engine.data.output) return;
+   if (!e->engine.data.output) return 0;
    /* add a draw context if we dont have one */
    if (!e->engine.data.context)
      e->engine.data.context =
      e->engine.func->context_new(e->engine.data.output);
    /* check if the display can do shm */
    re->shm = evas_software_x11_x_can_do_shm(re->disp);
+
+   return 1;
 }
 
 static void
@@ -321,7 +325,7 @@ eng_output_redraws_next_update_get(void *data, int *x, int *y, int *w, int *h, i
      }
    rect = re->cur_rect;
    ux = rect->x; uy = rect->y; uw = rect->w; uh = rect->h;
-   re->cur_rect = (Tilebuf_Rect *)(re->cur_rect->_list_data.next);
+   re->cur_rect = (Tilebuf_Rect *)((EINA_INLIST_GET(re->cur_rect))->next);
    if (!re->cur_rect)
      {
 	evas_common_tilebuf_free_render_rects(re->rects);
@@ -436,8 +440,6 @@ static void
 _tmp_out_process(Render_Engine *re, int out_x, int out_y, int w, int h)
 {
    Soft16_Image *d, *s;
-   DATA16 *dp, *sp;
-   int y, x, d_dir;
 
    d = re->shbuf->im;
    s = re->tmp_out;
@@ -454,7 +456,7 @@ _tmp_out_process(Render_Engine *re, int out_x, int out_y, int w, int h)
 }
 
 static void
-eng_output_redraws_next_update_push(void *data, void *surface, int x, int y, int w, int h)
+eng_output_redraws_next_update_push(void *data, void *surface __UNUSED__, int x, int y, int w, int h)
 {
    Render_Engine *re;
    XRectangle r;
@@ -502,7 +504,6 @@ static void
 eng_output_flush(void *data)
 {
    Render_Engine *re;
-   int w, h;
 
    re = (Render_Engine *)data;
    if (re->clip_rects)
